
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CF_Buffer
 *
 * DESCRIPTION:
 *	This unit contains all functions needed to control the buffer
 *	scheme used by the communication services.
 *
 * CONTENTS:
 *	CsCf_BuffInit
 *	CsCf_BuffTerm
 *	CsCf_BuffAlloc
 *	CsCf_BuffFree
 *	CsCf_BuffDebug
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>
#include <ir_net.h>
#include <resclist.h>


#define NUM_INIT_BUFS		10
#define NUM_GROW_BUFS		5

static IR_LIST CSCF_BuffList;
static void expandList();
static void CsCf_AddNewBuff();

/****************************************************************************
 * FUNCTION: CsCf_BuffInit
 *
 * PURPOSE:
 *	Initialize the buffers for the communication services.
 *
 * INPUT:
 *	none;
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *
 ***************************************************************************/

void CsCf_BuffInit()
{
	USHORT		i;

	RESC_ASN_NO_MALLOC(&CSCF_BuffList);
	RESC_LIST_INIT(&CSCF_BuffList);


	for (i = 0; i < NUM_INIT_BUFS; i++)
	{
		CsCf_AddNewBuff();
	}
}


/****************************************************************************
 * FUNCTION: CsCf_BuffTerm
 *
 * PURPOSE:
 *	Destroy the buffers for the communication services.
 *
 * INPUT:
 *	none;
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *
 ***************************************************************************/

void CsCf_BuffTerm()
{
	SHORT		i;
	CS_BUFFER	*buf;

	while (RESC_LIST_SIZE(&CSCF_BuffList))
	{
		RESC_LIST_SET_FIRST(&CSCF_BuffList);
		buf = RESC_LIST_GET(&CSCF_BuffList);
		RescIR_Free(IR_FILE, (CHAR *)buf->buf);
		if (RESC_ASN_IS_MALLOCED(buf))
		{
			RescIR_Free(IR_FILE, (CHAR *)buf);
		}
		RescList_Delete(IR_FILE, &CSCF_BuffList, 0);
	}
}

/****************************************************************************
 * FUNCTION:  CsCf_BuffAlloc
 *
 * PURPOSE:
 *	Alloc a new buffer from the pool. If the pool is empty then
 *	add a new block of buffers to the pool.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	New buffer pointer.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

CS_BUFFER *CsCf_BuffAlloc()
{
	USHORT		i;
	CS_BUFFER	*buf;


	if (RESC_LIST_SIZE(&CSCF_BuffList) == 0)
	{
		expandList();
	}
	RESC_LIST_SET_FIRST(&CSCF_BuffList);
	buf = RESC_LIST_GET(&CSCF_BuffList);
	RescList_Delete(IR_FILE, &CSCF_BuffList, 0);
	return (buf);
}

/****************************************************************************
 * FUNCTION:  CsCf_BuffFree
 *
 * PURPOSE:
 *	Return a used buffer to the pool. If the pool is too large
 *	then shrink it.
 *
 * INPUT:
 *	pointer to used buffer.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void CsCf_BuffFree( CS_BUFFER *buf)
{
	USHORT		i;


	buf->bufSize = 0;
	buf->numSent = 0;
	buf->bufState = CS_BUF_ST_EMPTY;
	buf->sizeOfAPDU = 0;
	RescList_Append(IR_FILE, &CSCF_BuffList, buf, NULL, NULL);
}

/****************************************************************************
 * LOCAL FUNCTION:  expandList
 *
 * PURPOSE:
 *	This function mallocs more buffers from the system memory and adds
 *	them to the free buffer list.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	none.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static void expandList()
{
	USHORT		i;

	for (i = 0; i < NUM_GROW_BUFS; i++)
	{
		CsCf_AddNewBuff();
	}
}

/****************************************************************************
 * LOCAL FUNCTION:  void CsCf_AddNewBuff
 *
 * PURPOSE:
 *	This function mallocs the new buffer, initializes it and adds it
 *	to the free list.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	none.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static void CsCf_AddNewBuff()
{

	CS_BUFFER	*buf;

	buf = (CS_BUFFER *)RescIR_Zmalloc(IR_FILE, sizeof(CS_BUFFER));
	RESC_ASN_MALLOCED(buf);
	buf->buf = (CHAR *)RescIR_Malloc(IR_FILE, CS_MAX_RD_BUF_SIZE);
	buf->bufMallocSize = CS_MAX_RD_BUF_SIZE;
	buf->bufSize = 0;
	buf->numSent = 0;
	buf->bufState = CS_BUF_ST_EMPTY;
	buf->sizeOfAPDU = 0;
	RescList_Append(IR_FILE, &CSCF_BuffList, buf, NULL, NULL);
}
