
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC  Communication Event
 *
 * DESCRIPTION:
 *	This interface unit is used to initiate communication service
 *	event processing. This unit supports both polled and event driven
 *	processing of communiction events.
 *
 * CONTENTS:
 *	IRC_CommEvent
 *	pollAllAssocs
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <resc.h>
#include <ir_cs.h>

/*
 * Local function prototypes.
 */
void pollAllAssocs(IRC_COMM_EVENT_REPLY *reply);

/****************************************************************************
 * FUNCTION: IRC_CommEvent
 *
 * PURPOSE:
 *
 * INPUT:
 *
 *	There is no input required to this function.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *	It is the responsibility of the calling function to generate
 *	the poll event themselves whenever this layer is running in
 *	polled mode.
 *
 ***************************************************************************/

IR_STATUS	IRC_CommEvent( IRC_COMM_EVENT_REQ *commReq,
		    IRC_COMM_EVENT_REPLY *commRep)
{
	IR_ASSOC_ID	assocId;
	IR_COMM_EVENT	*commEvent;
	CS_SOCK_ITEM	*sockItem;
	IR_STATUS	retStatus;
	int		errCode;

	if (!CsCf_IsIRCInitialized())
	{
		commRep->assocId = commReq->assocId;
		commRep->replyType = IR_COMM_DATA_ERROR;
		CS_LOG_PRINTV(IR_FILE, "CS event reply = %d\n", 
		    commRep->replyType);
		return(IR_FAIL);
	}

#ifdef WINSOCKET
	/* Check for an event to be current blocking */
	if (WSAIsBlocking())
	{
		CS_LOG_PRINTS(IR_FILE, 
		    "Windows socket is currently blocking\n");
	}
#endif

	/* Copy the event to a local pointer */
	commEvent = (IR_COMM_EVENT *)commReq->event;

	if (commEvent->event == (CS_SOCK_EVENT)CS_SOCK_EV_POLL)
	{
		/*
		 * Force the association Identifier to unknown.
		 * We will poll all known associations until one
		 * is found that must return data.
		 */
		assocId = IR_UNKNOWN_ID_TYPE;
	}
	else
	{
		/* Check for association to be known by CS */
		assocId = commReq->assocId;
		sockItem = CsCf_AssocGet(assocId);
		if (!sockItem)
		{
			/* 
			 * We don't recognize it. No error this is probably
			 * a left over event so just ignore it.
			 */
			CS_LOG_PRINTV(IR_FILE, "No Assoc for event %d. Event ignored\n",
			    commEvent->event);
			commRep->replyType == IR_COMM_DATA_NO_DATA;
			commRep->assocId = assocId;
			return(IR_SUCCESS);
		}
	}

	/* Default return values */
	commRep->replyType = IR_COMM_DATA_NO_DATA;
	commRep->assocId = commEvent->assocId;

	/* Process the events */
	switch(commEvent->event)
	{
	case CS_SOCK_EV_NONE:
	case CS_SOCK_EV_RECV:

		CS_LOG_PRINTV(IR_FILE, "Received no action event %d\n",
		    commEvent->event);
		/* No action for these events. They should never occur. */
		break;

		
	case CS_SOCK_EV_RWC:
#ifdef WINSOCKET
		commRep->assocId = commEvent->assocId;
		switch (WSAGETSELECTEVENT(commEvent->lParam))
		{
		case FD_READ:
			CS_LOG_PRINTS(IR_FILE, "Read from socket\n");
			commRep->replyType = 
			    CsCf_Poll(commEvent->assocId, CS_SOCK_EV_RECV);
			break;
		case FD_WRITE:
			CS_LOG_PRINTS(IR_FILE, "Write to socket\n");
			commRep->replyType = 
			    CsCf_Poll(commEvent->assocId, CS_SOCK_EV_SEND);
			break;
		case FD_CLOSE:
			CS_LOG_PRINTS(IR_FILE, "lost socket connection\n");
			CsCf_CloseSocket(commEvent->assocId);
			commRep->replyType = IR_COMM_DATA_A_P_ABORT;
			break;
		default:
			CS_LOG_PRINTS(IR_FILE, "Default OPER event\n");
			commRep->replyType = 
			    CsCf_Poll(commEvent->assocId, CS_SOCK_EV_POLL);
			break;
		}

		/* Check for abort */
		if (commRep->replyType == IR_COMM_DATA_A_P_ABORT)
		{
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(commEvent->assocId, CS_SOCK_EV_LOST_CONNECT);
		}
		else if (commRep->replyType == IR_COMM_DATA_APDU)
		{
			/* Otherwise Check for complete APDU and decode it */
			CsCf_RecvAPDU(commEvent->assocId, commRep);
		}

#endif
		break;

	case CS_SOCK_EV_RESOLVED:
		/* Reset the handle. It is nolonger required */
		sockItem->handle = IR_UNKNOWN_ID_TYPE;
		/*
		 * Make connection.
		 */
		CsCf_Connect(commEvent);
		break;
	
	case CS_SOCK_EV_CONNECTED:
		/* We are connected */
		CS_LOG_PRINTS(IR_FILE, "Connected\n");
		sockItem->state = CS_SOCK_ST_OPER;
		commRep->replyType = IR_COMM_DATA_CONNECT;
		commRep->assocId = commEvent->assocId;

#ifdef WINSOCKET
		if (WSAGETSELECTERROR(commEvent->lParam) 
		    == WSAECONNREFUSED)
		{
			/* Connection to server refused */
			CS_LOG_PRINTS(IR_FILE, 
			    "Connection to server refused");
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(commEvent->assocId, 
			    CS_SOCK_EV_CONN_REFUSED);
		}
		else if (WSAGETSELECTERROR(commEvent->lParam) != 0)
		{
			/* Connection to server error */
			CS_LOG_PRINTV(IR_FILE, 
			    "Connection to server error = %d\n",
			    WSAGETSELECTERROR(commEvent->lParam));
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(commEvent->assocId, 
			    CS_SOCK_EV_CONN_REFUSED);
		}
		else
		{
			/* Prime the port for read/close events */
			if (WSAAsyncSelect((SOCKET)sockItem->socketFd,
			    (HWND)sockItem->assocId,
			    (unsigned int)CS_SOCK_EV_RWC,
			    (long)(FD_READ|FD_CLOSE)) != 0)
			{
				errCode = WSAGetLastError();
				CS_LOG_PRINTV(IR_FILE, 
				   "WSAAsynSelect Error. Error = %d\n",
				   errCode);
				commRep->replyType = IR_COMM_DATA_NO_DATA;
				sockItem->state = CS_SOCK_ST_ABORT;
				CsCf_Notify(commEvent->assocId, 
				    CS_SOCK_EV_ERROR_ABORT);
			}
		}
#endif
		break;

	case CS_SOCK_EV_CONN_REFUSED:
		/* Connection refused by server */
		CS_LOG_PRINTS(IR_FILE, "Connection refused\n");
		commRep->replyType = IR_COMM_DATA_CONN_REFUSED;
		commRep->assocId = commEvent->assocId;
		CsCf_CloseSocket(commEvent->assocId);
		break;

	case CS_SOCK_EV_POLL:
		/*
		 * Poll all known associations.
		 */
		pollAllAssocs(commRep);

		break;

	case CS_SOCK_EV_SEND:
		/* Complete sending the pdata */
		commRep->replyType = CsCf_Poll(commEvent->assocId, 
		    commEvent->event);
		commRep->assocId = commEvent->assocId;
		break;

	case CS_SOCK_EV_USER_RELEASE:
	case CS_SOCK_EV_USER_ABORT:
	case CS_SOCK_EV_ERROR_ABORT:
	case CS_SOCK_EV_LOST_CONNECT:
	case CS_SOCK_EV_FAIL_RESOLVE:

		/* 
		 * Delete the association information entry in the
		 * association list. 
		 */
		commRep->assocId = assocId;
		CsCf_CloseSocket(assocId);

		switch(commEvent->event)
		{
		case CS_SOCK_EV_USER_RELEASE:
			commRep->replyType = IR_COMM_DATA_RELEASE;
			break;
		case CS_SOCK_EV_USER_ABORT:
			commRep->replyType = IR_COMM_DATA_A_ABORT;
			break;
		case CS_SOCK_EV_ERROR_ABORT:
			commRep->replyType = IR_COMM_DATA_ERROR;
			break;
		case CS_SOCK_EV_LOST_CONNECT:
			commRep->replyType = IR_COMM_DATA_A_P_ABORT;
			break;
		case CS_SOCK_EV_FAIL_RESOLVE:
			commRep->replyType = IR_COMM_DATA_HOST_NOT_FOUND;
			break;

		default:
			break;
		}
		break;
			
	default:
		break;
	}

	if (commRep->replyType != IR_COMM_DATA_NO_DATA)
	{
		CS_LOG_PRINTV(IR_FILE, 
		    "CS event reply = %d\n", commRep->replyType);
	}
	if (commRep->replyType == IR_COMM_DATA_APDU)
	{
		/* Display complete apdu for debug */
		CS_LOG_TRACES(IR_FILE, "Done IRC_CommEvent\n");
		CsCf_DisplayAPDU(commRep->apdu);
	}
	return(IR_SUCCESS);
}

/****************************************************************************
 * LOCAL FUNCTION: pollAllAssocs
 *
 * PURPOSE:
 *	This function will perform a poll of all associations until one
 *	association has returned data or until all active associations
 *	have been polled at least once.
 *
 *	This function will perform a round robin poll of all active 
 *	associations starting after the one that it last received 
 *	data on.
 *
 * INPUT:
 *
 *	There is no input required to this function.
 *
 * OUTPUT:
 *	This function will return an IR_COMM_EVENT_STATUS.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

void pollAllAssocs(IRC_COMM_EVENT_REPLY *reply)
{
	static IR_ASSOC_ID	lastAssocId=IR_UNKNOWN_ID_TYPE;
	BOOLEAN			doneFlag;
	IR_ASSOC_ID		assocId;

	/* Default return status. */
	reply->replyType = IR_COMM_DATA_NO_DATA;

	if (lastAssocId == IR_UNKNOWN_ID_TYPE)
	{
		/* 
		 * This is the first time into this function.
		 * Begin a round robin poll of all associations.
		 */
		assocId = CsCf_AssocGetFirst();
		lastAssocId = assocId;
	}
	else
	{
		/* Get the next association Id from the list. */
		assocId = CsCf_AssocGetNext(lastAssocId);
		lastAssocId = assocId;

	}

	/* Check for hanging events. ie events after release or abort */
	if (assocId == IR_UNKNOWN_ID_TYPE)
	{
		/* Flush the unnecessary events by ignoring them */
		reply->assocId = assocId;
		reply->replyType = IR_COMM_DATA_NO_DATA;
		CS_LOG_PRINTS(IR_FILE, "Poll abort. No targetIds\n");
		return;
	}

	doneFlag = FALSE;

	while (doneFlag == FALSE)
	{
		/* Check the first assocId */
		reply->assocId = assocId;
		reply->replyType = CsCf_Poll(assocId, CS_SOCK_EV_POLL);

		switch(reply->replyType)
		{
		case IR_COMM_DATA_APDU:
			/* Check for a complete APDU */
			if (CsCf_RecvAPDU(assocId, reply) == IR_SUCCESS)
			{
				doneFlag = TRUE;
			}
			/* 
			 * otherwise, there was not enough data read for
			 * an entire APDU so we continue searching.
 			 * The data will be queued until such time when
			 * an APDU can be generated.
			 */
			break;

		case IR_COMM_DATA_NO_DATA:
			/* Continue onto next one */
			break;
		default:
			/* Target connection lost or something like that */
			doneFlag = TRUE;
			break;
		}

		/* Get the next association. */
		assocId = CsCf_AssocGetNext(assocId);

		/* Check for completion of round robin play */
		if ((assocId == IR_UNKNOWN_ID_TYPE) ||
		    (assocId == lastAssocId))
		{
			doneFlag = TRUE;
		}
	}
}
