
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/***************************************************************************
 * UNIT: control.c
 *
 * DESCRIPTION: This unit contains all of the functionality for the
 *	Origin Kernel Control Function. 
 *
 * CONTENTS: IRK_CommEvent
 *	     IRK_Initialize
 *	     IRK_Terminate
 *	     IRK_ErrorReport	      
 *	     IRK_UpdateErrorReport
 * 	     OkCf_UpdateErrorReport 
 *	     OkCf_IsIRKInitialized
 *
 * NOTES: 
 *	Nothing of note.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ir.h>
#include <ir_ali.h>
#include <ir_pli.h>
#include <resc.h>
#include <ok/ok.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OKCF control.c"

static IRK_ERROR_REPLY currentError;
static BOOLEAN		irkInitialized=FALSE;

/****************************************************************************
 * FUNCTION: IRK_CommEvent
 *
 * PURPOSE: To service communication events.
 *
 * INPUT: 
 *	IR_ASSOC_ID		assocId
 *	IRK_COMM_EVENT_REQ	*commRequest
 *	IRK_COMM_EVENT_REPLY	*commReply
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRK_CommEvent(
	IRK_COMM_EVENT_REQ	*commRequest,
	IRK_COMM_EVENT_REPLY	*commReply)
{
	IR_STATUS 	status;
	IR_ASSOC_ID	assocId;
	IRC_COMM_EVENT_REQ ircReq;
	IRC_COMM_EVENT_REPLY ircReply;

	if (!OkCf_IsIRKInitialized())
	{
		CS_LOG_PRINTS(IR_FILE, "IRK interface not initialized\n");
		return (IR_FAIL);
	}


	/* Invoke the PLI primitive to service the event. */
	(void)memset((char *)&ircReply, (CHAR)NULL, 
	    sizeof(IRC_COMM_EVENT_REPLY));
	ircReq.assocId = commRequest->assocId;
	ircReq.event = commRequest->event;
	status = IRC_CommEvent(&ircReq, &ircReply);

	if (status == IR_FAIL)
	{
		/* 
		 * This error should only occur if an event is serviced
 		 * prior to the comms layer being initialized.
		 */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_PLI_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	commReply->assocId = ircReply.assocId;
	assocId = ircReply.assocId;
	commReply->replyType = ircReply.replyType;

	if (ircReply.apdu != (APDU *)NULL)
	{
		commReply->apdu = ircReply.apdu;
	}

	/* Do same for target address but wait until it is changed from a
 	 * char * to an array large enough to hold the string. Roland.
	 */

	/* Determine event type. */
	switch (ircReply.replyType)
	{
	case IR_COMM_DATA_ERROR:
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_COMMS_ERROR,
		    IR_FILE);
		status = IR_FAIL;
		(void)OkAcse_AbortAssoc(assocId);
		/* Just a single abort type known to ALI users */
		commReply->replyType = IR_COMM_DATA_ABORT;
		break;
		
	case IR_COMM_DATA_CONN_REFUSED:
		status = OkAcse_AbortAssoc(assocId);
		break;
	case IR_COMM_DATA_HOST_NOT_FOUND:
	case IR_COMM_DATA_UNKNOWN_HOST:
		OkAcse_StopConnectWait(assocId);
		break;
	case IR_COMM_DATA_NO_DATA: 
		status = IR_SUCCESS;
		break;
	case IR_COMM_DATA_CONNECT: 
		status = OkAcse_Association(assocId);
		break;
	case IR_COMM_DATA_RELEASE: 
                status = OkAcse_ReleaseAssoc(assocId);
		break;
	case IR_COMM_DATA_A_ABORT: 
	case IR_COMM_DATA_A_P_ABORT: 
		status = OkAcse_AbortAssoc(assocId);
		/* Just a single abort type known to ALI users */
		commReply->replyType = IR_COMM_DATA_ABORT;
		break;
	case IR_COMM_DATA_APDU:
		status = OkAse_ReceiveResponse(assocId, commReply->apdu);
		if (status != IR_SUCCESS)
		{
			CS_LOG_PRINTV(IR_FILE, "Invalid apdu for state for assoc %d\n", assocId);
			FreeAPDU(commReply->apdu);
			commReply->replyType = IR_COMM_DATA_PROTOCOL_ERR;
			OkAcse_AAbort(assocId);
		}
		else
		{
			switch(commReply->apdu->choiceId)
			{
        		case APDU_INITRESP:
				commReply->replyType =
				    IR_COMM_DATA_INIT_SRVC;
				break;
        		case APDU_SEARCHRESP:
				commReply->replyType =
				    IR_COMM_DATA_SEARCH_SRVC;
				break;
        		case APDU_PRESENTRESP:
				commReply->replyType =
				    IR_COMM_DATA_PRESENT_SRVC;
				break;
        		case APDU_DELETERESULTSETRESP:
				commReply->replyType =
				    IR_COMM_DATA_DELETE_SRVC;
				break;
        		case APDU_ACCESSCONTROLREQ:
				commReply->replyType =
				    IR_COMM_DATA_ACCS_SRVC;
				break;
        		case APDU_RESOURCECONTROLREQ:
				commReply->replyType =
				    IR_COMM_DATA_RSRC_CNTL_SRVC;
				break;
        		case APDU_RESOURCEREPORTRESP:
				commReply->replyType =
				    IR_COMM_DATA_RSRC_RPT_SRVC;
				break;

			default:
				OkCf_UpdateErrorReport(IR_ERROR_WARNING,
				    IRK_PROTOCOL_ERROR, IR_FILE);
				FreeAPDU(commReply->apdu);
				commReply->replyType = 
				    IR_COMM_DATA_PROTOCOL_ERR;
			}
		}
		break;
	default: 
		status = OkAcse_AAbort(assocId);
		commReply->replyType = IR_COMM_DATA_ABORT;
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_PROTOCOL_ERROR,
		    IR_FILE);
		CS_LOG_PRINTV(IR_FILE, "Unknown comm data type returned from IRC_CommEvent. %d\n",
		    ircReply.replyType);
	}	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: IRK_Initialize 
 *
 * PURPOSE: Configure communication services.
 *
 * INPUT: 
	IRK_INIT_REQ *initReq;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRK_Initialize(
	IRK_INIT_REQ *initReq)
{
	IR_STATUS	status;
	IRC_INIT_REQ	ircInit;

	if (irkInitialized)
	{
		return(IR_SUCCESS);
	}

	currentError.errorLevel = IR_ERROR_NO_ERROR;
	currentError.errorCode = IRK_NO_ERROR;
	strcpy(currentError.errorSource, "");

	/* Invoke PLI primitive to configure communication services. */
	ircInit.commConfig = initReq->commConfig;
	ircInit.debugLog = initReq->debugLog;
	ircInit.logAPDUs = initReq->logAPDUs;
	status = IRC_Initialize(&ircInit);
	if (status == IR_FAIL)
	{
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_PLI_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	irkInitialized = TRUE;
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: IRK_Terminate 
 *
 * PURPOSE: Terminate communication services.
 *
 * INPUT: nothing 
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS IRK_Terminate()
{
	IR_STATUS	status;

	/* Invoke PLI primitive to terminate communication services. */
	status = IRC_Terminate();
	if (status == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error terminating PLI\n");
	}

	/* Terminate remaining associations. */
	if (OkAcse_Terminate() == IR_FAIL)
	{
		status = IR_FAIL;
	}

	irkInitialized = FALSE;
	return (status);
}

/****************************************************************************
 * FUNCTION: IRK_ErrorReport 
 *
 * PURPOSE: Return report of last error.
 *
 * INPUT: 
	IRK_ERROR_REPLY *error;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRK_ErrorReport(
	IRK_ERROR_REPLY *error)
{
	if (!OkCf_IsIRKInitialized())
	{
		return(IR_FAIL);
	}
	error->errorLevel = currentError.errorLevel;
	error->errorCode = currentError.errorCode;
	strcpy(error->errorSource, currentError.errorSource);

	return (IR_SUCCESS);
}
/****************************************************************************
 * FUNCTION: OkCf_UpdateErrorReport 
 *
 * PURPOSE: Update report to reflect the latest error.
 *
 * INPUT: 
 *	IR_ERROR_LEVEL level
 *	IR_ERROR_CODE  code
 *	CHAR	       *source
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkCf_UpdateErrorReport(
	IR_ERROR_LEVEL level,
	IR_ERROR_CODE  code,
	CHAR	       *source)
{
	CHAR	buf[100];
	currentError.errorLevel = level;
	currentError.errorCode = code;
	if (source != NULL)
	{
		strcpy(buf, source);
		CS_LOG_PRINTV(IR_FILE, "Error detected by %s\n", buf);
		strncpy(currentError.errorSource, source, IRK_MAX_ERROR_SOURCE);
		currentError.errorSource[IRK_MAX_ERROR_SOURCE] = EOS;
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "Unknown Error detected. code %d\n", code);
		strcpy(currentError.errorSource, "");
	}
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkCf_IsIRKInitized
 *
 * PURPOSE: Verifies whether or not the IRK interface has been initialized.
 *
 * INPUT: 
 *	none.
 *
 * OUTPUT: BOOLEAN		- TRUE - IRK has been initialized.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
BOOLEAN	OkCf_IsIRKInitialized()
{
	return(irkInitialized);
}
