
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpAm_Association
 *
 * DESCRIPTION:
 *	This unit is used to perform all association related 
 *	IRO operations.
 *
 * CONTENTS:
 *	IRO_Association
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>

void OpAm_AssocReplyListItemFree(CHAR *srcFile, IRO_ASSOC_STRUCT *assocItem);
void OpAm_AssocReplyFree(IR_LIST *list);
void OpAm_AssocReplyFullFree(IRO_ASSOC_FULL_INFO_STRUCT *);
static void OpAm_AssocBriefItem(OP_ASSOC_ENTRY *, IRO_ASSOC_STRUCT *);

/****************************************************************************
 * FUNCTION:  IRO_Association
 *
 * PURPOSE:
 *	This function is used to service the API association interface
 *	function.
 *
 * INPUT:
 *	IRO_ASSOC_REQ_TYPE		- type of request
 *						Establish Association
 *						Terminate Association
 *						Abort Association
 *						Get Assoc Info
 *	IRO_ASSOC_REQ			- request info
 * OUTPUT:
 *	IR_STATUS			- status of operation
 *	IRO_ASSOC_REPLY			- reply info (Get request only)
 *
 * NOTES:
 *	The calling function should check the IRO_ErrorReport if the
 *	return status is IR_FAIL.
 *
 ***************************************************************************/

IR_STATUS	IRO_Association(IR_USER_ID userId,
		    IRO_ASSOC_REQ_TYPE reqType,
		    IRO_ASSOC_REQ *req, IRO_ASSOC_REPLY *reply)
{
	extern IR_LIST	*OpAm_AssocList;
	IR_STATUS 	 status;
	IRK_ASSOC_REQ    assocReq;
	IRK_ASSOC_REPLY  assocReply;
	OP_ASSOC_ENTRY	 *assocEntry;
	IR_TARGET_ASSOC_INFO infoStruct;
	SHORT		count;
	IR_LIST		*assocList;
	IRO_ASSOC_STRUCT *assocItem;
	IRO_ASSOC_FULL_INFO_STRUCT *fullInfo;
	OP_SESSION_INFO	*sessionInfo;

	RESC_ASN_NO_MALLOC(&infoStruct);

	if (!OpCf_IsOpInitialized())
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_PROTOCOL_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	CS_LOG_PRINTV(IR_FILE, "Association RequestType %d\n", reqType);

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_UNKNOWN_USER_ID,
		    IR_FILE);
		return(IR_FAIL);
	}

	   
	switch (reqType)
	{
	case IRO_ASSOC_ESTABLISH:
		assocReq.assocId = req->assocId;
		RESC_ASN_NO_MALLOC(&assocReq.hostName);
		RESC_ASN_INIT_VS(&assocReq.hostName);
		RESC_ASN_NO_MALLOC(&assocReq.hostAddr);
		RESC_ASN_INIT_VS(&assocReq.hostAddr);
		if (req->nameType == IRO_ASSOC_ADDR_TARGET_ID)
		{
		  	OpTm_TargetAssocInfo(req->targetId, &infoStruct);
			RescASN_CopyVS(IR_FILE, &assocReq.hostName,
			    &infoStruct.ipDomainName);
			RescASN_CopyVS(IR_FILE, &assocReq.hostAddr,
			    &infoStruct.ipAddress);
			assocReq.portNum = infoStruct.portNum;
		}
		else
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
			    IRO_UNKNOWN_ADDR_TARGET_TYPE,
		    	    IR_FILE);
			return(IR_FAIL);
		}

		/* Perform connection */
		status = IRK_Association(&assocReq, &assocReply);
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL,
			     IRO_ALI_ERROR, IR_FILE);
			/* 
		 	 * Destroy the host name and host address fields from
		 	 * the IRK request.
		 	 */
			RescASN_DestroyVS(IR_FILE, &assocReq.hostName);
			RescASN_DestroyVS(IR_FILE, &assocReq.hostAddr);
			OpTm_TargetAssocInfoFree(&infoStruct);
			return (IR_FAIL);
		}

		/* Create a new entry in the assoc list */
		assocEntry = OpAm_AssocCreate(req->assocId);
		if (assocEntry == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL,
			     IRO_MEMORY_ERROR, IR_FILE);
			return (IR_FAIL);
		}

		/* 
		 * Destroy the host name and host address fields from
		 * the IRK request.
		 */
		RescASN_DestroyVS(IR_FILE, &assocReq.hostName);
		RescASN_DestroyVS(IR_FILE, &assocReq.hostAddr);

		/* Update the new assoc entry with the target's data */
		assocEntry->targetId = req->targetId;

		/* Put the user's Id into the assoc table. */
		assocEntry->userId = sessionInfo->userId;


		RESC_ASN_NO_MALLOC(&assocEntry->address);
		RESC_ASN_NO_MALLOC(&assocEntry->hostName);

		/* Get target information from table management info */
		RescASN_CopyVS(IR_FILE, &assocEntry->address,
		    &infoStruct.ipAddress);

		RescASN_CopyVS(IR_FILE, &assocEntry->hostName,
		    &infoStruct.ipDomainName);

		RescASN_CopyVS(IR_FILE, &assocEntry->localName,
		    &infoStruct.ipDomainName);

		OpTm_TargetAssocInfoFree(&infoStruct);

		break;

	case IRO_ASSOC_TERMINATE:
		assocEntry = OpAm_AssocGet(req->assocId);
		if (assocEntry == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			     IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}

		if (assocEntry->state == (OP_ASSOC_STATE)OP_ASSOC_IDLE)
		{
			/* Release if idle. */
			status = IRK_SendRequest(IRK_RELEASE_REQ_TYPE,
			    req->assocId, 
			    (IRK_SEND_REQUEST_REQ *)NULL);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL,
				     IRO_ALI_ERROR, IR_FILE);
				return (IR_FAIL);
			}	
			else
			{
				assocEntry->state = OP_ASSOC_RELEASING;
			}
		}
		else if (!((assocEntry->state == 
		    (OP_ASSOC_STATE)OP_ASSOC_ABORTING) || 
		    (assocEntry->state == (OP_ASSOC_STATE)OP_ASSOC_RELEASING)))
		{
			/* Assoc is busy. Abort the connection instead */
			return(IRO_Association(userId,
			    (IRO_ASSOC_REQ_TYPE)IRO_ASSOC_ABORT,
		    	    req, reply));
		}
		break;

	case IRO_ASSOC_ABORT:
		assocEntry = OpAm_AssocGet(req->assocId);
		if (assocEntry == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			     IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}
		status = IRK_SendRequest(IRK_ABORT_REQ_TYPE,
			req->assocId,
			(IRK_SEND_REQUEST_REQ *)NULL);
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL,
			     IRO_ALI_ERROR, IR_FILE);
			return (IR_FAIL);
		}	
		assocEntry->state = OP_ASSOC_ABORTING;
		break;

	case IRO_ASSOC_GET_ASSOC_INFO:

		/* Create the reply list */
		assocList = RescList_Create(IR_FILE);

		if (assocList == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL,
			     IRO_MEMORY_ERROR, IR_FILE);
			return (IR_FAIL);
		}

		/* Copy the local list to the reply list */
		if (RESC_LIST_SIZE(OpAm_AssocList) > 0)
		{
			/* Reset the list current pointer to the start */
			RESC_LIST_SET_FIRST(OpAm_AssocList);

			/* Copy all assoc entries into the reply list */
			count = 0;
			while (count < RESC_LIST_SIZE(OpAm_AssocList))
			{
				/* Get the local list item */
				assocEntry = RESC_LIST_GET(OpAm_AssocList);

				/* Create the reply list item */
				assocItem = (IRO_ASSOC_STRUCT *)RescIR_Malloc(
	    			    IR_FILE, sizeof(IRO_ASSOC_STRUCT));

				if (assocItem == NULL)
				{
					 CS_LOG_PRINTS(IR_FILE, 
					    "Malloc error generating assocItem\n");
					RescList_Destroy(IR_FILE, assocList);
					return(IR_FAIL);
				}
				RESC_ASN_MALLOCED(assocItem);

				/* Create the reply list item */
				OpAm_AssocBriefItem(assocEntry, assocItem);

				RescList_Append(IR_FILE, assocList, assocItem,
				    OpAm_AssocReplyListItemFree, NULL);

				RESC_LIST_SET_NEXT(OpAm_AssocList);
				count++;
			}
		}

		reply->activeAssociations = assocList;
		OpCf_IROMark(OpAm_AssocReplyFree, assocList);
		break;

	case IRO_ASSOC_FULL_INFO:

		/* 
		 * This request is for a full report of association
		 * information. This contains both the brief info
		 * plus the additional init response info for a single
		 * association.
		 */
		assocEntry = OpAm_AssocGet(req->assocId);
		if (assocEntry == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			     IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}

		/* Create the reply list item */
		fullInfo = (IRO_ASSOC_FULL_INFO_STRUCT *)RescIR_Zmalloc(
		    IR_FILE, sizeof(IRO_ASSOC_FULL_INFO_STRUCT));

		RESC_ASN_MALLOCED(fullInfo);
		RESC_ASN_NO_MALLOC(&fullInfo->assocStruct);
		RESC_ASN_NO_MALLOC(&fullInfo->initResp);
		if (fullInfo == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "Could not malloc full info\n");
			return(IR_FAIL);
		}

		/* Get the brief information */
		OpAm_AssocBriefItem(assocEntry, &fullInfo->assocStruct);


		/* Get the full information */
		if (assocEntry->initResp)
		{
			OpAm_AssocInitInfo(assocEntry->initResp,
			    &fullInfo->initResp);
		}
		else
		{
			RESC_ASN_NO_MALLOC(&fullInfo->initResp.protocolVersion);
			RESC_ASN_INIT_VS(&fullInfo->initResp.protocolVersion);
			RESC_ASN_NO_MALLOC(&fullInfo->initResp.implementationId);
			RESC_ASN_INIT_VS(&fullInfo->initResp.implementationId);
			RESC_ASN_NO_MALLOC(&fullInfo->initResp.implementationName);
			RESC_ASN_INIT_VS(&fullInfo->initResp.implementationName);
			RESC_ASN_NO_MALLOC(&fullInfo->initResp.implementationVersion);
			RESC_ASN_INIT_VS(&fullInfo->initResp.implementationVersion);
			RESC_ASN_NO_MALLOC(&fullInfo->initResp.userInfo);
			RESC_ASN_INIT_OS(&fullInfo->initResp.userInfo);
		}

		reply->fullInfo = fullInfo;
		OpCf_IROMark(OpAm_AssocReplyFullFree, fullInfo);
		break;
	default:
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_PROTOCOL_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpAm_AssocReplyFree
 *
 * PURPOSE:
 *	This function is used to free all IRO_ASSOC_STRUCT entries from
 *	the previously generated list that was supplied to the user 
 *	application.
 *
 * INPUT:
 *	IRO_ASSOC_STRUCT		- pointer to list.
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpAm_AssocReplyFree(IR_LIST *list)
{

	if (list)
	{
		RescList_Destroy(IR_FILE, list);
	}
}

void OpAm_AssocReplyListItemFree(CHAR *srcFile, IRO_ASSOC_STRUCT *assocItem)
{
	RescASN_DestroyVS(IR_FILE, &assocItem->name);
	RescASN_DestroyVS(IR_FILE, &assocItem->address);
	RescASN_DestroyVS(IR_FILE, &assocItem->hostName);

	if (RESC_ASN_IS_MALLOCED(assocItem))
	{
		RescIR_Free(IR_FILE, (CHAR *)assocItem);
	}
}

void OpAm_AssocReplyFullFree(IRO_ASSOC_FULL_INFO_STRUCT *infoStruct)
{
	if (infoStruct)
	{
		OpAm_AssocReplyListItemFree(IR_FILE, &infoStruct->assocStruct);
		OpAm_InitServiceFree(&infoStruct->initResp);
	}

	if (RESC_ASN_IS_MALLOCED(infoStruct))
	{
		RescIR_Free(IR_FILE, (CHAR *)infoStruct);
	}
		
}

static void OpAm_AssocBriefItem(OP_ASSOC_ENTRY *assocEntry,
	    IRO_ASSOC_STRUCT *assocItem)
{
	assocItem->assocId = assocEntry->assocId;
	assocItem->targetId = assocEntry->targetId;

	RESC_ASN_NO_MALLOC(&assocItem->name);
	RescASN_CopyVS(IR_FILE, &assocItem->name, &assocEntry->localName);

	RESC_ASN_NO_MALLOC(&assocItem->address);
	RescASN_CopyVS(IR_FILE, &assocItem->address, &assocEntry->address);

	RESC_ASN_NO_MALLOC(&assocItem->hostName);
	RescASN_CopyVS(IR_FILE, &assocItem->hostName, &assocEntry->hostName);

	switch(assocEntry->state)
	{
	case OP_ASSOC_CONNECTING:
		assocItem->state = IRO_ASSOC_STATE_CONNECT;
		break;
	case OP_ASSOC_INITIALIZING:
		assocItem->state = IRO_ASSOC_STATE_INIT;
		break;
	case OP_ASSOC_IDLE:
		assocItem->state = IRO_ASSOC_STATE_IDLE;
		break;
	case OP_ASSOC_SEARCHING:
		assocItem->state = IRO_ASSOC_STATE_SEARCH;
		break;
	case OP_ASSOC_PRESENTING:
		assocItem->state = IRO_ASSOC_STATE_PRESENT;
		break;
	case OP_ASSOC_DELETING:
		assocItem->state = IRO_ASSOC_STATE_DELETE;
		break;
	case OP_ASSOC_RSRC_REPORT:
		assocItem->state = IRO_ASSOC_STATE_RSRC_RPRT;
		break;
	case OP_ASSOC_RELEASING:
	case OP_ASSOC_ABORTING:
		assocItem->state = IRO_ASSOC_STATE_ABORTING;
		break;
	default:
		break;
	}

}

/*
 * This function copies the initialize service info from one structure
 * to a copy of the same structure.
 */
void OpAm_AssocInitInfo(IRO_COMM_INIT_RESP *srcStruct, 
	IRO_COMM_INIT_RESP *destStruct)
{

	if (destStruct == NULL)
	{
		return;
	}
	destStruct->search = srcStruct->search;
	destStruct->present = srcStruct->present;
	destStruct->delSet = srcStruct->delSet;
	destStruct->resourceReport = srcStruct->resourceReport;
	destStruct->triggerResourceCtrl = srcStruct->triggerResourceCtrl;
	destStruct->resourceCtrl = srcStruct->resourceCtrl;
	destStruct->accessCtrl = srcStruct->accessCtrl;
	destStruct->preferredMessageSize = srcStruct->preferredMessageSize;
	destStruct->maximumRecordSize = srcStruct->maximumRecordSize;

	RESC_ASN_INIT_VS(&destStruct->protocolVersion);
	RESC_ASN_NO_MALLOC(&destStruct->protocolVersion);
	RescASN_CopyVS(IR_FILE, &destStruct->protocolVersion,
	    &srcStruct->protocolVersion);

	RESC_ASN_INIT_VS(&destStruct->implementationId);
	RESC_ASN_NO_MALLOC(&destStruct->implementationId);
	RescASN_CopyVS(IR_FILE, &destStruct->implementationId,
	    &srcStruct->implementationId);

	RESC_ASN_INIT_VS(&destStruct->implementationName);
	RESC_ASN_NO_MALLOC(&destStruct->implementationName);
	RescASN_CopyVS(IR_FILE, &destStruct->implementationName,
	    &srcStruct->implementationName);

	RESC_ASN_INIT_VS(&destStruct->implementationVersion);
	RESC_ASN_NO_MALLOC(&destStruct->implementationVersion);
	RescASN_CopyVS(IR_FILE, &destStruct->implementationVersion,
	    &srcStruct->implementationVersion);

	RESC_ASN_INIT_VS(&destStruct->userInfo);
	RESC_ASN_NO_MALLOC(&destStruct->userInfo);
	RescASN_CopyVS(IR_FILE, &destStruct->userInfo,
	    &srcStruct->userInfo);

}
