
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpAm_Events
 *
 * DESCRIPTION:
 *	This CSU contains functions which react to communication events
 *	that returned association related information.
 *
 * CONTENTS:
 *	OpAm_ConnectAssoc
 *	OpAm_AbortAssoc
 *	OpAm_ReleaseAssoc
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <ir.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>

/****************************************************************************
 * FUNCTION:  OpAm_ConnectAssoc
 *
 * PURPOSE:
 *	This function is invoke by the control function csc when a 
 *	connect event status is detected. This function will create
 *	the initialize apdu based on information maintained in the
 *	tm tables for the connected target. The wsRefId in the association
 *	entry table is used to coordinate the reply. All interim 
 *	access control, trigger, and resource control apdus must use
 *	this wsRefId while an init is in progress. The assocId will be
 *	used as the init's wsRefId. For subsequent search, present,
 *	and delete operations the related workspace id will be used.
 *
 * INPUT:
 *	ASSOC_ID	- association id
 *
 * OUTPUT:
 *	IR_STATUS		-
 *
 *
 * NOTES:
 * 	The User Application should use the IRO_ErrorReport function
 *	if an error status is returned.
 *
 ***************************************************************************/

IR_STATUS OpAm_ConnectAssoc(IR_ASSOC_ID assocId)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IR_STATUS	status;
	IRK_SEND_REQUEST_REQ	sendReq;
	OP_INIT_REQ_INFO	initReqInfo;
	IR_TARGET_INFO		*targetInfo;
	InitializeReq		*initReq;
	CHAR			tmpBuf[20];
	

	/* Verify association exists. */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* If not connecting then you don't know what you are doing. So Bye! */
	if (assocEntry->state != OP_ASSOC_CONNECTING)
	{
		IRK_SendRequest((IRK_REQ_SERVICE_TYPE)IRK_ABORT_REQ_TYPE, 
		    (IR_ASSOC_ID)assocId, 
		    (IRK_SEND_REQUEST_REQ *)NULL);	
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_PROTOCOL_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	/* Prepare init request apdu */
	sendReq.irApdu.choiceId = APDU_INITREQ;
	initReq = (InitializeReq *)RescIR_Zmalloc(
	    IR_FILE, sizeof(InitializeReq));
	sendReq.irApdu.a.initReq = initReq;
	if (initReq == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	/* Prepare the wsRefId */
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, (SHORT)assocId);
	RescASN_IRToSnaccOS(IR_FILE, &initReq->referenceId, 
	    &assocEntry->wsRefId);
	
	/* Get the initReq info from table management */
	if (OpTm_InitReqInfo(assocEntry->targetId, &initReqInfo) 
	    != IR_SUCCESS)
	{
		FreeAPDU(&sendReq.irApdu);
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&initReqInfo);

	/* Generate the reference Id */
	sprintf(tmpBuf, "%d", assocId);
	ASN_OctsAdd(&initReq->referenceId, tmpBuf, (ULONG)strlen(tmpBuf));

	/* Generate the protocol version */
	RescASN_IRToSnaccBS(IR_FILE, &initReq->protocolVersion,
	    &initReqInfo.protocolVersion);

	/* Generate the options */
	RescASN_IRToSnaccBS(IR_FILE, &initReq->options,
	    &initReqInfo.options);

	/* Copy the message sizing information */
	initReq->preferredMessageSize = 
	    initReqInfo.preferredMessageSize;
	initReq->maximumRecordSize = 
	    initReqInfo.maximumRecordSize;

	/* Generate the implementation Id */
	RescASN_IRToSnaccVS(IR_FILE, &initReq->implementationId,
	    &initReqInfo.implementationId);

	/* Generate the implementation Name */
	RescASN_IRToSnaccVS(IR_FILE, &initReq->implementationName,
	    &initReqInfo.implementationName);

	/* Generate the implementation version */
	RescASN_IRToSnaccVS(IR_FILE, &initReq->implementationVersion,
	    &initReqInfo.implementationVersion);

	/* Free all data malloced for the initReqInfo struct */
	OpTm_InitReqInfoFree(&initReqInfo);

	/* Get the target specific information from table management */
	targetInfo = (IR_TARGET_INFO *)RescIR_Zmalloc(IR_FILE,
     	     sizeof(IR_TARGET_INFO));
	if (OpTm_TargetInfo(assocEntry->targetId, targetInfo) == IR_FAIL)
	{
		FreeAPDU(&sendReq.irApdu);
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_MALLOCED(targetInfo);

	/* 
	 * Copy the target name into the association entry.
	 * The connect function did not have access to this structure
	 * so it put in the ipDomain name. Now that we have this structure
	 * loaded we can replace the local name value with the actual name
	 * rather than the ipDomainName.
	 */
	RescASN_DestroyVS(IR_FILE, &assocEntry->localName);
	RESC_ASN_INIT_VS(&assocEntry->localName);
	RescASN_CopyVS(IR_FILE, &assocEntry->localName,
	    &targetInfo->targetName);

	
	/* Generate the authentication string if neccessary */
	RescASN_IRToSnaccVS(IR_FILE, &initReq->idAuthentication,
	    &targetInfo->idAuthentication);

	/* Set parameters maintained in the assocEntry */
	assocEntry->resultNaming = targetInfo->resultSetNamingSupported;
	assocEntry->elementSetNaming = targetInfo->elementSetNamingSupported;

	/* Free all data malloced for the initReqInfo struct */
	OpTm_TargetGenInfoFree(targetInfo);
	
	/* Send the init request to the kernel */
	status = IRK_SendRequest((IRK_REQ_SERVICE_TYPE)IRK_INIT_REQ_TYPE, 
		    (IR_ASSOC_ID)assocId, 
		    (IRK_SEND_REQUEST_REQ *)&sendReq);
	assocEntry->state = OP_ASSOC_INITIALIZING;
	FreeAPDU(&sendReq.irApdu);
	if (status != IR_SUCCESS)
	{
		CS_LOG_PRINTS(IR_FILE, "Error Sending Init Req to IRK\n");
	}

	return (status);
}

/****************************************************************************
 * FUNCTION:  OpAm_AbortAssoc
 *
 * PURPOSE:
 *	This function is invoke by the control function csc when an
 *	abort status is detected.
 *
 * INPUT:
 *	ASSOC_ID	- association id
 *
 * OUTPUT:
 *	IR_STATUS		-
 *
 *
 * NOTES:
 * 	The User Application should use the IRO_ErrorReport function
 *	if an error status is returned.
 *
 ***************************************************************************/

IR_STATUS OpAm_AbortAssoc(IR_ASSOC_ID assocId)
{
	IR_STATUS status;

	/* 
	 * Call the Work space manager to delete the association from
	 * all workspaces.
 	 */
	OpWs_WsDeleteAssoc(assocId);

	status = IR_SUCCESS;
	if (OpAm_AssocDelete(assocId) == IR_FAIL)
	{
		status = IR_FAIL;
	}
	
	return (status);
}

/****************************************************************************
 * FUNCTION:  OpAm_ReleaseAssoc
 *
 * PURPOSE:
 *	This function is invoke by the control function csc when a 
 *	release event status is detected.
 *
 * INPUT:
 *	ASSOC_ID	- association id
 *
 * OUTPUT:
 *	IR_STATUS		-
 *
 *
 * NOTES:
 * 	The User Application should use the IRO_ErrorReport function
 *	if an error status is returned.
 *
 ***************************************************************************/

IR_STATUS OpAm_ReleaseAssoc(IR_ASSOC_ID assocId)
{
	IR_STATUS status;

	/* 
	 * Call the Work space manager to delete the association from
	 * all workspaces.
 	 */
	OpWs_WsDeleteAssoc(assocId);

	status = OpAm_AbortAssoc(assocId);

	return (status);
}
