

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpArc_ResourceControlService
 *
 * DESCRIPTION:
 *	This unit contains the functions used to service the resource
 *	control IR service apdu.
 *
 * CONTENTS:
 *	IRO_ResourceControl
 *	OpArc_RsrcControlService
 *	OpArc_RsrcControlServiceFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>

/****************************************************************************
 * FUNCTION:  IRO_ResourceControl
 *
 * PURPOSE:
 *	This function is used to generate the resource control response
 *	from the user application. The user application supplied information
 *	is converted into a resource control response apdu and sent to the 
 *	associated target.
 *
 * INPUT:
 *	IRO_RSRC_CNTRL_REQ		- resource control request.
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_ResourceControl(IR_USER_ID userId, IRO_RSRC_CNTRL_REQ *resp)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IR_STATUS	status;
	IRK_SEND_REQUEST_REQ	rsrcResp;
	OP_SESSION_INFO		*sessionInfo;

	/* Check if user exists */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_UNKNOWN_USER_ID, IR_FILE);
		return(IR_FAIL);
	}

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(resp->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}
	
	/* Create resource control response apdu */
	rsrcResp.irApdu.choiceId = APDU_RESOURCECONTROLRESP;
	rsrcResp.irApdu.a.resourceControlResp = (ResourceControlResp *)
	    RescIR_Zmalloc(IR_FILE, sizeof(ResourceControlResp));
	if (rsrcResp.irApdu.a.resourceControlResp == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		FreeAPDU(&(rsrcResp.irApdu));
		return (IR_FAIL);
	}


	/* If resource request had a reference id */
	if (assocEntry->refId.len > 0)
	{
		/* return the reference id with response */
		RescASN_IRToSnaccOS(IR_FILE,
	    	    &rsrcResp.irApdu.a.resourceControlResp->referenceId, 
		    &assocEntry->refId);
		RescASN_DestroyOS(IR_FILE, &assocEntry->refId);
	}	

	rsrcResp.irApdu.a.resourceControlResp->continueFlag =
	    (AsnBool)resp->continueFlag;

	if (resp->wantedFlagValid)
	{
		rsrcResp.irApdu.a.resourceControlResp->resultSetWanted = 
		    (AsnBool *)RescIR_Malloc(IR_FILE, sizeof(AsnBool));
	   	*rsrcResp.irApdu.a.resourceControlResp->resultSetWanted = 
	    	    (AsnBool)resp->resultSetWanted;
	}

	/* send resource response request */
	status = IRK_SendRequest(IRK_RESRC_CNTL_RES_TYPE, resp->assocId, 
	    &rsrcResp);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(rsrcResp.irApdu));
		return (IR_FAIL);
	}
	FreeAPDU(&(rsrcResp.irApdu));
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpArc_RsrcControlService
 *
 * PURPOSE:
 *	This function is used to process the resource control service
 *	apdu received from the target. If resource report information
 *	is contained in the resource control service then the resource
 *	report functions will be used to process it. The resource control
 *	information is converted from the apdu structure to the internal
 *	IR structure. If resource report information is available then it
 *	will be saved in the assoc list entry.
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to rsrc control info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY 		- pointer pointer to reply
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpArc_RsrcCntlService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_RSRC_CNTL_REQ *req)
{
	OP_ASSOC_ENTRY		*assocEntry;
	ResourceControlReq	*apduReq;

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Received access control request for unknown assocId\n");
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		FreeAPDU(apdu);
		return (IR_FAIL);
	}
	
	apduReq = apdu->a.resourceControlReq;

	/* Make sure there is not an old refId hanging around. */
	RescASN_DestroyOS(IR_FILE, &assocEntry->refId);
	/* Save the reference Id */
	RescASN_SnaccToIROS(IR_FILE, &assocEntry->refId,
	    &apduReq->referenceId);

	/* Decode the request. */
	/* Ignore the resource report for now */
	req->resourceReport = (IRO_COMM_RSRC_REPORT *)NULL;
	req->targetCmdSuspended = FALSE;
	if (apduReq->suspendedFlag)
	{
		req->targetCmdSuspended = (BOOLEAN)*apduReq->suspendedFlag;
	}
	req->partialResultsAvail = IR_RESULT_SET_NO_STATUS;
	if (apduReq->partialResultsAvailable)
	{
		req->partialResultsAvail = 
		    (IR_RESULT_SET_STATUS)*apduReq->partialResultsAvailable;
	}
	req->responseRequired = (BOOLEAN)apduReq->responseRequired;
	req->triggeredRequest = FALSE;
	if (apduReq->triggeredRequestFlag)
	{
		req->triggeredRequest = (BOOLEAN)*apduReq->triggeredRequestFlag;
	}
	
	FreeAPDU(apdu);
	return (IR_SUCCESS);
}


/****************************************************************************
 * FUNCTION:  OpArc_RsrcControlServiceFree 
 *
 * PURPOSE:
 *	This function is used to free the data returned to the user
 *	application.
 *
 * INPUT:
 *	
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/


void OpArc_RsrcControlServiceFree( IRO_COMM_RSRC_CNTL_REQ *req)
{
	if (req)
	{
		if (req->resourceReport != NULL)
		{
			OpArc_ResourceReportFree(req->resourceReport);
		}
		if (RESC_ASN_IS_MALLOCED(req))
		{
			RescIR_Free(IR_FILE, (CHAR *)req);
		}
	}
}
