 

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpArc_ResourceReportService
 *
 * DESCRIPTION:
 *	This unit contains the functions used generate and resource
 *	report request to send to a target; service a resource report 
 *	response from a target, process the resource report information
 *	supplied in either a resource report response message or resource 
 *	control request message provided by a target.
 *
 * CONTENTS:
 *	IRO_ResourceReport
 *	OpArc_RsrcReportService
 *	OpArc_RsrcReportServiceFree
 *	OpArc_ResourceReport
 *	OpArc_ResourceReportFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>
#include <string.h>
#include <report.h>


static void convertBib1RsrcRprt(IR_EXTERNAL *irExt, IRO_COMM_RSRC_REPORT *report);

static IRO_COMM_RSRC_RPRT_EST *createBib1EstimateStr();
static void destroyBib1EstimateStr(CHAR *src, void *data);
static IRO_COMM_RSRC_RPRT_EST *copyBib1EstimateStr(CHAR *src, void *data);

/****************************************************************************
 * FUNCTION:  IRO_ResourceReport
 *
 * PURPOSE:
 *	This function will take a user application supplied resource
 *	report structure and convert it to an APDU resource report request
 *	structure. The apdu will then be sent to the associated target.
 *
 * INPUT:
 *	IRO_RSRC_RPRT_REQ		- resource report request.
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_ResourceReport(IR_USER_ID userId, IRO_RSRC_RPRT_REQ *req)
{
	OP_ASSOC_ENTRY 	*assocEntry;
	IRK_SEND_REQUEST_REQ	sendReq;
	ResourceReportReq	*rrpt;
	OP_SESSION_INFO	*sessionInfo;
	
	IR_STATUS 	status;

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(req->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_UNKNOWN_USER_ID,
		    IR_FILE);
		return(IR_FAIL);
	}

	/* Create resource report apdu */
	sendReq.irApdu.choiceId = APDU_RESOURCEREPORTREQ;
	sendReq.irApdu.a.resourceReportReq = (ResourceReportReq *)
	    RescIR_Zmalloc(IR_FILE, sizeof(ResourceReportReq));

	if (sendReq.irApdu.a.resourceReportReq == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	/* Use a local pointer for readability */
	rrpt = sendReq.irApdu.a.resourceReportReq;

	/* Verify the preferred resource report format . */
	if (OpArc_VerifyRsrcRprtSyntaxOI(&req->rsrcRprtFmt) == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_UNKNOWN_OID, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}

	/* 
	 * Copy the preferred resource report format to the 
	 * outgoing apdu request.
	 */
	RescASN_IRToSnaccOI(IR_FILE, &rrpt->preferredRsrcRprtFmt,
	    &req->rsrcRprtFmt);

	/* Send resource report request */
	status = IRK_SendRequest(IRK_RES_RPT_REQ_TYPE, 
	    req->assocId, &sendReq);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}
	/* Update the assoc entry status */
	assocEntry->state =  OP_ASSOC_RSRC_REPORT;
	
	FreeAPDU(&(sendReq.irApdu));
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpArc_RsrcReportService
 *
 * PURPOSE:
 *	This function is used to process the resource report service
 *	reply received from the target.
 *	Use the OpArc_ResourceReport function to process the apdu because
 *	this common function is used by the resource control service to
 *	to process a resource report received as part of the resource 
 *	control request.
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to rsrc report info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY 		- pointer pointer to reply
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpArc_RsrcReportService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_RSRC_REPORT *reply)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IR_STATUS	status;

	/* check if assoc exists */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		FreeAPDU(apdu);
		return (IR_FAIL);
	}


	/* copy resource report to reply structure */
	status = OpArc_ResourceReport(
	    apdu->a.resourceReportResp->resourceReport, 
	    (SHORT)apdu->a.resourceReportResp->resourceReportStatus,
	    reply);

	if (status == IR_FAIL)
	{
		FreeAPDU(apdu);
		return (IR_FAIL);
	}

	assocEntry->state = OP_ASSOC_IDLE;

	FreeAPDU(apdu);

	return (IR_SUCCESS);
}


/****************************************************************************
 * FUNCTION:  OpArc_RsrcReportServiceFree 
 *
 * PURPOSE:
 *	This function is used to free the data returned to the user
 *	application.
 *
 * INPUT:
 *	
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/


void OpArc_RsrcReportServiceFree( IRO_COMM_RSRC_REPORT *rep)
{
	OpArc_ResourceReportFree(rep);
}

/****************************************************************************
 * FUNCTION:  OpArc_ResourceReport
 *
 * PURPOSE:
 *	This function is used to convert the resource report from the
 *	apdu format to an internal IR format. If the resource report
 *	is not in the bib1 format then convert the unknown object id
 *	and store that in the IRO_COMM_RSRC_REPORT structure.
 *
 * INPUT:
 *	APDU->resourceReport		- resource report info
 *	SHORT				- resource report status
 *	
 *
 * OUTPUT:
 *	IRO_COMM_RSRC_REPORT		- pointer to IR structure.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS	OpArc_ResourceReport(ResourceReport *rpt,
			SHORT	reportStatus,
			IRO_COMM_RSRC_REPORT *report)
{
	IR_EXTERNAL	*irExt;
	static CHAR		tmpBuf[100];
	
	/* Prepare the reply structure before using it */
	RESC_ASN_NO_MALLOC(&report->message);
	RESC_ASN_INIT_VS(&report->message);
	RESC_ASN_NO_MALLOC(&report->estimates);
	RESC_LIST_INIT(&report->estimates);
	RESC_ASN_NO_MALLOC(&report->oid);
	RESC_ASN_INIT_OI(&report->oid);

	report->reportStatus = reportStatus;

	/* Check for the optional resource report to be absent. */
	if (rpt == NULL)
	{
		return(IR_SUCCESS);
	}


	irExt = RescASN_CreateExternal(IR_FILE);

	/* Convert the resource report external to an IR external */
	if (RescASN_SnaccToIRExternal(IR_FILE, irExt, rpt) == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, "Couldn't create resource report external\n");
		RescASN_DestroyExternal(IR_FILE, irExt);
		return(IR_FAIL);
	}
	else
	{
		/* Convert the external into resource report structure */
		VSToStrNullTerm(tmpBuf, &irExt->direct, 100);
		if (strcmp(tmpBuf, OID_RESOURCE_REPORT_FMT_SKL) == 0)
		{
			RescASN_CopyVS(IR_FILE, &report->message, 
			    &irExt->octetAligned);
		}
		else if (strcmp(tmpBuf, OID_RESOURCE_REPORT_FMT_BIB1) == 0)
		{
			(void)convertBib1RsrcRprt(irExt, report);
		}

		/* Copy the object identifier to the report structure */
		RescASN_AddOI(IR_FILE, &report->oid, tmpBuf, 
		    LENPLUSNULL(tmpBuf));
			
		RescASN_DestroyExternal(IR_FILE, irExt);
		CS_LOG_PRINTV(IR_FILE, "Got the resource report: %s\n",
		    tmpBuf);

		if (!report->message.sPtr)
		{
			RescASN_AddVS(IR_FILE, &report->message,
			    "No message supplied by target",
			    strlen("No message supplied by target"));
		}
			
		return (IR_SUCCESS);
	}
}


/****************************************************************************
 * FUNCTION:  OpArc_ResourceReportFree
 *
 * PURPOSE:
 *	This function is used to free all resource report information
 *	from the IRO_COMM_RSRC_REPORT structure.
 *
 * INPUT:
 *	IRO_COMM_RSRC_REPORT		- pointer to structure to free.
 *
 * OUTPUT:
 *	none.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpArc_ResourceReportFree(IRO_COMM_RSRC_REPORT *report)
{
	if (report)
	{
		RescASN_DestroyOS(IR_FILE, &report->message);
		RescASN_DestroyOI(IR_FILE, &report->oid);
		RescList_Destroy(IR_FILE, &report->estimates);
		if (RESC_ASN_IS_MALLOCED(report))
		{
			RescIR_Free(IR_FILE, (CHAR *)report);
		}
	}
}

/****************************************************************************
 * FUNCTION:  OpArc_VerifyRsrcRprtSyntaxOI
 *
 * PURPOSE:
 *	This function is used to verify the object identifier used
 *	to identify the preferred resource report syntax.
 *
 * INPUT:
 *	OBJECT_IDENTIFIER	
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpArc_VerifyRsrcRprtSyntaxOI(OBJECT_IDENTIFIER *oid)
{
	SHORT i;
	static CHAR	*knownOids[]=
	{
		(CHAR *)OID_RESOURCE_REPORT_FMT_BIB1,
		(CHAR *)OID_RESOURCE_REPORT_FMT_SKL,
		(CHAR *)NULL
	};


	i = 0;

	while (knownOids[i] != NULL)
	{
		if (strncmp(oid->sPtr, knownOids[i], oid->len) == 0)
		{
			return(IR_SUCCESS);
		}
		else
		{
			i++;
		}
	}

	return(IR_FAIL);
}

/****************************************************************************
 * LOCAL FUNCTION:  convertBib1RsrcReportBib1
 *
 * PURPOSE:
 *	This function is used to convert an IR external containing a bib1
 *	resource report format object identifier to the IRO_COMM_RSRC_REPORT
 *	structure used internally by the IR client. This function will
 *	only convert the estimates and the message from the bib1 resource
 *	report to the internal IR message format.
 *
 * INPUT:
 *	IR_EXTERNAL pointer 		- data to convert.
 *
 * OUTPUT:
 *	IRO_COMM_RSRC_REPORT pointer	- preinitialized structure.
 *
 *
 * NOTES:
 *	This function assumes that the bib1 resource report is being used.
 *
 ***************************************************************************/

static void convertBib1RsrcRprt(IR_EXTERNAL *irExt, 
    IRO_COMM_RSRC_REPORT *report)
{

	RsrcReport	*rrPtr;
	IRO_COMM_RSRC_RPRT_EST *est;
	Estimate	*snaccEst;

	/* Cast the any_asn1.type pointer to a bib1. resource report. */
	if (irExt->any == (void *)NULL)
	{
		/* Nothing to convert. */
		CS_LOG_PRINTS(IR_FILE, 
		    "No resource report found in external. \n");
		return;
	}

	/* Begin conversion */
	rrPtr = (RsrcReport *)irExt->any;

	
	/* 
	 * Foreach element in the apdu list create a corresponding one
	 * in the IR_LIST.
	 */
	FOR_EACH_LIST_ELMT(snaccEst, rrPtr->estimates)
	{
		est = createBib1EstimateStr();
		est->estimateType = (USHORT)snaccEst->type;
		est->value = (ULONG)snaccEst->value;
		if (snaccEst->curreny_code)
		{
			est->currency = (ULONG)*snaccEst->curreny_code;
			est->knownCurrency = TRUE;
		}
		RescList_Append(IR_FILE, &report->estimates, 
		    est, destroyBib1EstimateStr, (void *(*)())copyBib1EstimateStr);
	}
	RescASN_SnaccToIRVS(IR_FILE, &report->message,
	   &rrPtr->message);
}


/****************************************************************************
 * LOCAL FUNCTION:  createBib1EstimateStr
 *
 * PURPOSE:
 *	This function creates a bib1 resource report estimate structure
 *	that is used in the IR_RSRC_RPRT_COMM structure's estimate list.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	pointer to IRO_COMM_RSRC_RPRT_EST
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static IRO_COMM_RSRC_RPRT_EST *createBib1EstimateStr()
{

	IRO_COMM_RSRC_RPRT_EST	*est;

	est = (IRO_COMM_RSRC_RPRT_EST *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IRO_COMM_RSRC_RPRT_EST));

	if (est)
	{
		RESC_ASN_MALLOCED(est);
		est->knownCurrency = FALSE;
	}
	else
	{
		CS_LOG_PRINTS(IR_FILE, 
		    "Error mallocing IR bib estimate structure\n");
	}
	return(est);
}

/****************************************************************************
 * LOCAL FUNCTION:  destroyBib1EstimateStr
 *
 * PURPOSE:
 *	This function destroys a bib1 resource report estimate structure
 *	that is used in the IR_RSRC_RPRT_COMM structure's estimate list.
 *
 * INPUT:
 *	pointer to IRO_COMM_RSRC_RPRT_EST
 *
 * OUTPUT:
 *	none.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static void destroyBib1EstimateStr(CHAR *src, void *data)
{

	IRO_COMM_RSRC_RPRT_EST *est;
	if (data)
	{
		est = (IRO_COMM_RSRC_RPRT_EST *)data;
		
		if (RESC_ASN_IS_MALLOCED(est))
		{
			RescIR_Free(IR_FILE, (CHAR *)est);
		}
	}
}

/****************************************************************************
 * LOCAL FUNCTION: copyBib1EstimateStr
 *
 * PURPOSE:
 *	This function creates a bib1 resource report estimate structure
 *	that is used in the IR_RSRC_RPRT_COMM structure's estimate list.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	pointer to IRO_COMM_RSRC_RPRT_EST
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static IRO_COMM_RSRC_RPRT_EST *copyBib1EstimateStr(CHAR *src, void *data)
{

	IRO_COMM_RSRC_RPRT_EST	*old;
	IRO_COMM_RSRC_RPRT_EST	*new;

	if (!data)
	{
		return((IRO_COMM_RSRC_RPRT_EST *)NULL);
	}

	old = (IRO_COMM_RSRC_RPRT_EST *)data;
	new = createBib1EstimateStr();

	new->estimateType = old->estimateType;
	new->value = old->value;
	new->currency = old->currency;
	new->knownCurrency = old->knownCurrency;

	return(new);
}
