
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: tm_dbase.c
 *
 * DESCRIPTION:
 *	This unit contains the functions used to get database info
 *
 * CONTENTS:
 *	IRO_DatabaseInfo
 *	OpTm_DBInfoFree
 *	OpTm_DBFree
 *	OpTm_ConvertRecordToDbInfo
 *	OpTm_ConvertDbInfoToRecord
 *	OpTm_GetDBInfo
 *	OpTm_DBClone
 *	OpTm_DbInfoInit
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <stdlib.h>
#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <rescio.h>
#include <resclist.h>
#include <asn_tool.h>
#include <op/ir_op.h>


/****************************************************************************
 * FUNCTION: IRO_DatabaseInfo 
 *
 * PURPOSE:
 *	This function is used to return database information.
 *
 * INPUT:
 *	IRO_DB_INFO_TYPE	type of info required
 * 	IRO_DB_INFO_REQ*	pointer to request
 *
 * OUTPUT:
 *	IRO_DB_INFO_REPLY*	pointer to return structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_DatabaseInfo(IR_USER_ID userId, IRO_DB_INFO_TYPE infoType,
		    IRO_DB_INFO_REQ *req, IRO_DB_INFO_REPLY *reply)
{
	IR_STATUS		status;
	VISIBLE_STRING		key;
	CHAR			buf[RESC_IO_KEY_LENGTH+1];
	VISIBLE_STRING		*record;
	IR_DATABASE_INFO	*info;
	RESC_IO_FILE_STRUCT	filePtr;
	VISIBLE_STRING		targetInfoFileName;

	/* Get target file name and verify userId with current user id */
	status = OpTm_GetTargetFileName(userId, &targetInfoFileName);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	/* Open target info file */
	status = RescIO_OpenFile(IR_FILE, OpTm_WorkPath(), 
	    targetInfoFileName, &filePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_OPEN_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	
	/* Create key for target */
	RESC_ASN_INIT_VS(&key);
	RESC_ASN_NO_MALLOC(&key);
	sprintf(buf, "%d", req->targetId);
	status = RescASN_AddVS(IR_FILE, &key, buf, strlen(buf));	
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	/* Seek to target */
	status = RescIO_Seek(IR_FILE, &filePtr, RESC_IO_TARGET_INFO, 
	    &key, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	if (RescIO_EOF(filePtr))
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_TARGET_UNKNOWN, IR_FILE);
		return (IR_FAIL);
	}

	/* Dump target record */
	status = RescIO_GetRecord(IR_FILE, filePtr, &record);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, record);


	/* Create db info list */
	reply->dbInfo = (IR_LIST *)RescList_Create(IR_FILE);
	if (reply->dbInfo == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	switch (infoType)
	{
		case IRO_DB_INFO_ALL:
			/* Get all databases for target */
			status = OpTm_GetDBInfo(&filePtr, req->targetId, 
			    reply->dbInfo);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}
			break;
		case IRO_DB_INFO_SINGLE:
			/* Create key = targetId + databaseName */
			memcpy(buf, key.sPtr, key.len);
			buf[key.len] = EOS;
			strncat(buf, req->dbName.sPtr, req->dbName.len);
			RescASN_DestroyVS(IR_FILE, &key);
			RESC_ASN_INIT_VS(&key);
			RESC_ASN_NO_MALLOC(&key);
			status = RescASN_AddVS(IR_FILE, &key, buf, strlen(buf));
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}

			/* Seek to start of record */
			status = RescIO_Seek(IR_FILE, &filePtr, 
			    RESC_IO_DATABASE_INFO, &key, NULL);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
				    IRO_FILE_READ_ERROR, IR_FILE);
				return (IR_FAIL);
			}
			if (RescIO_EOF(filePtr))
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
				    IRO_DATABASE_UNKNOWN, IR_FILE);
				return (IR_FAIL);
			}

			/* Get record */
			status = RescIO_GetRecord(IR_FILE, filePtr, &record);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
				    IRO_FILE_READ_ERROR, IR_FILE);
				return (IR_FAIL);
			}

			status = OpTm_ConvertRecordToDbInfo(record, &info);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}
			RescASN_DestroyVS(IR_FILE, record);

			/* Add record to list */
			status = RescList_Append(IR_FILE, reply->dbInfo, 
			    info, OpTm_DBFree, (void *)OpTm_DBClone);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}
			break;
		default:
			return (IR_FAIL);
	}

	/* Close user info file */
	status = RescIO_CloseFile(IR_FILE, &filePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	OpCf_IROMark(OpTm_DBInfoFree, reply->dbInfo);	
	RescASN_DestroyVS(IR_FILE, &key);
	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION: OpTm_DBInfoFree 
 *
 * PURPOSE:
 *	This function is used to free database info reply.
 *
 * INPUT:
 *	IR_LIST	pointer to reply structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_DBInfoFree(IR_LIST *reply)
{
	if (reply != NULL)
	{
		RescList_Destroy(IR_FILE, reply);
	}
}

/****************************************************************************
 * FUNCTION: OpTm_DBFree 
 *
 * PURPOSE:
 *	This function is used to free database info.
 *
 * INPUT:
 *	CHAR 			pointer to source file
 *	IR_DATABASE_INFO	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_DBFree(CHAR *srcFile, IR_DATABASE_INFO *info)
{
	if (info != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &(info->databaseName));
		RescASN_DestroyVS(IR_FILE, &(info->description));
		RescASN_DestroyVS(IR_FILE, &(info->chargingPolicy));
		if (RESC_LIST_SIZE(&(info->attrSetId)) != 0)
		{
			RescList_Destroy(IR_FILE, &(info->attrSetId));
		}
		if (RESC_LIST_SIZE(&(info->recordSyntaxList)) != 0)
		{
			RescList_Destroy(IR_FILE, &(info->recordSyntaxList));
		}
		if (RESC_LIST_SIZE(&(info->elementSetNames)) != 0)
		{
			RescList_Destroy(IR_FILE, &(info->elementSetNames));
		}
		if (info->isMalloced)
		{
			RescIR_Free(IR_FILE, (CHAR *)info);
		}	
	}
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertRecordToDbInfo
 *
 * PURPOSE:
 *	This function is used to convert a record to the user info struct.
 *
 * INPUT:
 *	VISIBLE_STRING	pointer to the record 
 *
 * OUTPUT:
 *	IR_DATABASE_INFO 	pointer to pointer to info	
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertRecordToDbInfo(VISIBLE_STRING *record, 
			 IR_DATABASE_INFO **infoPtr)
{
	IR_DATABASE_INFO	*info;
	IR_STATUS		status;
	CHAR			*elem;
	CHAR			*buf;
	USHORT			tag;
	ULONG			len;
	BOOLEAN			recEnd;
	ULONG			ePtr;
	VISIBLE_STRING		*name;
	OBJECT_IDENTIFIER	*oid;

	/* Allocate memory for info structure */
	info = (IR_DATABASE_INFO *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IR_DATABASE_INFO));
	if (info == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
	 	   IRO_MEMORY_ERROR, IR_FILE);
	 	   return (IR_FAIL);
	}
	RESC_ASN_MALLOCED(info);
	OpTm_DbInfoInit(info);
	*infoPtr = info;

	/* Get first tag in record */
	ePtr = 0;
	buf = OpTm_GetElem(*record, 0, RESC_IO_TAG_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	tag = (USHORT)atoi(buf);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}

	/* Get length of first element in record */
	buf = OpTm_GetElem(*record, RESC_IO_TAG_LENGTH, 
	    RESC_IO_LEN_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	len = atol(buf);

	/* Get first element */
	elem = OpTm_GetElem(*record, 
	    (RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), len);
	if (elem == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}

	/* while not end of record */
	recEnd = FALSE;
	while (!recEnd)
	{
		/* Add element to info structure */
		switch (tag)
		{
			case OP_TRGT_ID:
				info->targetId = (USHORT)atoi(elem);
				break;
			case OP_DB_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &info->databaseName, elem, (SHORT)len);
				break;
			case OP_DESC:
				status = RescASN_AddVS(IR_FILE, 
				    &info->description, elem, (SHORT)len);
				break;
			case OP_CHRG_POLICY:
				status = RescASN_AddVS(IR_FILE, 
				    &info->chargingPolicy, elem, (SHORT)len);
				break;
			case OP_ATTR_SET_ID:
				oid = RescASN_CreateOI(IR_FILE);
				if (oid == NULL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescASN_AddOI(IR_FILE, oid, 
				    elem, (SHORT)strlen(elem));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescList_Append(IR_FILE, 
				    &info->attrSetId, oid, RescASN_DestroyOI,
				    (void *)RescASN_CloneVS);
				break;
			case OP_REC_SYN:
				oid = RescASN_CreateOI(IR_FILE);
				if (oid == NULL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescASN_AddOI(IR_FILE, oid, 
				    elem, (SHORT)strlen(elem));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescList_Append(IR_FILE, 
				    &(info->recordSyntaxList), oid, 
				    RescASN_DestroyOI, (void *)RescASN_CloneVS);
				break;
			case OP_SET_NAME:
				name = RescASN_CreateOI(IR_FILE);
				if (name == NULL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescASN_AddVS(IR_FILE, name, 
				    elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				status = RescList_Append(IR_FILE, 
				    &(info->elementSetNames), name, 
				    RescASN_DestroyVS, (void *)RescASN_CloneVS);
				break;
			default:
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_DATA_ERROR, IR_FILE);
				return (IR_FAIL);
		}
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
			    IRO_MEMORY_ERROR, IR_FILE);
			return (IR_FAIL);
		}
	
		/* Get next tag */
		ePtr = ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH + len;
		buf = OpTm_GetElem(*record, ePtr, RESC_IO_TAG_LENGTH);

		/* If not end of record get next element */
		if (buf == NULL)
		{
			recEnd = TRUE;
		}
		else
		{
			tag = (USHORT)atoi(buf);
			if (buf == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
			        return (IR_FAIL);
			}
			buf = OpTm_GetElem(*record, (ePtr + RESC_IO_TAG_LENGTH),
			    RESC_IO_LEN_LENGTH);
			if (buf == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
			        return (IR_FAIL);
			}
			len = atol(buf);
			elem = OpTm_GetElem(*record, 
			    (ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), 
			    len);
		}
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertDbInfoToRecord 
 *
 * PURPOSE:
 *	This function is used to convert database info to a record.
 *
 * INPUT:
 *	IR_DATABASE_INFO 	pointer to pointer to record	
 *
 * OUTPUT:
 *	VISIBLE_STRING	pointer to the user info 
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertDbInfoToRecord(IR_DATABASE_INFO *info,
				VISIBLE_STRING **recordPtr)
{
	VISIBLE_STRING		*record;
	IR_STATUS		status;
	CHAR			buf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			tmpBuf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			*rec;
	OBJECT_IDENTIFIER	*oid;
	VISIBLE_STRING		*name;

	/* Allocate space for record */
	record = RescASN_CreateVS(IR_FILE);
	if (record == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}
	*recordPtr = record;
	rec = buf;

	/* Concatenate each element of info structure to a buffer */
	sprintf(tmpBuf, "%d", info->targetId);
	OpTm_PutElem(rec, OP_TRGT_ID, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	OpTm_PutElem(rec, OP_DB_NAME, info->databaseName.sPtr, 
	    info->databaseName.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_DESC, info->description.sPtr, 
	    info->description.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_CHRG_POLICY, info->chargingPolicy.sPtr, 
	    info->chargingPolicy.len, (ULONG *)&rec);

	if (RESC_LIST_SIZE(&(info->attrSetId)) != 0)
	{
		RESC_LIST_SET_FIRST(&(info->attrSetId));
		while (!RESC_LIST_IS_LAST(&(info->attrSetId)))
		{
			oid = RESC_LIST_GET(&(info->attrSetId));
			OpTm_PutElem(rec, OP_ATTR_SET_ID, oid->sPtr, oid->len, 
			    (ULONG *)&rec);
			RESC_LIST_SET_NEXT(&(info->attrSetId));
		}
		oid = RESC_LIST_GET(&(info->attrSetId));
		OpTm_PutElem(rec, OP_ATTR_SET_ID, oid->sPtr, oid->len, 
		    (ULONG *)&rec);
	}
	
	if (RESC_LIST_SIZE(&(info->recordSyntaxList)) != 0)
	{
		RESC_LIST_SET_FIRST(&(info->recordSyntaxList));
		while (!RESC_LIST_IS_LAST(&(info->recordSyntaxList)))
		{
			oid = RESC_LIST_GET(&(info->recordSyntaxList));
			OpTm_PutElem(rec, OP_REC_SYN, oid->sPtr, oid->len, 
			    (ULONG *)&rec);
			RESC_LIST_SET_NEXT(&(info->recordSyntaxList));
		}
		oid = RESC_LIST_GET(&(info->recordSyntaxList));
		OpTm_PutElem(rec, OP_REC_SYN, oid->sPtr, oid->len, 
		    (ULONG *)&rec);
	}
	
	if (RESC_LIST_SIZE(&(info->elementSetNames)) != 0)
	{
		RESC_LIST_SET_FIRST(&(info->elementSetNames));
		while (!RESC_LIST_IS_LAST(&(info->elementSetNames)))
		{
			name = RESC_LIST_GET(&(info->elementSetNames));
			OpTm_PutElem(rec, OP_SET_NAME, name->sPtr, name->len, 
			    (ULONG *)&rec);
			RESC_LIST_SET_NEXT(&(info->elementSetNames));
		}
		name = RESC_LIST_GET(&(info->elementSetNames));
		OpTm_PutElem(rec, OP_SET_NAME, name->sPtr, name->len, 
		    (ULONG *)&rec);
	}
	
	/* Copy buffer to record */
	status = RescASN_AddVS(IR_FILE, record, buf, strlen(buf));	
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_GetDBInfo 
 *
 * PURPOSE:
 *	This function is used to get all database info for a target.
 *
 * INPUT: 
 *	RESC_IO_FILE_STRUCT	filePtr
 *	IR_TARGET_ID		targetId
 *
 * OUTPUT:
 *	IR_LIST 	pointer to list 	
 *	IR_STATUS
 *
 * NOTES:
 *	This function gets all databases for the target found 
 *	at the current file cursor position.
 *
 ***************************************************************************/

IR_STATUS OpTm_GetDBInfo(RESC_IO_FILE_STRUCT *filePtr, IR_TARGET_ID targetId,
				IR_LIST *dbInfo)
{
	BOOLEAN		newTarget;
	IR_DATABASE_INFO	*info;
	IR_STATUS	status;
	VISIBLE_STRING	*record;
	fpos_t		oldRec;	

	/* Save pointer to record */
	status = RescIO_GetRecPtr(IR_FILE, *filePtr, &oldRec);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	/* Seek to start of first record */
	status = RescIO_Seek(IR_FILE, filePtr, 
	    RESC_IO_DATABASE_INFO, NULL, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	newTarget = FALSE;
	/* while not eof or new target convert record and get next */
	while (!RescIO_EOF(*filePtr) && !newTarget)
	{
		status = RescIO_GetRecord(IR_FILE, *filePtr, &record);
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_FILE_READ_ERROR, IR_FILE);
			return (IR_FAIL);
		}
		status = OpTm_ConvertRecordToDbInfo(record, &info);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
		RescASN_DestroyVS(IR_FILE, record);

		/* if targetId matches, add database record to info struct */
		if (info->targetId == targetId)
		{
			status = RescList_Append(IR_FILE, 
			    dbInfo, info, OpTm_DBFree, (void *)OpTm_DBClone);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
			    	    IRO_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}
		}
		else
		{
			newTarget = TRUE;
			OpTm_DBFree(IR_FILE, info);
		}

		if (!newTarget)
		{
			/* Save pointer to record */
			status = RescIO_GetRecPtr(IR_FILE, *filePtr, &oldRec);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
				    IRO_FILE_READ_ERROR, IR_FILE);
				return (IR_FAIL);
			}

			/* Seek to start of first record */
			status = RescIO_Seek(IR_FILE, filePtr, 
			    RESC_IO_DATABASE_INFO, NULL, NULL);
			if (status == IR_FAIL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING,
				    IRO_FILE_READ_ERROR, IR_FILE);
				return (IR_FAIL);
			}
		}
	}
	
	/* Reset pointer to previous record */
	status = RescIO_SetRecPtr(IR_FILE, *filePtr, oldRec);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_WRITE_ERROR, IR_FILE);
		return (IR_FAIL);
	}


	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_DBClone 
 *
 * PURPOSE:
 *	This function is used to copy an IR_DATABASE_INFO struct.
 *
 * INPUT:
 *	CHAR			pointer to source file
 *	IR_DATABASE_INFO	source
 *
 * OUTPUT:
 *	IR_DATABASE_INFO	dest
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_DATABASE_INFO *OpTm_DBClone(CHAR *srcFile, IR_DATABASE_INFO *sInfo)
{
	IR_DATABASE_INFO	*dInfo;
	IR_STATUS		status;
	
	dInfo = (IR_DATABASE_INFO *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IR_DATABASE_INFO));
	if (dInfo == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	RESC_ASN_MALLOCED(dInfo);

	dInfo->targetId = sInfo->targetId; 

	RESC_ASN_NO_MALLOC(&(dInfo->databaseName));
	RESC_ASN_INIT_VS(&(dInfo->databaseName));
	status = RescASN_CopyVS(IR_FILE, &(dInfo->databaseName),
	    &(sInfo->databaseName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}

	RESC_ASN_NO_MALLOC(&(dInfo->description));
	RESC_ASN_INIT_VS(&(dInfo->description));
	status = RescASN_CopyVS(IR_FILE, &(dInfo->description),
	    &(sInfo->description));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}

	RESC_ASN_NO_MALLOC(&(dInfo->chargingPolicy));
	RESC_ASN_INIT_VS(&(dInfo->chargingPolicy));
	status = RescASN_CopyVS(IR_FILE, &(dInfo->chargingPolicy),
	    &(sInfo->chargingPolicy));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}

	RESC_ASN_NO_MALLOC(&(dInfo->attrSetId));
	status = RescList_Copy(IR_FILE, &(dInfo->attrSetId),
	    &(sInfo->attrSetId));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}

	RESC_ASN_NO_MALLOC(&(dInfo->recordSyntaxList));
	status = RescList_Copy(IR_FILE, &(dInfo->recordSyntaxList),
	    &(sInfo->recordSyntaxList));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}

	RESC_ASN_NO_MALLOC(&(dInfo->elementSetNames));
	status = RescList_Copy(IR_FILE, &(dInfo->elementSetNames),
	    &(sInfo->elementSetNames));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}


	return (dInfo);
}

/****************************************************************************
 * FUNCTION: OpTm_DbInfoInit 
 *
 * PURPOSE:
 *	This function is initializes a db info struct
 *
 * INPUT:
 *	IR_DATABASE_INFO	pointer to the database info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpTm_DbInfoInit(IR_DATABASE_INFO *info)
{
	RESC_ASN_NO_MALLOC(&(info->databaseName));
	RESC_ASN_INIT_VS(&(info->databaseName));
	RESC_ASN_NO_MALLOC(&(info->description));
	RESC_ASN_INIT_VS(&(info->description));
	RESC_ASN_NO_MALLOC(&(info->chargingPolicy));
	RESC_ASN_INIT_VS(&(info->chargingPolicy));

	/* Initialize lists in structure */
	RESC_LIST_INIT(&(info->attrSetId));
	RESC_ASN_NO_MALLOC(&(info->attrSetId));
	RESC_LIST_INIT(&(info->recordSyntaxList));
	RESC_ASN_NO_MALLOC(&(info->recordSyntaxList));
	RESC_LIST_INIT(&(info->elementSetNames));
	RESC_ASN_NO_MALLOC(&(info->elementSetNames));
}

