

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: tm_trgt.c
 *
 * DESCRIPTION:
 *	This unit contains the functions used get target info
 *
 * CONTENTS:
 *	IRO_TargetInfo
 *	OpTm_TargetInfo
 *	OpTm_TargetAssocInfo
 *	OpTm_TargetAssocInfoFree
 *	OpTm_TargetGenInfoFree
 *	OpTm_GetTargetInfo
 *	OpTm_GetNextTargetInfo
 *	OpTm_CloseTargetInfoFile
 *	OpTm_TargetInfoFree
 *	OpTm_ConvertTargetInfoToRecord
 *	OpTm_ConvertRecordToTargetInfo
 *	OpTm_TargetInfoInit
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <stdlib.h>
#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <rescio.h>
#include <resclist.h>
#include <asn_tool.h>
#include <op/ir_op.h>


static IR_TARGET_ASSOC_INFO *optm_AssocInfoCreate();
static IR_TARGET_INFO *optm_GenInfoCreate();


/****************************************************************************
 * FUNCTION: IRO_TargetInfo 
 *
 * PURPOSE:
 *	This function is used to return target information.
 *
 * INPUT:
 *	IR_USER_ID		user id
 *	IRO_TARGET_INFO_TYPE	type of info required
 * 	IRO_TARGET_INFO_REQ*	pointer to request
 *
 * OUTPUT:
 *	IRO_TARGET_INFO_REPLY*	pointer to return structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_TargetInfo(IR_USER_ID userId, IRO_TARGET_INFO_TYPE infoType,
		    IR_TARGET_ID targetId, IRO_TARGET_INFO_REPLY *reply)
{
	IR_STATUS	status;

	switch (infoType)
	{
		case IRO_TRGT_INFO_ASSOC:
			/* Allocate space for reply */
			reply->targetAssocInfo = optm_AssocInfoCreate();

			/* Get association info */
			status = OpTm_TargetAssocInfo(targetId, 
			    reply->targetAssocInfo);
			if (status == IR_FAIL)
			{
				OpTm_TargetAssocInfoFree(reply->targetAssocInfo);
				return (IR_FAIL);
			}
			OpCf_IROMark(OpTm_TargetAssocInfoFree, 
			    reply->targetAssocInfo);	
			break;
		case IRO_TRGT_INFO_GEN:
			/* Allocate space for reply */
			reply->targetGenInfo = optm_GenInfoCreate();

			/* Get general info */
			status = OpTm_TargetInfo(targetId, 
			    reply->targetGenInfo);
			if (status == IR_FAIL)
			{
				reply->targetGenInfo = (IR_TARGET_INFO *)
				    RescIR_Free(IR_FILE, 
				    (CHAR *)reply->targetGenInfo);
				return (IR_FAIL);
			}
			OpCf_IROMark(OpTm_TargetGenInfoFree, 
			    reply->targetGenInfo);	
			break;
		default:
			return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpTm_TargetInfo
 *
 * PURPOSE:
 *	This function is used to return target information.
 *
 * INPUT:
 *	IR_TARGET_ID	target id
 *
 * OUTPUT:
 *	IR_TARGET_INFO	pointer to target info
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_TargetInfo(IR_TARGET_ID targetId,
		    IR_TARGET_INFO *info)
{
	IR_STATUS	status;
	OP_TARGET_INFO_STRUCT	*targetInfo;
	VISIBLE_STRING	targetFileName;
	RESC_IO_FILE_STRUCT	targetFilePtr;

	status = OpTm_GetTargetFileName(OpTm_UserId(), &targetFileName);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	/* Get all info for target */
	status = OpTm_GetTargetInfo(&targetId, targetFileName, &targetFilePtr,
	    &targetInfo);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, &targetFileName);
	/* Copy general info to return structure */
	info->targetId = targetInfo->targetId;

	RESC_ASN_NO_MALLOC(&(info->targetName));
	RESC_ASN_INIT_VS(&(info->targetName));
	status = RescASN_CopyVS(IR_FILE, &(info->targetName),
	    &(targetInfo->targetName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->description));
	RESC_ASN_INIT_VS(&(info->description));
	status = RescASN_CopyVS(IR_FILE, &(info->description),
	    &(targetInfo->description));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->options));
	RESC_ASN_INIT_BS(&(info->options));
	status = RescASN_CopyBS(IR_FILE, &(info->options),
	    &(targetInfo->options));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->contactInfo));
	RESC_ASN_INIT_VS(&(info->contactInfo));
	status = RescASN_CopyVS(IR_FILE, &(info->contactInfo),
	    &(targetInfo->contactInfo));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	info->resultSetNamingSupported = 
	    targetInfo->defResultSetNamingSupported;

	info->maxNumResultSets = targetInfo->maxNumResultSets;

	info->hoursOfOperation = targetInfo->hoursOfOperation;

	RESC_ASN_NO_MALLOC(&(info->chargingPolicy));
	RESC_ASN_INIT_VS(&(info->chargingPolicy));
	status = RescASN_CopyVS(IR_FILE, &(info->chargingPolicy),
	    &(targetInfo->chargingPolicy));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->idAuthentication));
	RESC_ASN_INIT_VS(&(info->idAuthentication));
	status = RescASN_CopyVS(IR_FILE, &(info->idAuthentication),
	    &(targetInfo->idAuthentication));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	/* Get database info for target */
	status = OpTm_GetDBInfo(&targetFilePtr, info->targetId, 
	    &(info->dbInfo));
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	status = OpTm_CloseTargetInfoFile(&targetFilePtr);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}
	OpTm_TargetInfoFree(targetInfo);

	return (IR_SUCCESS);
	
}

/****************************************************************************
 * FUNCTION:  OpTm_TargetAssocInfo
 *
 * PURPOSE:
 *	This function is used to return target association information.
 *
 * INPUT:
 *	IR_TARGET_ID	target id
 *
 * OUTPUT:
 *	IR_TARGET_INFO	pointer to target info
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_TargetAssocInfo(IR_TARGET_ID targetId,
		    IR_TARGET_ASSOC_INFO *info)
{
	IR_STATUS	status;
	OP_TARGET_INFO_STRUCT	*targetInfo;
	RESC_IO_FILE_STRUCT	targetFilePtr;
	VISIBLE_STRING		targetFileName;

	status = OpTm_GetTargetFileName(OpTm_UserId(), &targetFileName);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	/* Get all target information */
	status = OpTm_GetTargetInfo(&targetId, targetFileName, &targetFilePtr,
	    &targetInfo);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, &targetFileName);

	status = OpTm_CloseTargetInfoFile(&targetFilePtr);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	/* Copy to assocation info to return structure */
	info->targetId = targetInfo->targetId;

	RESC_ASN_NO_MALLOC(&(info->applCntxName));
	RESC_ASN_INIT_VS(&(info->applCntxName));
	status = RescASN_CopyVS(IR_FILE, &(info->applCntxName),
	    &(targetInfo->applCntxName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->applProcessTitle));
	RESC_ASN_INIT_VS(&(info->applProcessTitle));
	status = RescASN_CopyVS(IR_FILE, &(info->applProcessTitle),
	    &(targetInfo->applProcessTitle));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->applEntityQual));
	RESC_ASN_INIT_VS(&(info->applEntityQual));
	status = RescASN_CopyVS(IR_FILE, &(info->applEntityQual),
	    &(targetInfo->applEntityQual));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->ipAddress));
	RESC_ASN_INIT_VS(&(info->ipAddress));
	status = RescASN_CopyVS(IR_FILE, &(info->ipAddress),
	    &(targetInfo->ipAddress));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&(info->ipDomainName));
	RESC_ASN_INIT_VS(&(info->ipDomainName));
	status = RescASN_CopyVS(IR_FILE, &(info->ipDomainName),
	    &(targetInfo->ipDomainName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	info->portNum = targetInfo->portNum;


	OpTm_TargetInfoFree(targetInfo);
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_TargetAssocInfoFree 
 *
 * PURPOSE:
 *	This function is used to free target assoc info reply.
 *
 * INPUT:
 *	IR_TARGET_ASSOC_INFO	pointer to reply structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_TargetAssocInfoFree(IR_TARGET_ASSOC_INFO *reply)
{
	if (reply != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &(reply->applCntxName));
		RescASN_DestroyVS(IR_FILE, &(reply->applProcessTitle));
		RescASN_DestroyVS(IR_FILE, &(reply->applEntityQual));
		RescASN_DestroyVS(IR_FILE, &(reply->ipAddress));
		RescASN_DestroyVS(IR_FILE, &(reply->ipDomainName));

		if (RESC_ASN_IS_MALLOCED(reply))
		{
			RescIR_Free(IR_FILE, (CHAR *)reply);
		}
	}
}

/****************************************************************************
 * FUNCTION: OpTm_TargetGenInfoFree 
 *
 * PURPOSE:
 *	This function is used to free target info reply.
 *
 * INPUT:
 *	IR_TARGET_INFO	pointer to reply structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_TargetGenInfoFree(IR_TARGET_INFO *reply)
{
	if (reply != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &(reply->targetName));
		RescASN_DestroyVS(IR_FILE, &(reply->description));
		RescASN_DestroyBS(IR_FILE, &(reply->options));
		RescASN_DestroyVS(IR_FILE, &(reply->contactInfo));
		RescASN_DestroyVS(IR_FILE, &(reply->chargingPolicy));
		RescList_Destroy(IR_FILE, &(reply->dbInfo));
		RescASN_DestroyVS(IR_FILE, &(reply->idAuthentication));
		if (RESC_ASN_IS_MALLOCED(reply))
		{
			RescIR_Free(IR_FILE, (CHAR *)reply);
		}
	}
}

/****************************************************************************
 * FUNCTION: OpTm_GetTargetInfo 
 *
 * PURPOSE:
 *	This function is used to return user information.
 *
 * INPUT:
 *	IR_TARGET_ID		targetId
 *	VISIBLE_STRING		target info file name	
 *
 * OUTPUT:
 *	OP_TARGET_INFO_STRUCT	*pointer to info structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_GetTargetInfo(IR_TARGET_ID *targetId, 
			VISIBLE_STRING	targetInfoFileName,
			RESC_IO_FILE_STRUCT *targetFilePtr,
			OP_TARGET_INFO_STRUCT **info)
{
	IR_STATUS	status;
	VISIBLE_STRING		*record;
	VISIBLE_STRING	*key;
	CHAR		buf[RESC_IO_KEY_LENGTH+1];

	if (targetId == NULL)
	{
		key = NULL;
	}
	else
	{
		/* Create key from target id */
		key = RescASN_CreateVS(IR_FILE);
		if (key == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
			    IR_FILE);
			return (IR_FAIL);
		}
		sprintf(buf, "%d", *targetId);
		status = RescASN_AddVS(IR_FILE, key, buf, strlen(buf));	
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
			    IR_FILE);
			return (IR_FAIL);
		}
	}

	/* Get all info for target */
	status = RescIO_OpenFile(IR_FILE, OpTm_WorkPath(), 
	    targetInfoFileName, targetFilePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_OPEN_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	
	*info = NULL;
	/* Seek to record */
	status = RescIO_Seek(IR_FILE, targetFilePtr, 
	    RESC_IO_TARGET_INFO, key, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	else if (RescIO_EOF(*targetFilePtr))
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_USER_UNKNOWN,
		    IR_FILE);
		return (IR_FAIL);
	}
	if (key != NULL)
	{
		RescASN_DestroyVS(IR_FILE, key);
	}

	/* Get record */
	status = RescIO_GetRecord(IR_FILE, *targetFilePtr, &record);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	status = OpTm_ConvertRecordToTargetInfo(record, info);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, record);

}

/****************************************************************************
 * FUNCTION: OpTm_GetNextTargetInfo 
 *
 * PURPOSE:
 *	This function is used to return user information.
 *
 * INPUT:
 *	RESC_IO_FILE_STRUCT	pointer to file
 *
 * OUTPUT:
 *	OP_TARGET_INFO_STRUCT	*pointer to info structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_GetNextTargetInfo(RESC_IO_FILE_STRUCT *targetFilePtr,
				OP_TARGET_INFO_STRUCT **info)
{
	IR_STATUS	status;
	VISIBLE_STRING		*record;

	*info = NULL;
	/* Seek to start of next record */
	status = RescIO_Seek(IR_FILE, targetFilePtr, 
	    RESC_IO_TARGET_INFO, NULL, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	if (!RescIO_EOF(*targetFilePtr))
	{
		status = RescIO_GetRecord(IR_FILE, *targetFilePtr, &record);
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
			    IRO_FILE_READ_ERROR, IR_FILE);
			return (IR_FAIL);
		}

		status = OpTm_ConvertRecordToTargetInfo(record, info);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
		RescASN_DestroyVS(IR_FILE, record);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_CloseTargetInfoFile 
 *
 * PURPOSE:
 *	This function is used to close a target info file.
 *
 * INPUT:
 *	RESC_IO_FILE_STRUCT	pointer to file 
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_CloseTargetInfoFile(RESC_IO_FILE_STRUCT *targetFilePtr)
{
	IR_STATUS status;

	/* Close user info file */
	status = RescIO_CloseFile(IR_FILE, targetFilePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_TargetInfoFree 
 *
 * PURPOSE:
 *	This function is used to free target info returned by GetTargetInfo.
 *
 * INPUT:
 *	OP_TARGET_INFO_STRUCT	pointer to reply structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_TargetInfoFree(OP_TARGET_INFO_STRUCT *info)
{
	if (info != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &(info->applCntxName));
		RescASN_DestroyVS(IR_FILE, &(info->applProcessTitle));
		RescASN_DestroyVS(IR_FILE, &(info->applEntityQual));
		RescASN_DestroyVS(IR_FILE, &(info->ipAddress));
		RescASN_DestroyVS(IR_FILE, &(info->ipDomainName));
		RescASN_DestroyVS(IR_FILE, &(info->targetName));
		RescASN_DestroyVS(IR_FILE, &(info->description));
		RescASN_DestroyBS(IR_FILE, &(info->options));
		RescASN_DestroyVS(IR_FILE, &(info->contactInfo));
		RescASN_DestroyVS(IR_FILE, &(info->chargingPolicy));
		RescASN_DestroyVS(IR_FILE, &(info->defPreferedRecSyntax));
		RescASN_DestroyVS(IR_FILE, &(info->displayFormats));
		if (RESC_LIST_SIZE(&(info->dbInfo)) != 0)
		{
			RescList_Destroy(IR_FILE, &(info->dbInfo));
		}
		RescASN_DestroyVS(IR_FILE, &(info->idAuthentication));
		RescASN_DestroyVS(IR_FILE, &(info->userAuthentication));
		RescASN_DestroyVS(IR_FILE, &(info->defElementSetName));
		if (info->isMalloced)
		{
			RescIR_Free(IR_FILE, (CHAR *)info);
		}
	}
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertRecordToTargetInfo 
 *
 * PURPOSE:
 *	This function is used to convert a record to the target info struct.
 *
 * INPUT:
 *	VISIBLE_STRING	pointer to the record 
 *
 * OUTPUT:
 *	IR_USER_TARGET_INFO 	pointer to pointer to info	
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertRecordToTargetInfo(VISIBLE_STRING *record, 
			OP_TARGET_INFO_STRUCT **infoPtr)
{
	OP_TARGET_INFO_STRUCT	*info;
	IR_STATUS		status;
	CHAR			*elem;
	CHAR			*buf;
	USHORT			tag;
	ULONG			len;
	BOOLEAN			recEnd;
	ULONG			ePtr;

	/* Allocate space for info structure */
	*infoPtr = NULL;
	info = (OP_TARGET_INFO_STRUCT *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(OP_TARGET_INFO_STRUCT));
	if (info == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
	 	   IRO_MEMORY_ERROR, IR_FILE);
	 	   return (IR_FAIL);
	}
	RESC_ASN_MALLOCED(info);
	OpTm_TargetInfoInit(info);
	*infoPtr = info;

	/* Get first tag from record */
	ePtr = 0;
	buf = OpTm_GetElem(*record, 0, RESC_IO_TAG_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	tag = (USHORT)atoi(buf);

	/* Get length of first element in record */
	buf = OpTm_GetElem(*record, RESC_IO_TAG_LENGTH, 
	    RESC_IO_LEN_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	len = atol(buf);

	/* Get first element from record */
	elem = OpTm_GetElem(*record, 
	    (RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), len);
	if (elem == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}

	/* While not end of record */
	recEnd = FALSE;
	while (!recEnd)
	{
		/* Copy element to info structure */
		switch (tag)
		{
			case OP_TRGT_ID:
				info->targetId = (USHORT)atoi(elem);
				break;
			case OP_TRGT_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->targetName), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_USER_AUTH:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->userAuthentication), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_UPR_BND:
				info->defSmallSetUpperBound =
				    (LONG)atol(elem);
				break;
			case OP_LWR_BND:
				info->defLargeSetLowerBound = 
				    (LONG)atol(elem);
				break;
			case OP_PRSNT_NUM:
				info->defPresentNumber = (LONG)atol(elem);
				break;
			case OP_REC_SYN:
				status = RescASN_AddOI(IR_FILE, 
				    &(info->defPreferedRecSyntax), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_SET_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->defElementSetName), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_DSPLY_FMT:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->displayFormats), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_DESC:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->description), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_OPTIONS:
				OpTm_HexStrToByteStr(elem);
				status = RescASN_AddBS(IR_FILE, 
				    &(info->options), elem, 7);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_CNTCT_INFO:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->contactInfo), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_RSLT_SET:
				info->defResultSetNamingSupported = 
				    (BOOLEAN)atoi(elem);
				break;
			case OP_MAX_RSLT_SET:
				info->maxNumResultSets = (SHORT)atoi(elem);
				break;
			case OP_HRS_OP:
				info->hoursOfOperation = (SHORT)atoi(elem);
				break;
			case OP_CHRG_POLICY:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->chargingPolicy), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_ID_AUTH:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->idAuthentication), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_CNTX_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->applCntxName), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_PRCSS_TITLE:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->applProcessTitle), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_ENTTY_QUAL:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->applEntityQual), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_IP_ADDR:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->ipAddress), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_IP_DMN_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &(info->ipDomainName), elem, (SHORT)len);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				break;
			case OP_PORT_NUM:
				info->portNum = (USHORT)atoi(elem);
				break;
			default:
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_DATA_ERROR, IR_FILE);
				CS_LOG_PRINTV(IR_FILE, "Unknown Tag:  %d\n",
				    tag);
		}
	
		/* Get next tag in record */
		ePtr = ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH + len;
		buf = OpTm_GetElem(*record, ePtr, RESC_IO_TAG_LENGTH);
		if (buf == NULL)
		{
			recEnd = TRUE;
		}
		else
		{
			/* If not end of record, get next element */
			tag = (USHORT)atoi(buf);
			if (buf == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
			        return (IR_FAIL);
			}
			buf = OpTm_GetElem(*record, (ePtr + RESC_IO_TAG_LENGTH),
			    RESC_IO_LEN_LENGTH);
			if (buf == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
			        return (IR_FAIL);
			}
			len = atol(buf);
			elem = OpTm_GetElem(*record, 
			    (ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), 
			    len);
		}
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertTargetInfoToRecord 
 *
 * PURPOSE:
 *	This function is used to convert target info to a record.
 *
 * INPUT:
 *	OP_TARGET_INFO_STRUCT 	pointer to pointer to record	
 *
 * OUTPUT:
 *	VISIBLE_STRING	pointer to the target info 
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertTargetInfoToRecord(OP_TARGET_INFO_STRUCT *info,
				VISIBLE_STRING **recordPtr)
{
	VISIBLE_STRING		*record;
	IR_STATUS		status;
	CHAR			buf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			tmpBuf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			*rec;

	/* Create record structure */
	record = RescASN_CreateVS(IR_FILE);
	if (record == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}
	*recordPtr = record;
	rec = buf;

	/* Concatenate each element of info structure to a buffer */
	sprintf(tmpBuf, "%d", info->targetId);
	OpTm_PutElem(rec, OP_TRGT_ID, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	OpTm_PutElem(rec, OP_TRGT_NAME, info->targetName.sPtr, 
	    info->targetName.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_USER_AUTH, info->userAuthentication.sPtr, 
	    info->userAuthentication.len, (ULONG *)&rec);

	sprintf(tmpBuf, "%ld", info->defSmallSetUpperBound);
	OpTm_PutElem(rec, OP_UPR_BND, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	sprintf(tmpBuf, "%ld", info->defLargeSetLowerBound);
	OpTm_PutElem(rec, OP_LWR_BND, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	sprintf(tmpBuf, "%ld", info->defPresentNumber);
	OpTm_PutElem(rec, OP_PRSNT_NUM, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	OpTm_PutElem(rec, OP_REC_SYN, info->defPreferedRecSyntax.sPtr, 
	    info->defPreferedRecSyntax.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_SET_NAME, info->defElementSetName.sPtr, 
	    info->defElementSetName.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_DSPLY_FMT, info->displayFormats.sPtr, 
	    info->displayFormats.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_DESC, info->description.sPtr, 
	    info->description.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_OPTIONS, info->options.sPtr, 
	    info->options.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_CNTCT_INFO, info->contactInfo.sPtr, 
	    info->contactInfo.len, (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->defResultSetNamingSupported);
	OpTm_PutElem(rec, OP_RSLT_SET, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->maxNumResultSets);
	OpTm_PutElem(rec, OP_MAX_RSLT_SET, tmpBuf, strlen(tmpBuf), 
	    (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->hoursOfOperation);
	OpTm_PutElem(rec, OP_HRS_OP, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	OpTm_PutElem(rec, OP_CHRG_POLICY, info->chargingPolicy.sPtr, 
	    info->chargingPolicy.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_ID_AUTH, info->idAuthentication.sPtr, 
	    info->idAuthentication.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_CNTX_NAME, info->applCntxName.sPtr, 
	    info->applCntxName.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_PRCSS_TITLE, info->applProcessTitle.sPtr, 
	    info->applProcessTitle.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_ENTTY_QUAL, info->applEntityQual.sPtr, 
	    info->applEntityQual.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_IP_ADDR, info->ipAddress.sPtr, 
	    info->ipAddress.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_IP_DMN_NAME, info->ipDomainName.sPtr, 
	    info->ipDomainName.len, (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->portNum);
	OpTm_PutElem(rec, OP_PORT_NUM, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	/* Copy buffer to record */
	status = RescASN_AddVS(IR_FILE, record, buf, strlen(buf));	
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_TargetInfoInit 
 *
 * PURPOSE:
 *	This function is used to initialize target information.
 *
 * INPUT:
 *	OP_TARGET_INFO_STRUCT 	pointer to structure	
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpTm_TargetInfoInit(OP_TARGET_INFO_STRUCT *info)
{
	
	RESC_ASN_NO_MALLOC(&(info->targetName));
	RESC_ASN_INIT_OI(&(info->targetName));
	RESC_ASN_NO_MALLOC(&(info->userAuthentication));
	RESC_ASN_INIT_OI(&(info->userAuthentication));
	RESC_ASN_NO_MALLOC(&(info->defPreferedRecSyntax));
	RESC_ASN_INIT_OI(&(info->defPreferedRecSyntax));
	RESC_ASN_NO_MALLOC(&(info->defElementSetName));
	RESC_ASN_INIT_VS(&(info->defElementSetName));
	RESC_ASN_NO_MALLOC(&(info->displayFormats));
	RESC_ASN_INIT_VS( &(info->displayFormats));
	RESC_ASN_NO_MALLOC(&(info->description));
	RESC_ASN_INIT_VS(&(info->description));
	RESC_ASN_NO_MALLOC(&(info->options));
	RESC_ASN_INIT_BS(&(info->options));
	RESC_ASN_NO_MALLOC(&(info->contactInfo));
	RESC_ASN_INIT_VS(&(info->contactInfo));
	RESC_ASN_NO_MALLOC(&(info->chargingPolicy));
	RESC_ASN_INIT_VS(&(info->chargingPolicy));
	RESC_ASN_NO_MALLOC(&(info->idAuthentication));
	RESC_ASN_INIT_VS(&(info->idAuthentication));
	RESC_ASN_NO_MALLOC(&(info->applCntxName));
	RESC_ASN_INIT_VS(&(info->applCntxName));
	RESC_ASN_NO_MALLOC(&(info->applProcessTitle));
	RESC_ASN_INIT_VS(&(info->applProcessTitle));
	RESC_ASN_NO_MALLOC(&(info->applEntityQual));
	RESC_ASN_INIT_VS(&(info->applEntityQual));
	RESC_ASN_NO_MALLOC(&(info->ipAddress));
	RESC_ASN_INIT_VS(&(info->ipAddress));
	RESC_ASN_NO_MALLOC(&(info->ipDomainName));
	RESC_ASN_INIT_VS(&(info->ipDomainName));

	RESC_LIST_INIT(&(info->dbInfo));
	RESC_ASN_NO_MALLOC(&(info->dbInfo));
}

static IR_TARGET_ASSOC_INFO *optm_AssocInfoCreate()
{
	IR_TARGET_ASSOC_INFO *new;

	new = (IR_TARGET_ASSOC_INFO *)RescIR_Zmalloc(IR_FILE, 
	   sizeof(IR_TARGET_ASSOC_INFO));

	if (new == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
			    IRO_MEMORY_ERROR, IR_FILE);
		return (new);
	}

	RESC_ASN_MALLOCED(new);
	new->targetId = IR_UNKNOWN_ID_TYPE;
	RESC_ASN_INIT_VS(&new->applCntxName);
	RESC_ASN_NO_MALLOC(&new->applCntxName);
	RESC_ASN_INIT_VS(&new->applProcessTitle);
	RESC_ASN_NO_MALLOC(&new->applProcessTitle);
	RESC_ASN_INIT_VS(&new->applEntityQual);
	RESC_ASN_NO_MALLOC(&new->applEntityQual);
	RESC_ASN_INIT_VS(&new->ipAddress);
	RESC_ASN_NO_MALLOC(&new->ipAddress);
	RESC_ASN_INIT_VS(&new->ipDomainName);
	RESC_ASN_NO_MALLOC(&new->ipDomainName);
	new->portNum = 0;

	return(new);
}

static IR_TARGET_INFO *optm_GenInfoCreate()
{
	IR_TARGET_INFO *new;

	new = (IR_TARGET_INFO *)RescIR_Zmalloc(IR_FILE, sizeof(IR_TARGET_INFO));

	if (new == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		return (new);
	}

	RESC_ASN_MALLOCED(new);
	new->targetId = IR_UNKNOWN_ID_TYPE;
	RESC_ASN_NO_MALLOC(&new->targetName);
	RESC_ASN_INIT_VS(&new->targetName);
	RESC_ASN_NO_MALLOC(&new->description);
	RESC_ASN_INIT_VS(&new->description);
	RESC_ASN_NO_MALLOC(&new->options);
	RESC_ASN_INIT_BS(&new->options);
	RESC_ASN_NO_MALLOC(&new->contactInfo);
	RESC_ASN_INIT_VS(&new->contactInfo);
	new->resultSetNamingSupported = FALSE;
	new->elementSetNamingSupported = FALSE;
	new->maxNumResultSets = 0;
	new->hoursOfOperation = 0;
	RESC_ASN_NO_MALLOC(&new->chargingPolicy);
	RESC_ASN_INIT_VS(&new->chargingPolicy);
	RESC_ASN_NO_MALLOC(&new->dbInfo);
	RESC_LIST_INIT(&new->dbInfo);
	RESC_ASN_NO_MALLOC(&new->idAuthentication);
	RESC_ASN_INIT_VS(&new->idAuthentication);

	return(new);
}


void OpTm_HexStrToByteStr(CHAR *buf)
{
	CHAR	c;
	USHORT	d, i, j, n;

	j = 0;
	for (i = 0; i < strlen(buf); i++)
	{
		c = buf[i];
		d = (USHORT)(isdigit(c) ? c - '0' : toupper(c) - 'A' + 0x0A);
		if (i % 2)
		{
			n = n + d;
			buf[j++] = (CHAR)n;
		}
		else
		{
			n = d * 0x10;
			buf[j] = (CHAR)n;
		}
	}
}
