
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpWs  Work Space Delete
 *
 * DESCRIPTION:
 *	This unit provides the functions required to support the
 *	IRO UA/OP API Delete functions for both local and remote
 *	delete operations. Also, the remote delete response message
 *	is processed by this unit.
 *
 * CONTENTS:
 *	IRO_Delete		- IRO interface function.
 *	OpWs_DeleteService	- Remote delete message processing
 *	OpWs_DeleteServiceFree  - Delete response free function.
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>
#include <asn_tool.h>


static IR_STATUS OpWs_DeleteTargetResults(IRO_DELETE_REQ *req);
static IR_STATUS OpWs_DeleteTargetWsList(IR_LIST *, IR_ASSOC_ID);
static IR_STATUS OpWs_DeleteTargetAll(IR_ASSOC_ID assocId);
static IR_DELETE_STATUS_STRUCT 	*OpWs_CreateDelStatStruct();
static void OpWs_DestroyDelStatStruct(CHAR *src, void *data);
static void *OpWs_CopyDelStatStruct(CHAR *src, void *data);

/****************************************************************************
 * FUNCTION:  IRO_Delete
 *
 * PURPOSE:
 *	This function supports all of the IRO Delete operations. These
 *	include all of the following:
 *
 *	IRO_DELETE_WS_REC_ALL		- Delete all ws records from
 *					  a specified work space but
 *					  leave the work space intact.
 *	IRO_DELETE_WS_REC_SPEC		- Delete a specific ws record for
 *					  in the specified work space.
 *	IRO_DELETE_TARGET_RSLTS		- Delete target result set and remove
 *					  the reference to the remote result
 *					  set from the corresponding work
 *					  work space.
 *	IRO_DESTROY_WS			- Destroy the  workspace and all 
 *					  records in the work space.
 *
 *
 *
 * INPUT:
 *	IR_USER_ID
 *	IRO_DELETE_TYPE
 *	IRO_DELETE_REQ
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_Delete(IR_USER_ID userId, 
		    IRO_DELETE_TYPE reqType,
		    IRO_DELETE_REQ *req)
{

	OP_SESSION_INFO	*sessionInfo;
	OP_WS_INFO	*ws;

CS_LOG_PRINTV(IR_FILE, "IRO_Delete info. Type = %d\n",
    reqType);

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_UNKNOWN_USER_ID, IR_FILE);
		return (IR_FAIL);
	}

	switch(reqType)
	{
	case IRO_DELETE_WS_REC_ALL:
		/* Get the work space */
		if ((ws = OpWs_WsGet(req->wsId)) == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    	    IRO_WORKSPACE_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}
		RescList_Destroy(IR_FILE, &ws->records);
		return(IR_SUCCESS);
		break;
	case IRO_DELETE_TARGET_RSLTS:
		return(OpWs_DeleteTargetResults(req));
		break;
	case IRO_DESTROY_WS:
		/* Get the work space */
		if ((ws = OpWs_WsGet(req->wsId)) == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    	    IRO_WORKSPACE_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}
		else
		{
			OpWs_WsDelete(req->wsId);
			return(IR_SUCCESS);
		}
		break;
			
	default:
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_UNSUPPORTED_REQ_TYPE ,IR_FILE);
		return(IR_FAIL);
	}

}


/****************************************************************************
 * LOCAL FUNCTION:  OpWs_DeleteTargetResults
 *
 * PURPOSE:
 *	This function is used to perform user application delete requests
 *	aimed at the specified target. There are two types of delete
 *	requests that can be sent to the target system:
 *		a) Bulk delete	- ie. delete all known result sets
 *		b) list delete  - ie. delete specific result sets on the
 *				      target. The result sets are specified
 *				      in a list to the target.
 *
 *	The origin system maintains work spaces for all search transactions.
 *	It is the responsibility of this unit to ensure that if a result set
 *	is deleted on the target then the workspace corresponding to that
 *	result set will be dereferenced from the target. The work space on
 *	the origin system will still exist but will not be attached to any
 *	target. 
 *
 *	If the delete request is a list delete then this function will 
 *	use the specified wsIds to obtain a list of result set names used
 *	by the target system. The list of wsIds and names will be saved in
 *	the association information table so that the SR delete response can
 *	verify the delete request and unlink the origin work spaces from their
 *	now non-existent target result sets. After the list is created the
 *	OpWs_DeleteTargetWsList function will be invoked to generate the delete
 *	message and send it to the origin kernel.
 *
 *	If the delete request is a bulk delete then the OpWs_DeleteTargetAll
 *	function will be invoked to generate the delete message and send it
 *	to the origin kernel.
 *
 * INPUT:
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static IR_STATUS OpWs_DeleteTargetResults(IRO_DELETE_REQ *req)
{
	SHORT		index;
	SHORT		index1;
	OP_WS_INFO	*wsPtr;
	IRO_WORK_SPACE_ID *wsId;
	IR_WS_ID	*wsIdArray;
	IR_LIST		*resultSetList;
	IR_DELETE_STATUS_STRUCT *newDelStr;

	switch(req->deleteFunc)
	{
	case IR_DELETE_SPECIFIC_SET:

		/*
		 * Check for specified list.
		 */
		if (RESC_LIST_SIZE(&req->wsList) == 0)
		{
			/* No list. No Delete. Return. */
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_NO_DELETE_LIST_SPECIFIED, IR_FILE);
			return(IR_FAIL);
		}

		/* 
		 * Find all ws entries belonging to the specified association.
		 */
CS_LOG_PRINTV(IR_FILE, "AssocId in delete is %d\n", req->assocId);
		wsIdArray = OpWs_WsGetAllAssoc(req->assocId);
CS_LOG_PRINTS(IR_FILE, "Got all assocs \n");


		/* 
		 * Generate a list of work space id and target result set
		 * ids that can be saved in the association table until
		 * the delete reply is received.
		 */

		resultSetList = RescList_Create(IR_FILE);
CS_LOG_PRINTS(IR_FILE, "Created list for target result set ids\n");

		index = 0;
		while (wsIdArray[index] != IR_UNKNOWN_ID_TYPE)
		{
CS_LOG_PRINTV(IR_FILE, "wsId in delete = %d\n", wsIdArray[index]);

			/* 
			 * Try and match the work space with the 
			 * ids specified in the request.
			 */
		       wsPtr = OpWs_WsGet(wsIdArray[index]);
			RESC_LIST_SET_FIRST(&req->wsList);
			for (index1 = 0; 
			    index1 < RESC_LIST_SIZE(&req->wsList);
			    index1++)
			{
				wsId = RESC_LIST_GET(&req->wsList);
CS_LOG_PRINTV(IR_FILE, "wsId in delete request = %d\n", *wsId);
				if ((wsPtr->wsId == *wsId) && 
				    (wsPtr->targetResultsDeleted == FALSE))
				{
					newDelStr = OpWs_CreateDelStatStruct();
					newDelStr->wsId = wsPtr->wsId;
					RescASN_CopyVS(IR_FILE,
					    &newDelStr->resultSetId,
					    &wsPtr->resultSetName);
					OpWs_WsSetDeleteFlag(wsPtr->wsId,
					    TRUE);

					/* 
					 * Save the wsId and resultSet name
					 * in the result set list.
					 */
					RescList_Append(IR_FILE, resultSetList,
					    newDelStr, 
					    (void *)OpWs_DestroyDelStatStruct,
					    (void *)OpWs_CopyDelStatStruct);

					/* Get next wsId */
					break;
				}
				else
				{
					/* Try next  entry in supplied list */
					RESC_LIST_SET_NEXT(&req->wsList);
				}
			}

			/* Get next wsId */
			index++;
		}

		if (RESC_LIST_SIZE(resultSetList) == 0)
		{
			CS_LOG_PRINTS(IR_FILE, "No target/wsId match\n");
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    	    IRO_NO_RESULTS_FOR_DELETE, IR_FILE);
			/* Free the wsId list */
			RescIR_Free(IR_FILE, (CHAR *)wsIdArray);

			return (IR_FAIL);
		}

		/* Free the wsId list */
		RescIR_Free(IR_FILE, (CHAR *)wsIdArray);
		
		/* Generate the delete request and send it */
		return(OpWs_DeleteTargetWsList(resultSetList, req->assocId));
		
	case IR_DELETE_ALL_SETS:
		/* Generate the delete request and send it */
		return(OpWs_DeleteTargetAll(req->assocId));
		break;
	default:
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_UNKNOWN_DELETE_TYPE,IR_FILE);
		return(IR_FAIL);
	}
}


/****************************************************************************
 * LOCAL FUNCTION:  OpWs_DeleteTargetWsList
 *
 * PURPOSE:
 *	This function will generate an SR delete request and send it to the
 *	specified target. The delete request will be generated from the list
 *	of result set names provided. Also, the list of result set names will
 *	be stored in the associations entry table so that the delete response
 *	can verify against the request.
 *
 * INPUT:
 *	IR_LIST - list of IR_DELETE_STATUS_STRUCT
 *	IR_ASSOC_ID - identifier of target
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	If for any reason the delete request cannot be sent to the target
 *	then this function is responsible for destroying the IR_LIST provided
 *	in the input parameters.
 *
 ***************************************************************************/

static IR_STATUS OpWs_DeleteTargetWsList(IR_LIST *wsList, IR_ASSOC_ID assocId)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IRK_SEND_REQUEST_REQ delReq;
	DeleteResultSetReq *del;
	AsnOcts		*octs;
	IR_STATUS	status;
	IR_DELETE_STATUS_STRUCT *delItem;

	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST,IR_FILE);
		RescList_Destroy(IR_FILE, wsList);
		return(IR_FAIL);
	}

	/* Check to see whether the association is busy or not */
	if (assocEntry->state != OP_ASSOC_IDLE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_BUSY, IR_FILE);
		RescList_Destroy(IR_FILE, wsList);
		return(IR_FAIL);
	}

	/* Check to see if the operation is supported */
	if (OpAm_AssocServiceAllowed(OPAM_DELETE, assocId) 
	    == FALSE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_REQ_SERVICE_NOT_SUPPORTED, IR_FILE);
		RescList_Destroy(IR_FILE, wsList);
		return(IR_FAIL);
	}
	/* Initialize APDU */

	/* Create the delete request */
	delReq.irApdu.choiceId = APDU_DELETERESULTSETREQ;
	delReq.irApdu.a.deleteResultSetReq = (DeleteResultSetReq *)
	    RescIR_Zmalloc(IR_FILE, sizeof(DeleteResultSetReq));

	del = delReq.irApdu.a.deleteResultSetReq;

	/* Create the reference Id based on the assocId */
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, (SHORT)assocId);
	RescASN_IRToSnaccOS(IR_FILE, &del->referenceId, 
	    &assocEntry->wsRefId);
	
	del->deleteOperation = LIST;

	del->resultSetList = ASN_ListCreate(sizeof(AsnOcts));
	
	RESC_LIST_SET_FIRST(wsList);
	while (RESC_LIST_CUR(wsList))
	{
		delItem = RESC_LIST_GET(wsList);
		RescUT_DebugVS(IR_FILE, &delItem->resultSetId);
		octs = ASN_OctsCreate();
		RescASN_IRToSnaccVS(IR_FILE, octs, &delItem->resultSetId);
		ASN_ListAppend(del->resultSetList, octs);
		RESC_LIST_SET_NEXT(wsList);
	}
	
	status = IRK_SendRequest(IRK_DELETE_REQ_TYPE,
	    assocId, &delReq);

	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(delReq.irApdu));
		RescList_Destroy(IR_FILE, wsList);
		return (IR_FAIL);
	}	

	assocEntry->state = OP_ASSOC_DELETING;
	assocEntry->resultList = wsList;
	assocEntry->delFunc = IR_DELETE_SPECIFIC_SET;

	/* Destroy the apdu. */
	FreeAPDU(&(delReq.irApdu));

	/* Done.  */
	return(IR_SUCCESS);
}


/****************************************************************************
 * LOCAL FUNCTION:  OpWs_DeleteTargetAll
 *
 * PURPOSE:
 *	This function is responsible for generating an SR delete request
 *	for a bulk delete and sending it to the specified target.
 *
 * INPUT:
 *	IR_ASSOC_ID
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static IR_STATUS OpWs_DeleteTargetAll(IR_ASSOC_ID assocId)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IRK_SEND_REQUEST_REQ delReq;
	IR_STATUS	status;
	DeleteResultSetReq *del;

	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST,IR_FILE);
		return(IR_FAIL);
	}

	/* Check to see whether the association is busy or not */
	if (assocEntry->state != OP_ASSOC_IDLE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_BUSY, IR_FILE);
		return(IR_FAIL);
	}

	/* Initialize APDU */
	/* Create the delete request */
	delReq.irApdu.choiceId = APDU_DELETERESULTSETREQ;
	delReq.irApdu.a.deleteResultSetReq = (DeleteResultSetReq *)
	    RescIR_Zmalloc(IR_FILE, sizeof(DeleteResultSetReq));

	del = delReq.irApdu.a.deleteResultSetReq;

	/* Initialize the apdu */

	/* Create the reference Id based on the assocId */
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, (SHORT)assocId);
	RescASN_IRToSnaccOS(IR_FILE, &del->referenceId, 
	    &assocEntry->wsRefId);

	del->deleteOperation = ALL;

	status = IRK_SendRequest(IRK_DELETE_REQ_TYPE,
	    assocId, &delReq);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(delReq.irApdu));
		return (IR_FAIL);
	}	

	assocEntry->state = OP_ASSOC_DELETING;
	assocEntry->resultList = (IR_LIST *)NULL;
	assocEntry->delFunc = IR_DELETE_ALL_SETS;

	/* Destroy the apdu */
	FreeAPDU(&(delReq.irApdu));

	return(IR_SUCCESS);

}



/****************************************************************************
 * FUNCTION:  OpWs_DeleteService
 *
 * PURPOSE:
 *	This function is responsible for processing the SR delete response
 *	message received from a target.
 *	If the message corresponds to a delete list request then this 
 *	function will verify the delete status for each result set against
 *	the stored list used to generate the delete request. The work space
 *	will be updated to reflect the outcome of each delete (both bulk and
 *	specified list) as well as being added to the IRO delete response
 *	message.
 *
 * INPUT:
 *	IR_ASSOC_ID	- target identifier
 *	APDU		- received message
 *	IRO_COMM_DEL_RESP - IRO delete response message to be supplied to
 *			    the user application.
 *
 * OUTPUT:
 *	IRO_COMM_DEL_RESP - completed response message.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS OpWs_DeleteService(IR_ASSOC_ID assocId, APDU *apdu,
	    IRO_COMM_DEL_RESP *delResp)
{

	OP_ASSOC_ENTRY	*assocEntry;
	DeleteResultSetResp *apduResp;
	ListStatusesSeq		*statElem;
	IR_DELETE_STATUS_STRUCT *delStatStruct;
	IR_DELETE_STATUS_STRUCT *delReqStruct;
	IR_DELETE_SET_FUNCTION	delFunc;
	SHORT			index;
	SHORT			index1;
	OP_WS_INFO		*wsInfo;
	VISIBLE_STRING		*refId;
	IR_LIST			*delReqList;
	IR_WS_ID		*wsIdArray;

	RESC_ASN_NO_MALLOC(delResp);

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Received delete response for unknown association\n");
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		FreeAPDU(apdu);
		return(IR_FAIL);
	}

	assocEntry->state = OP_ASSOC_IDLE;

	/* Extract the result set list from the assoc entry */
	delReqList = assocEntry->resultList;
	delFunc = assocEntry->delFunc;
	assocEntry->resultList = (IR_LIST *)NULL;
	assocEntry->delFunc = IR_UNKNOWN_ID_TYPE;

	/* Prepare the IRO delete response message */
	RESC_ASN_NO_MALLOC(&delResp->deleteMessage);
	RESC_ASN_INIT_VS(&delResp->deleteMessage);
	RESC_ASN_NO_MALLOC(&delResp->statusList);
	RESC_LIST_INIT(&delResp->statusList);
	RESC_ASN_NO_MALLOC(&delResp->bulkStatusList);
	RESC_LIST_INIT(&delResp->bulkStatusList);


	/* Begin analizing the decoded delete response apdu */
	apduResp = apdu->a.deleteResultSetResp;

	delResp->deleteStatus = (IR_DELETE_SET_STATUS)apduResp->deleteOperStatus;

	/* Verify the referenceId against the one stored in assocEntry */
	refId = RescASN_CreateVS(IR_FILE);
	RescASN_SnaccToIRVS(IR_FILE, refId, &apduResp->referenceId);

	if (strncmp(refId->sPtr, assocEntry->wsRefId.sPtr, refId->len) != 0)
	{
		/* Reference Id does not correspond. Z3950 protocol no no! */
		CS_LOG_PRINTS(IR_FILE, "Mismatched referenceId on delete.\n");
	}

	/* Destroy the refId tmp variable */
	RescASN_DestroyVS(IR_FILE, refId);

	/* Reset the assocEntry wsRefId */
	RescASN_DestroyVS(IR_FILE, &assocEntry->wsRefId);

	/*
	 * Check for a list of deleteStatuses. This will be returned
	 * for delete requests that specify a list of result set ids
	 * to be deleted.
	 */
	if (apduResp->deleteListStatuses)
	{
		/* Copy the result set id and status to the IRO delResp */
		FOR_EACH_LIST_ELMT(statElem, apduResp->deleteListStatuses)
		{
			/* Convert each status and id in list to status list */
			delStatStruct = OpWs_CreateDelStatStruct();
			delStatStruct->resultSetStatus = 
			     (IR_DELETE_SET_STATUS)statElem->deleteSetStatus;
			RescASN_SnaccToIRVS(IR_FILE, 
			    &delStatStruct->resultSetId,
			    &statElem->resultSetId);
			RescList_Append(IR_FILE, &delResp->statusList,
			    delStatStruct, OpWs_DestroyDelStatStruct,
			    OpWs_CopyDelStatStruct);
		}
	}

	/* 
	 * Check for the number of result sets not deleted. This should
	 * only be returned by a target if a bulk delete could not complete.
 	 * This parameter is should only be returned if the delete operation
	 * status is failure-8.
	 */
	if (apduResp->numberNotDeleted)
	{
		delResp->numberNotDeleted = 
		     (LONG)*apduResp->numberNotDeleted;
	}
	else
	{
		delResp->numberNotDeleted = (LONG)IR_UNKNOWN_ID_TYPE;
	}
	
	/* 
 	 * Check for a list of result set ids and their delete status. This
	 * list will only be valid if a bulk delete could not delete all
	 * requested result sets.
 	 * This parameter is should only be returned if the delete operation
	 * status is failure-8.
	 */
	if (apduResp->bulkStatuses)
	{
		/* Copy the result set id and status to the IRO delResp */
		FOR_EACH_LIST_ELMT(statElem, apduResp->bulkStatuses)
		{
			/* Convert each status and id in list to status list */
			delStatStruct = OpWs_CreateDelStatStruct();
			delStatStruct->resultSetStatus = 
			     (IR_DELETE_SET_STATUS)statElem->deleteSetStatus;
			RescASN_SnaccToIRVS(IR_FILE, 
			    &delStatStruct->resultSetId,
			    &statElem->resultSetId);
			RescList_Append(IR_FILE, &delResp->bulkStatusList,
			    delStatStruct, OpWs_DestroyDelStatStruct,
			    OpWs_CopyDelStatStruct);
		}
	}

	/* Copy the optional deleteMessage to the IRO delResp structure. */
	RescASN_SnaccToIRVS(IR_FILE, &delResp->deleteMessage,
	    &apduResp->deleteMessage);

	/* 
	 * We have extracted all known information from the apdu. Free
	 * all memory associated with the apdu.
	 */
	FreeAPDU(apdu);
	

	/*
  	 * Check the delete request for the type of delete requested. 
	 */
	switch(delFunc)
	{
	case IR_DELETE_SPECIFIC_SET:
	
		/*
	 	 *  Check for lists to be present before trying to update
		 * the work space.
	 	 */
		if ((RESC_LIST_SIZE(&delResp->statusList) == 0) ||
	    	    (RESC_LIST_SIZE(delReqList) == 0))
		{
			CS_LOG_PRINTS(IR_FILE, 
			    "Some kinda delete protocol error.\n");
			/* Exit from switch */
			break;
		}

		/*
		 * match result set id in delete with result set id
		 * in request. Use the request wsId to update the
		 * work space entry with the delete status. (wsId 
	 	 * could not be sent to target so we do it this way.
		 * We will use the list of wsIds and resultset ids
		 * saved in the assocEntry.).
		 */
			
		RESC_LIST_SET_FIRST(&delResp->statusList);
		for (index = 0; 
		     index < RESC_LIST_SIZE(&delResp->statusList);
		     index++)
		{
			delStatStruct = RESC_LIST_GET(&delResp->statusList);

			RESC_LIST_SET_FIRST(delReqList);
			for (index1 = 0; index1 < RESC_LIST_SIZE(delReqList);
			    index1++)
			{
				delReqStruct = RESC_LIST_GET(delReqList);
				if (RescASN_CompareVS(IR_FILE,
				    &delReqStruct->resultSetId, 
				    &delStatStruct->resultSetId) == 0)
				{
					/* They matched */
					CS_LOG_PRINTS(IR_FILE,
					    "Deleted target result ");
					RescUT_DebugVS(IR_FILE,
					    &delReqStruct->resultSetId);

					if (delStatStruct->resultSetStatus 
				  	    == IR_DEL_SUCCESS)
					{
						OpWs_WsDeletedRemoteResult(
						    delReqStruct->wsId);
					}
					RescList_Delete(IR_FILE, delReqList, 
					    index1);
					break;
				}
			}

		}
		if (RESC_LIST_SIZE(delReqList) > 0)
		{
			CS_LOG_PRINTV(IR_FILE, "Not all requested result sets were specified in the delete list status. %d forgotten.\n",
			    RESC_LIST_SIZE(delReqList));
		}
		break;

	case IR_DELETE_ALL_SETS:

		/* Get a list of all work spaces for this association */
		wsIdArray = OpWs_WsGetAllAssoc(assocEntry->assocId);
		index = 0;
		while (wsIdArray[index] != IR_UNKNOWN_ID_TYPE)
		{
			if (delResp->numberNotDeleted == 0)
			{
				/* mark the work space as deleted */
				OpWs_WsDeletedRemoteResult(wsIdArray[index]);
			}
			else
			{
				/* 
				 * If the work space is in the bulk status
				 * list then it is not deleted. Do not 
				 * change the work space.
				 */
				wsInfo = OpWs_WsGet(wsIdArray[index]);
				
				/* Find the resultSetId that matched wsInfo */
				for (index1 = 0; index1 < 
				    RESC_LIST_SIZE(&delResp->bulkStatusList);
				    index1++)
				{
					delStatStruct = RESC_LIST_GET(
					    &delResp->bulkStatusList);
					/* Compare result set Ids */
					if (RescASN_CompareVS(IR_FILE,
					    &delStatStruct->resultSetId,
					    &wsInfo->resultSetName) == 0)
					{
						/* Not deleted */
						break;
					}
				}
				if (index1 == 
				    RESC_LIST_SIZE(&delResp->bulkStatusList))
				{
					/* WsId not in bulk list. Delete */
					OpWs_WsDeletedRemoteResult(
					    wsIdArray[index]);
					
				}
			}
			index++;
		}

		/* Free the list of wsIds that was malloced by WsGetAllAssoc */
		RescIR_Free(IR_FILE, (void *)wsIdArray);

		break;
	default:
		CS_LOG_PRINTS(IR_FILE, "Delete response received when delete not sent\n");
		break;
	}


	/* Free the delReqList */
	if (delReqList)
	{
		RescList_Destroy(IR_FILE, delReqList);
	}

	return(IR_SUCCESS);
}



IR_DELETE_STATUS_STRUCT 	*OpWs_CreateDelStatStruct()
{
	IR_DELETE_STATUS_STRUCT	*new;

	new = (IR_DELETE_STATUS_STRUCT *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IR_DELETE_STATUS_STRUCT));
	if (new == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Memory error. Gotta go!\n");
		return(new);
	}

	RESC_ASN_MALLOCED(new);
	RESC_ASN_NO_MALLOC(&new->resultSetId);
	RESC_ASN_INIT_VS(&new->resultSetId);
	new->resultSetStatus = -1;
	new->wsId = IR_UNKNOWN_ID_TYPE;
}

void OpWs_DestroyDelStatStruct(CHAR *src, void *data)
{
	IR_DELETE_STATUS_STRUCT *delStr;

	if (!data)
	{
		return;
	}
	
	delStr = (IR_DELETE_STATUS_STRUCT *)data;

	RescASN_DestroyVS(src, &delStr->resultSetId);
	if (RESC_ASN_IS_MALLOCED(delStr))
	{
		RescIR_Free(src, (CHAR *)delStr);
	}
}

void *OpWs_CopyDelStatStruct(CHAR *src, void *data)
{
	IR_DELETE_STATUS_STRUCT *delStr;
	IR_DELETE_STATUS_STRUCT *new;

	if (!data)
	{
		return(IR_DELETE_STATUS_STRUCT *)NULL;
	}
	
	delStr = (IR_DELETE_STATUS_STRUCT *)data;


	new = OpWs_CreateDelStatStruct();

	if (!new)
	{
		return(new);
	}

	RescASN_CopyVS(src, &new->resultSetId, &delStr->resultSetId);
	new->resultSetStatus = delStr->resultSetStatus;
	new->wsId = delStr->wsId;
	
	return(new); 
} 

void OpWs_DeleteServiceFree( IRO_COMM_DEL_RESP *delResp)
{

	if (!delResp)
	{
		return;
	}

	RescASN_DestroyVS(IR_FILE, &delResp->deleteMessage);
	RescList_Destroy(IR_FILE, &delResp->statusList);
	RescList_Destroy(IR_FILE, &delResp->bulkStatusList);

	if (RESC_ASN_IS_MALLOCED(delResp))
	{
		RescIR_Free(IR_FILE, (CHAR *)delResp);
	}
		
}
