
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpWs  Work Space Maintenance
 *
 * DESCRIPTION:
 *	This unit contains functions used for the maintenance of the
 *	work space entries which are maintained from the time that a
 *	search is sent until the work space is explicitly deleted by the
 *	user application or until the end of the user session.
 *
 * CONTENTS:
 *	OpWs_WsDeleteAssoc
 *	OpWs_WsNewResultName
 *	OpWs_WsDeletedRemoteResult
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>

extern IR_LIST	*OpWs_WsList;


/****************************************************************************
 * FUNCTION:  OpWs_WsDeleteAssoc
 *
 * PURPOSE:
 *	This function is used to remove all references of the provided
 *	association from the workspace. This includes things such as
 *	assocId and  result set name.
 *
 * INPUT:
 *	IR_ASSOC_ID
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsDeleteAssoc(IR_ASSOC_ID assocId)
{
	SHORT		index;
	OP_WS_INFO 	*wsInfo;

	RESC_LIST_SET_FIRST(OpWs_WsList);

	for (index = 0; index < RESC_LIST_SIZE(OpWs_WsList); index++)
	{
		wsInfo = RESC_LIST_GET(OpWs_WsList);
		if (wsInfo->assocId == assocId)
		{
			wsInfo->assocId = IR_UNKNOWN_ID_TYPE;
			RescASN_DestroyVS(IR_FILE, &wsInfo->resultSetName);
			wsInfo->totalTargetRecords = 0;
			wsInfo->nextTargetRec = 0;
			wsInfo->targetResultsDeleted = TRUE;
		}
		RESC_LIST_SET_NEXT(OpWs_WsList);
	}
}


/****************************************************************************
 * FUNCTION:  OpWs_WsNewResultName
 *
 * PURPOSE:
 *	This function will act kinda like a target system. It will use
 *	the input association id to find all work spaces related to a
 *	specific target. For each work space found it will apply the
 *	rules specified in Z39.50 about result set naming and the
 *	replace indicator to determine if a previous result set name
 *	should be deleted.
 *
 * INPUT:
 *	IR_ASSOC_ID
 *	VISIBLE_STRING
 *	BOOLEAN	replaceIndicator.
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes. 
 *
 ***************************************************************************/

void OpWs_WsNewResultName(IR_ASSOC_ID assocId, VISIBLE_STRING *name,
    BOOLEAN replaceIndicator)
{
	SHORT		index;
	OP_WS_INFO 	*wsInfo;

	RESC_LIST_SET_FIRST(OpWs_WsList);

	for (index = 0; index < RESC_LIST_SIZE(OpWs_WsList); index++)
	{
		wsInfo = RESC_LIST_GET(OpWs_WsList);
		if ((wsInfo->assocId == assocId) && 
		    (RescASN_CompareVS(IR_FILE, &wsInfo->resultSetName,
		         name) == 0))
		{
			wsInfo->assocId = IR_UNKNOWN_ID_TYPE;
			RescASN_DestroyVS(IR_FILE, &wsInfo->resultSetName);
			wsInfo->totalTargetRecords = 0;
			wsInfo->nextTargetRec = 0;
			wsInfo->targetResultsDeleted = TRUE;
		}
	}
}

/****************************************************************************
 * FUNCTION:  OpWs_WsDeletedRemoteResult
 *
 * PURPOSE:
 *	This function is used to remove all references to a remote result
 * 	set for the provided workspace id.
 *
 * INPUT:
 *	IR_ASSOC_ID
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsDeletedRemoteResult(IR_WS_ID wsId)
{
	OP_WS_INFO 	*wsInfo;


	wsInfo = OpWs_WsGet(wsId);
	
	wsInfo->assocId = IR_UNKNOWN_ID_TYPE;
	RescASN_DestroyVS(IR_FILE, &wsInfo->resultSetName);
	wsInfo->totalTargetRecords = 0;
	wsInfo->nextTargetRec = 0;
	wsInfo->targetResultsDeleted = TRUE;

}

/****************************************************************************
 * FUNCTION:  OpWs_WsSetDeleteFlag
 *
 * PURPOSE:
 *	This function is used to set/reset the deleting remote flag for
 *	a work space.
 *
 * INPUT:
 *	IR_WS_ID
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsSetDeleteFlag(IR_WS_ID wsId, BOOLEAN flag)
{
	OP_WS_INFO 	*wsInfo;

	wsInfo = OpWs_WsGet(wsId);

	if (wsInfo)
	{
		wsInfo->deleting = flag;
	}
}

/****************************************************************************
 * FUNCTION:  OpWs_WsSetPresentFlag
 *
 * PURPOSE:
 *	This function is used to set/reset the presenting remote flag for
 *	a work space.
 *
 * INPUT:
 *	IR_WS_ID
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsSetPresentFlag(IR_WS_ID wsId, BOOLEAN flag)
{
	OP_WS_INFO 	*wsInfo;

	wsInfo = OpWs_WsGet(wsId);

	if (wsInfo)
	{
		wsInfo->presenting = flag;
	}
}
