
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/


/****************************************************************************
 * UNIT: ASN External
 *
 * DESCRIPTION:
 *	This file contains the functions used to manage external ASN
 *	data types.
 *
 * CONTENTS:
 *	RescASN_CreateExternal
 *	RescASN_DestroyExternal
 *	RescASN_SnaccToIRExternal
 *	RescASN_IRToSnaccExternal
 *
 * NOTES:
 *
 *
 *
/***************************************************************************/


#include <ir.h>
#include <resc.h>
#include <asn_tool.h>
#include <debug.h>

/****************************************************************************
 * FUNCTION:  RescASN_CreateExternal
 *
 * PURPOSE:
 *	Create and initialize a new IR external structure (Kinda like the
 *	SNACC ASN.1 external structure).
 *
 * INPUT:
 *	Pointer to source of caller
 *
 * OUTPUT:
 *	pointer to newly created and initialized IR EXTERNAL.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_EXTERNAL 	*RescASN_CreateExternal(CHAR *src)
{
	IR_EXTERNAL	*irElem;

	if ((irElem = (IR_EXTERNAL *)RescIR_Zmalloc(src, 
	    sizeof(IR_EXTERNAL))) == NULL)
	{
		CS_LOG_PRINTS(src, "Could not malloc external\n");
		return((IR_EXTERNAL *)NULL);
	}
	else
	{
		RESC_ASN_MALLOCED(irElem);
		irElem->referenceType = IR_EXT_REF_NO_REF;
		RESC_ASN_NO_MALLOC(&irElem->direct);
		RESC_ASN_INIT_OI(&irElem->direct);
		irElem->indirect = 0;
		RESC_ASN_NO_MALLOC(&irElem->dataValueDesc);
		RESC_ASN_INIT_VS(&irElem->dataValueDesc);

		irElem->encType = IR_EXT_ENC_NONE;
		irElem->anyType = 0;
		irElem->any = (void *)NULL;
		RESC_ASN_NO_MALLOC(&irElem->octetAligned);
		RESC_ASN_INIT_OS(&irElem->octetAligned);
		RESC_ASN_NO_MALLOC(&irElem->arbitrary);
		RESC_ASN_INIT_BS(&irElem->arbitrary);
	
		return(irElem);
	}

}

/****************************************************************************
 * FUNCTION:  RescASN_DestroyExternal
 *
 * PURPOSE:
 *	Destroy an IR external structure and all of its dependant 
 *	data elements.
 *
 * INPUT:
 *	pointer to IR external structure
 *
 * OUTPUT:
 *	IR STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescASN_DestroyExternal(CHAR *src, IR_EXTERNAL *irElem)
{

	if (irElem)
	{
		RescASN_DestroyOI(IR_FILE, &irElem->direct);
		RescASN_DestroyVS(IR_FILE, &irElem->dataValueDesc);
		RescASN_DestroyOS(IR_FILE, &irElem->octetAligned);
		RescASN_DestroyBS(IR_FILE, &irElem->arbitrary);

		if (RESC_ASN_IS_MALLOCED(irElem))
		{
			RescIR_Free(IR_FILE, (CHAR *)irElem);
		}
	}
	return(IR_SUCCESS);

}

/****************************************************************************
 * FUNCTION:  RescASN_IRToSnaccExternal
 *
 * PURPOSE:
 *	This function is used to convert an IR external structure to
 *	an outgoing SNACC ASN.1 external structure.
 *
 * INPUT:
 *	pointer to source of caller
 *	pointer to SNACC external structure
 *	pointer to IR external structure
 *
 * OUTPUT:
 *	external data in SNACC external structure
 *
 *
 * NOTES:
 * Currently this conversion only supports object identifier with
 * an octet octet aligned data type.
 *
 ***************************************************************************/

IR_STATUS RescASN_IRToSnaccExternal(CHAR *src, 
	      EXTERNAL *snaccElem, IR_EXTERNAL *irElem)
{

	if (irElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Missing irElem\n");
		return(IR_FAIL);
	}

	if (snaccElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Missing snaccElem\n");
		return(IR_FAIL);
	}

	
	switch (irElem->referenceType)
	{
	case IR_EXT_REF_NO_REF:
		CS_LOG_PRINTS(IR_FILE, "No reference type provided\n");
		return(IR_FAIL);
		break;
	case IR_EXT_REF_DIRECT:
		/* Check for a known type of encoding */
		if (irElem->encType == IR_EXT_ENC_ALIGNED)
		{

			/* Convert the object Identifier to the snacc struct */
			RescASN_IRToSnaccOI(IR_FILE, 
			    &snaccElem->direct_reference,
		    	    &irElem->direct);

			/* Convert the octet string */
			snaccElem->encoding = (EXTERNALChoice *)
		 	    RescIR_Zmalloc(IR_FILE, sizeof(EXTERNALChoice));

			snaccElem->encoding->choiceId = 
			    EXTERNALCHOICE_OCTET_ALIGNED;

			/* Create the snacc octet structure */
			snaccElem->encoding->a.octet_aligned = ASN_OctsCreate();

			/* Copy the IR Element octet string to snacc fmt */
			RescASN_IRToSnaccOS(IR_FILE, 
			    snaccElem->encoding->a.octet_aligned,
			    &irElem->octetAligned);
		}
		else
		{
			CS_LOG_PRINTV(IR_FILE, "Not octet aligned.%d\n",
			    irElem->encType);
			return(IR_FAIL);
		}
		break;

	case IR_EXT_REF_INDIRECT:
		
		CS_LOG_PRINTS(IR_FILE, 
		    "Indirect reference conversion not suppoerted\n");
		return(IR_FAIL);
		break;

	case IR_EXT_REF_DVD:
		CS_LOG_PRINTS(IR_FILE, 
		    "Data value descriptor conversion not suppoerted\n");
		return(IR_FAIL);
		break;
	default:
		CS_LOG_PRINTV(IR_FILE, "Unknown external reference type %d\n",
		    irElem->referenceType);
		return(IR_FAIL);
		break;
	}
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  RescASN_SnaccToIRExternal
 *
 * PURPOSE:
 *	This function is used to convert an external ASN.1 type from a
 *	SNACC structure to an IR structure.
 *
 * INPUT:
 *	pointer to source of caller
 *	pointer snacc external structure
 *	pointer to IR external structure
 *
 * OUTPUT:
 *	converted external data in IR external structure
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS RescASN_SnaccToIRExternal(CHAR *src, 
	      IR_EXTERNAL *irElem, EXTERNAL *snaccElem)
{

	if (irElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Missing irElem\n");
		return(IR_FAIL);
	}

	if (snaccElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Missing snaccElem\n");
		return(IR_FAIL);
	}


	irElem->referenceType = IR_EXT_REF_NO_REF;

	/* Check for an object Idenfifier */
	RescASN_SnaccToIROI(IR_FILE, &irElem->direct, 
	    &snaccElem->direct_reference);

	if (snaccElem->indirect_reference)
	{
		irElem->referenceType = IR_EXT_REF_INDIRECT;
		irElem->indirect = *snaccElem->indirect_reference;
		CS_LOG_PRINTV(IR_FILE, "External indirect reference %d\n", 
    		    irElem->indirect);
	}

	RescASN_SnaccToIROS(IR_FILE, &irElem->dataValueDesc,
	    &snaccElem->data_value_descriptor);

	if (irElem->direct.len > 0)
	{
		irElem->referenceType = IR_EXT_REF_DIRECT;
	}
	else if (irElem->dataValueDesc.len > 0)
	{
		irElem->referenceType = IR_EXT_REF_DVD;
	}
			
	if (snaccElem->encoding)
	{
		switch(snaccElem->encoding->choiceId)
		{
		case EXTERNALCHOICE_SINGLE_ASN1_TYPE:
			CS_LOG_PRINTS(IR_FILE, 
			    "external encoding type is single_ASN1_type\n");
			irElem->encType = IR_EXT_ENC_ANY;
			irElem->any = snaccElem->encoding->a.single_ASN1_type;
			break;

		case EXTERNALCHOICE_OCTET_ALIGNED:
			CS_LOG_PRINTS(IR_FILE, 
			    "external encoding type is octet aligned\n");
			irElem->encType = IR_EXT_ENC_ALIGNED;
			RescASN_SnaccToIROS(IR_FILE, &irElem->octetAligned,
			    snaccElem->encoding->a.octet_aligned);
			break;

		case EXTERNALCHOICE_ARBITRARY:
			CS_LOG_PRINTS(IR_FILE, 
			    "external encoding type is arbitrary\n");
			irElem->encType = IR_EXT_ENC_ARBITRARY;
			RescASN_SnaccToIRBS(IR_FILE, &irElem->arbitrary,
			    snaccElem->encoding->a.arbitrary);
			break;
		default:
			CS_LOG_PRINTS(IR_FILE, 
			    "external encoding type is unknown\n");
			irElem->encType = IR_EXT_ENC_NONE;
			CS_LOG_PRINTV(IR_FILE, 
			    "Illegal snacc external encoding type %d\n",
			    snaccElem->encoding->choiceId);
			break;
		}
	}
	return(IR_SUCCESS);
}
