
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: ASN List Utilities
 *
 * DESCRIPTION:
 *	This file contains fuinctions used to manipulate IR_LIST data
 *	structures.
 *
 * CONTENTS:
 *	ASN_ListCreate		- Create a new list
 *	ASN_ListDestroy		- Destroy an existing list and all of its
 *				  elements. Not Implemented yet.
 *	ASN_ListAppend		- Append a new element to the end of the list.
 *	ASN_ListDelete		- Delete a specific element from the list.
 *
 * NOTES:
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <asn_tool.h>
#include <resc.h>

/****************************************************************************
 * FUNCTION: ASN_ListCreate
 *
 * PURPOSE:
 *	This function will create a new AsnList structure with 0 elements.
 *
 * INPUT:
 *	None.
 *
 * OUTPUT:
 *	Pointer to the created list. Null if list could not be created.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

AsnList *ASN_ListCreate(USHORT dataElemSize)
{
	AsnList *list;

	if ((list = (AsnList *)RescIR_Malloc(IR_FILE, sizeof(AsnList))) 
	    != (AsnList *)NULL)
	{
		ASN_LIST_INIT(list);
		return(list);
	}
	else
	{
		CS_LOG_PRINTS(IR_FILE, "Error mallocing AsnList\n");
		return(list);
	}
}

/****************************************************************************
 * FUNCTION: ASN_ListDestroy
 *
 * PURPOSE:
 *	This function will destroy an AsnList and all elements plus each
 *	elements data.
 *
 * INPUT:
 *	IR_LIST pointer.
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	This function is not yet written.
 *
 ***************************************************************************/

IR_STATUS ASN_ListDestroy(AsnList *list)
{
	
	return(IR_FAIL);
} 

/****************************************************************************
 * FUNCTION: ASN_ListAppend
 *
 * PURPOSE:
 *	This function will append a new element to the end of a list and
 *	insert the provided data pointer into the new element.
 *
 * INPUT:
 *	IR_LIST pointer
 *	void 	data pointer (pointer to data of any size and type.)
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS ASN_ListAppend(AsnList *list, void *elemData)
{
	AsnListNode *listNode;

	/* Malloc the new node structure */
	if ((listNode = (AsnListNode *)RescIR_Malloc(IR_FILE, 
	    sizeof(AsnListNode)))
	    == (AsnListNode *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error mallocing list node\n");
		return(IR_FAIL);
	}

	/* Set the data pointer into the new element */
	listNode->data = elemData;

	if (ASN_LIST_SIZE(list) == 0)
	{
		/* First element so add to start of list */
		list->first = listNode;
		list->last = listNode;
		list->curr = listNode;
		listNode->next = (AsnListNode *)NULL;
		listNode->prev = (AsnListNode *)NULL;
	}
	else
	{
		/* Append to end of list */
		ASN_LIST_SET_LAST(list);
		list->last = listNode;
		list->curr->next = listNode;
		listNode->prev = list->curr;
		listNode->next = (AsnListNode *)NULL;
	}
	list->count++;
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: ASN_ListDelete
 *
 * PURPOSE:
 *	This function will delete a specific list element and free 
 *	and data pointed to by the list element data pointer.
 *
 * INPUT:
 *	AsnList pointer
 *	SHORT	pos
 *
 * OUTPUT:
 *	Returns pointer to data. 
 *
 * NOTES:
 *	The list node is deleted and a pointer to the data is returned to
 *	the caller. It is the callers responsibilty to destroy the data.
 *
 ***************************************************************************/

void	*ASN_ListDelete(AsnList *list, SHORT pos)
{
	AsnListNode *listNode;
	void 	    *data;

	if (pos >= ASN_LIST_SIZE(list))
	{
		/* Position not in list */
		CS_LOG_PRINTS(IR_FILE, "Position not in list \n");
		return(IR_SUCCESS);
	}

	ASN_ListSetPos(list, pos);

	listNode = list->curr;

	if (ASN_LIST_SIZE (list) == 1)
	{
		/* Removing the first and last element from the list */
		list->first = (AsnListNode *)NULL;
		list->last = (AsnListNode *)NULL;
		list->curr = (AsnListNode *)NULL;
	}
	else if (list->curr == list->last)
	{
		/* Removing last element from list */
		list->last = listNode->prev;
		listNode->prev->next = (AsnListNode *)NULL;
	}
	else if (list->curr == list->first)
	{
		/* Removing first element from list */
		list->first = listNode->next;
		listNode->next->prev = (AsnListNode *)NULL;
	}
	else
	{
		/* Removing from somewhere in the middle of the list */
		listNode->next->prev = listNode->prev;
		listNode->prev->next = listNode->next;
	}
	list->count--;

	data = listNode->data;

	RescIR_Free(IR_FILE, (CHAR *)listNode);

	return(data);
}

/****************************************************************************
 * FUNCTION: ASN_ListSetPos
 *
 * PURPOSE:
 *	This function will set the current list pointer to the position
 *	specified. If the position specified is not within the range of
 *	this list then the current list pointer will be set to the last
 *	element.
 *
 * INPUT:
 *	AsnList pointer
 *	SHORT	pos
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *
 ***************************************************************************/

void	ASN_ListSetPos(AsnList *list, SHORT pos)
{
	SHORT		i;


	ASN_LIST_SET_FIRST(list);
	i = 0;
	while ((ASN_LIST_CUR(list)) && (i < pos))
	{
		ASN_LIST_SET_NEXT(list);
		i++;
		
	}
	
	if (i != pos)
	{
		ASN_LIST_SET_LAST(list);
	}
}
