
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IR ASN.1 OID TOOLS
 *
 * DESCRIPTION:
 *	This file contains functions for manipulating object identifiers.
 *	It is used in close conjunction with the SNACC set of functions.
 *
 * CONTENTS:
 *	ASN_DottedStringToOid - Make an AsnOid from a dotted string. Save it as 
 *			  an AsnOid type (ie. Octet string).
 *	ASN_AsnOidToDottedString - Convert an AsnOid encoded oid to 
 *			  a dotted string format.
 *	ASN_FreeOid	- Free the OID list. Should only be visible to this 
 *			  file.
 *
 * NOTES:
 *
 * $Header$
 * $Log$
 *
/***************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ir.h>
#include <asn_incl.h>
#include <asn_tool.h>
#include <resc.h>
#include <debug.h>

static void ASN_FreeOid(OID *oidList);

/****************************************************************************
 * FUNCTION: ASN_DottedStringToOid
 *
 * PURPOSE:
 *	This function will generate an Object identifier from a dotted
 *	string. The dotted string is in the form "1.2.840.10003.7.1" which
 *	in this case the resource report object identifier. 
 *	This function will convert this dotted string to an OID list which
 *	is a SNACC list type for object identifiers. The OID list will then
 *	be converted to an AsnOid type. Once the AsnOid type is created
 *	then the OID list will be destroyed. The AsnOid type is treated as
 *	an octet string.
 *
 * INPUT:
 *	CHAR		*dottedString	string to convert.
 *
 * OUTPUT:
 *	AsnOid		*oidEntry
 *
 * NOTES:
 *	The dotted string is expected to be null terminated. The len field
 *	is not used.
 *	This function assumes that the base for the AsnOid has already been
 *	created.
 *
 ***************************************************************************/


SHORT ASN_DottedStringToOid(AsnOid *oidEntry, CHAR *dottedString, USHORT len)
{
	OID	*oidList;
	OID	*curOid;
	OID	*lastOid;
	LONG	arcNum;
	CHAR	*s;
	CHAR	tmpBuf[IR_LEN_50];
	SHORT	i;
	AsnLen	oidByteLen;
	SHORT	toolStat;
	BOOLEAN	done=FALSE;
	USHORT	counter;
	
	oidList = (OID *)NULL;
	curOid = oidList;
	lastOid = oidList;
	s = dottedString;
	counter = 0;
	while (done == FALSE)
	{
		
		/* Skip the dot if currently positioned over one */
		while (*s == '.')
		{
			s++;
			counter++;
			
		}
		/* Extract an entry from the dotted string notation */
		i = 0;
		while ((*s != '.') && (isdigit((int)*s)))
		{
			tmpBuf[i++] = *s++;
			counter++;
		}
		/* Check for last entry */
		if ((!isdigit((int)*s)) && (*s != '.') && (i == 0)) 
		{
			done = TRUE;
		}
		else
		{
			/* Terminate the arc num string */
			tmpBuf[i] = '\0';
			/* Convert the arc num string to a long */
			arcNum = atol(tmpBuf);

			/* Malloc the OID entry. Clearing the mem. */
			curOid = (OID *)Asn1Alloc(sizeof(OID));

			/* Set the arc num into the OID */
			curOid->arcNum = (long int)arcNum;
			curOid->next = (OID *)NULL;

			/* Create linked list of OIDs */
			if (oidList == (OID *)NULL)
			{
				/* Save the start of the list */
				oidList = curOid;
				lastOid = curOid;
			}
			else
			{
				/* Link the OID into the list of OIDs */
				lastOid->next = curOid;
				lastOid = lastOid->next;
			}
		}

		if ((!isdigit((int)*s)) && (*s != '.'))
		{
			done = TRUE;
		}
	}

	oidByteLen = EncodedOidLen(oidList);
	
	/* Create the oid but do not populate it with oid data */
	toolStat = ASN_OctsAdd(oidEntry, tmpBuf, (ULONG)oidByteLen);
	if (toolStat != 0)
	{
		CS_LOG_PRINTV(IR_FILE, "Error (%d) creating OidEntry\n", 
		    toolStat);
	}
	/* Now populate it with oid data */
	BuildEncodedOid(oidList, oidEntry);
	ASN_FreeOid(oidList);

	return(0);
}
		

/****************************************************************************
 * FUNCTION: ASN_AsnOidToDottedString
 *
 * PURPOSE:
 *	This function can be used to convert an object identifier in
 *	SNACC AsnOid format to a dotted string.
 *
 * INPUT:
 *	AsnOid		*oidEntry
 *
 * OUTPUT:
 *	CHAR		*dottedString
 *
 * NOTES:
 *	The calling function is responsible for the dottedString to be
 *	large enough to hold the object identifier. In most cases the
 *	object identifier should be less than 20 characters in our Z39.50
 *	application.
 *
 ***************************************************************************/

void ASN_AsnOidToDottedString(AsnOid *oidEntry, CHAR *dottedString)
{
	OID	*resultOid;
	OID	*curOid;
	SHORT	cnt;
	CHAR	tmpBuf[20];

	/* Use the SNACC tool to generate an OID list. */
	UnbuildEncodedOid(oidEntry, &resultOid);

	/* Convert the OID list to a string */
	cnt = 0;
	curOid = resultOid;
	dottedString[0] = '\0';
	while (curOid->next != (OID *)NULL)
	{
		(void)sprintf(tmpBuf, "%1ld", (LONG)curOid->arcNum);
		curOid = curOid->next;
		(void)strcat(dottedString, tmpBuf);
		(void)strcat(dottedString, ".");
	}
	/* Convert the last arcNum */
	(void)sprintf(tmpBuf, "%1ld", (LONG)curOid->arcNum);
	(void)strcat(dottedString, tmpBuf);

	/* Free the OID list */
	ASN_FreeOid(resultOid);
}

/****************************************************************************
 * LOCAL FUNCTION: ASN_FreeOid
 *
 * PURPOSE:
 *	This function is used to remove all elements of an OID list.
 *
 * INPUT:
 *	OID		*oidList
 *
 * OUTPUT:
 *	none
 *
 * NOTES:
 *
 ***************************************************************************/

static void ASN_FreeOid(OID *oidList)
{
	OID	*tmpOid;
	OID	*lastOid;

	/* Scan OID list and remove each entry in turn */
	for (tmpOid = oidList; tmpOid->next != (OID *)NULL; 
	    tmpOid = lastOid)
	{
		lastOid = tmpOid->next;
		RescIR_Free(IR_FILE, (CHAR *)tmpOid);
	}
	/* Free the last one in the list */
	RescIR_Free(IR_FILE, (CHAR *)tmpOid);
}

/****************************************************************************
 * FUNCTION: ASN_OidCopy
 *
 * PURPOSE:
 *	This function will copy an AsnOid structure to a NEW AsnOid
 *	structure. The srcOid item must contain the octet to copy and
 *	the destOid must be creatable from the ASN_MakeOid function.
 *
 * INPUT:
 *	AsnOid		*destOid
 *	AsnOid		*srcOid
 *
 * OUTPUT:
 *	SHORT		0 - copied. < 0 - error.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

SHORT ASN_OidCopy(AsnOid *destOid, AsnOid *srcOid)
/*
 * Return 0 if copied.
 * Return -1 if no src oid.
 */
{
	CHAR buf[20];

	if (ASNOID_PRESENT(srcOid))
	{
		ASN_AsnOidToDottedString(srcOid, buf);
		ASN_DottedStringToOid(destOid, buf, strlen(buf));
		return (0);
	}
	else
	{
		return (-1);
	}
}

