
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: RescList Utilities
 *
 * DESCRIPTION:
 *	This file contains fuinctions used to manipulate IR_LIST data
 *	structures.
 *
 * CONTENTS:
 *	RescList_Create		- Create a new list
 *	RescList_Initialize	- Initialize the pointers in a list.
 *	RescList_Destroy	- Destroy an existing list and all of its
 *				  elements.
 *	RescList_Append		- Append a new element to the end of the list.
 *	RescList_Insert		- Insert a new element into the list.
 *	RescList_Delete		- Delete a specific element from the list.
 *	RescList_Clone		- Generate a clone of the specified list.
 *
 * NOTES:
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <resclist.h>
#include <resc.h>


/****************************************************************************
 * FUNCTION: RescList_Create
 *
 * PURPOSE:
 *	This function will create a new IR_LIST structure with 0 elements.
 *
 * INPUT:
 *	None.
 *
 * OUTPUT:
 *	Pointer to the created list. Null if list could not be created.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_LIST	*RescList_Create(CHAR *srcFile)
{
	
	IR_LIST	*list;

	if ((list = (IR_LIST *)RescIR_Malloc(srcFile, sizeof(IR_LIST))) 
	    != (IR_LIST *)NULL)
	{
		RESC_LIST_INIT(list);
		RESC_ASN_MALLOCED(list);
		return(list);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error mallocing IR_LIST\n");
		return(list);
	}
}

/****************************************************************************
 * FUNCTION: RescList_Destroy
 *
 * PURPOSE:
 *	This function will destroy an IR_LIST and all elements plus each
 *	elements data.
 *
 * INPUT:
 *	IR_LIST pointer.
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS  RescList_Destroy(CHAR *srcFile, IR_LIST *list)
{
	
	if (list == (IR_LIST *)NULL)
	{
		/* Error */
		CS_LOG_PRINTS(srcFile, "list already destroyed\n");
		return(IR_SUCCESS);
	}

	/* Delete each element in the list first */
 	while(RESC_LIST_SIZE(list) > 0)
	{
		RescList_Delete(srcFile, list, 0);
	}

	if (RESC_ASN_IS_MALLOCED(list))
	{
		RescIR_Free(srcFile, (CHAR *)list);
	}

	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: RescList_Append
 *
 * PURPOSE:
 *	This function will append a new element to the end of a list and
 *	insert the provided data pointer into the new element.
 *
 * INPUT:
 *	IR_LIST pointer
 *	void 	data pointer (pointer to data of any size and type.)
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS RescList_Append(CHAR *srcFile, IR_LIST *list, void *elemData, 
    void (*destFn)(), void *(*dupFn)())
{
	IR_LIST_ELEM *tmpElem;

	if ((list == (IR_LIST *)NULL) || (elemData == (void *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Null pointer not allowed in list \n");
		return(IR_FAIL);
	}

	/* Malloc the new element structure */
	if ((tmpElem = (IR_LIST_ELEM *)RescIR_Malloc(srcFile, 
	    sizeof(IR_LIST_ELEM)))
	    == (IR_LIST_ELEM *)NULL)
	{
		CS_LOG_PRINTS(srcFile, "Error mallocing list element\n");
		return(IR_FAIL);
	}

	/* Set the data pointer into the new element */
	tmpElem->elem = elemData;
	tmpElem->destFn = destFn;
	tmpElem->dupFn = dupFn;
	tmpElem->isMalloced = TRUE;

	if (RESC_LIST_SIZE(list) == 0)
	{
		/* First element so add to start of list */
		list->first = tmpElem;
		list->last = tmpElem;
		list->current = tmpElem;
		tmpElem->next = (IR_LIST_ELEM *)NULL;
		tmpElem->prev = (IR_LIST_ELEM *)NULL;
	}
	else
	{
		/* Append to end of list */
		RESC_LIST_SET_LAST(list);
		list->last = tmpElem;
		list->current->next = tmpElem;
		tmpElem->prev = list->current;
		tmpElem->next = (IR_LIST_ELEM *)NULL;
	}
	list->size++;
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: RescList_Insert
 *
 * PURPOSE:
 *	This function will insert a new element into the provided list 
 *	at the position specified. If the position specified is greater
 *	than the size of the list then an error will be returned unless
 *	the position would be the next position in the list. In this case
 *	the RescList_Append function will be called to append the new element
 *	to the end of the list.
 *
 * INPUT:
 *	IR_LIST pointer
 *	void 	data pointer (pointer to data of any size and type.)
 *	SHORT   position (0 being the first position in the list)
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS RescList_Insert(CHAR *srcFile, IR_LIST *list, void *elemData, 
    void (*destFn)(), void *(*dupFn)(), SHORT pos)
{
	IR_LIST_ELEM *tmpElem;

	if ((list == (IR_LIST *)NULL) || (elemData == (void *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Null pointer not allowed in list \n");
		return(IR_FAIL);
	}

	if ((pos > RESC_LIST_SIZE(list)) && (RESC_LIST_SIZE(list) > 0))
	{
		/* Error. Position not in list nor last position for append */
		CS_LOG_PRINTS(IR_FILE, "New list position out of range\n");
		return(IR_FAIL);
	}

	if ((pos == RESC_LIST_SIZE(list)) || (RESC_LIST_SIZE (list) == 0))
	{
		/* Last position. Append this one to the end of the list */
		return(RescList_Append(srcFile, list, elemData, 
		    destFn, dupFn));
	}

	/* Malloc the new element structure */
	if ((tmpElem = (IR_LIST_ELEM *)RescIR_Malloc(srcFile, 
	    sizeof(IR_LIST_ELEM)))
	    == (IR_LIST_ELEM *)NULL)
	{
		CS_LOG_PRINTS(srcFile, "Error mallocing list element\n");
		return(IR_FAIL);
	}

	/* Set the data pointer into the new element */
	tmpElem->elem = elemData;
	tmpElem->destFn = destFn;
	tmpElem->dupFn = dupFn;
	tmpElem->isMalloced = TRUE;

	/* Set the current pointer to the specified position */
	RESC_LIST_SET_CUR(list, pos);

	if (pos == 0)
	{
		/* First element so add to start of list */
		list->first = tmpElem;
		list->current->prev = tmpElem;
		tmpElem->prev = (IR_LIST_ELEM *)NULL;
		tmpElem->next = list->current;
	}
	else 
	{
		/* Insert into middle of list */
		tmpElem->prev = list->current->prev;
		tmpElem->next = list->current;
		list->current->prev->next = tmpElem;
		list->current->prev = tmpElem;
	}
	list->size++;
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: RescList_Delete
 *
 * PURPOSE:
 *	This function will delete a specific list element and free 
 *	and data pointed to by the list element data pointer.
 *
 * INPUT:
 *	IR_LIST pointer
 *	SHORT	pos
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS RescList_Delete(CHAR *srcFile, IR_LIST *list, SHORT pos)
{
	IR_LIST_ELEM *tmpElem;

	if (list == (IR_LIST *)NULL) 
	{
		CS_LOG_PRINTS(srcFile, "Null pointer not allowed in list \n");
		return(IR_FAIL);
	}

	if (pos >= RESC_LIST_SIZE(list))
	{
		/* Position not in list */
		CS_LOG_PRINTV(srcFile, "Position not in list %d\n", pos);
		return(IR_SUCCESS);
	}


	RESC_LIST_SET_CUR(list, pos);

	tmpElem = list->current;

	if (RESC_LIST_SIZE (list) == 1)
	{
		/* Removing the first and last element from the list */
		list->first = (IR_LIST_ELEM *)NULL;
		list->last = (IR_LIST_ELEM *)NULL;
		list->current = (IR_LIST_ELEM *)NULL;
	}
	else if (list->current == list->last)
	{
		/* Removing last element from list */
		list->last = tmpElem->prev;
		tmpElem->prev->next = (IR_LIST_ELEM *)NULL;
	}
	else if (list->current == list->first)
	{
		/* Removing first element from list */
		list->first = tmpElem->next;
		tmpElem->next->prev = (IR_LIST_ELEM *)NULL;
	}
	else
	{
		/* Removing from somewhere in the middle of the list */
		tmpElem->next->prev = tmpElem->prev;
		tmpElem->prev->next = tmpElem->next;
	}
	list->size--;

	if (tmpElem->destFn)
	{
		tmpElem->destFn(IR_FILE, (void *)tmpElem->elem);
	}

	if (tmpElem->isMalloced)
	{
		RescIR_Free(srcFile, (CHAR *)tmpElem);
	}

	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: RescList_Clone
 *
 * PURPOSE:
 *	This function will clone an IR list. Each element in the list
 *	will be copyied to the duplicate list. The dud function provided
 *	with each element will be used to copy the elements data into
 *	a new element. 
 *
 * INPUT:
 *	IR_LIST pointer
 *
 * OUTPUT:
 *	IR_LIST pointer of cloned list.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_LIST	*RescList_Clone(CHAR *srcFile, IR_LIST *srcList)
{
	IR_LIST		*cloneList;


	CS_LOG_PRINTS(srcFile, "Help! I'm being cloned\n");
	cloneList = RescList_Create(srcFile);


	RescList_Copy(srcFile, cloneList, srcList);

	return(cloneList);
}
		

/****************************************************************************
 * FUNCTION: RescList_Copy
 *
 * PURPOSE:
 *	This function will copy all elements of the source list and append
 *	them to the end of the dest list.
 *
 * INPUT:
 *	calling function file and line
 *	IR_LIST pointer
 *	IR_LIST pointer
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS RescList_Copy(CHAR *srcFile, IR_LIST *destList, IR_LIST *srcList)
{
	void 		*copyElemData;
	SHORT		count;
	void		*cmpPtr;


	RESC_LIST_SET_FIRST(srcList);

	cmpPtr = (void *)0xffff;
	for (count = 0; count < RESC_LIST_SIZE(srcList); count++)
	{

		if (srcList->current->dupFn != NULL)
		{
			copyElemData = 
			    srcList->current->dupFn(srcFile, 
			    srcList->current->elem);
			/* Check for invalid pointer */
			if (!((long)copyElemData & (long)cmpPtr)) 
			{
			}
			if (copyElemData == NULL)
			{
				CS_LOG_PRINTV(srcFile, 
				    "Couldn't copy element %d\n", count);
			}
			else
			{
				RescList_Append(srcFile, destList, copyElemData,
				    srcList->current->destFn,
				    srcList->current->dupFn);
			}
		}
		RESC_LIST_SET_NEXT(srcList);
	}
	return(IR_SUCCESS);
}
		
