/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Communication CSC
 *
 * DESCRIPTION:
 *	The Communictation CSC maintains system specific requirements 
 *	for communciation.  In this case the Communication CSC uses
 *	the UNIX based,WINSOCK standard to perform TCP/IP communications 
 *	on a DOS/Windows based platform.
 *
 *
 * CONTENTS:
 *	UaCs_Init
 *	UaCs_Terminate
 *	UaCs_GetNewAssocIid
 *	UaCs_DeleteAssocId
 *
 * NOTES:
 *	This file is platform specific. The system used to obtain
 *	socket messages is based solely on the WINSOCK standard
 *	for MS Windows and DOS.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_net.h>
#include <resc.h>
#include <windows.h>
#ifdef XVT_MSDOS
#include <xvt.h>
#endif
#include <commune.h>

#define NETWINDOW "NetWindowC"


LONG PASCAL NetWndProc(HWND hWnd, UINT wMsg, UINT wParam, LONG lParam);
static IR_STATUS InitNetApp(void);

static HWND	 appNetParentWnd;
static HANDLE   hInst;

#ifdef WINSOCKET
WSADATA  WSAData;         /* windows sockets info return */
#endif

/****************************************************************************
 * FUNCTION:	UaCs_Init
 *
 * PURPOSE:
 * 	UaCs_init - Called from XVT application main before the xvt_system
 * 	function is invoked. This function will establish the network window
 * 	infrastructure. This function will create the initial window class
 * 	used to generate each association's communication window. 
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	There is no output from this function.
 *
 * NOTES:
 *	This function should be called before XVT is started.
 *
 ***************************************************************************/

IR_STATUS UaCs_Init(void)
{
#ifdef WINSOCKET
	int err;
   	if (err = WSAStartup(0x0101, &WSAData))   /* register task with */
   	{                                         /* winsock tcp/ip API */ 
		return(IR_FAIL);
   	}                   
#endif

#ifdef XVT_MSDOS

	hInst = (HANDLE)get_value(NULL_WIN, ATTR_WIN_INSTANCE);

	/* Get the parent native window */
	appNetParentWnd = (HWND)get_value(NULL_WIN, ATTR_NATIVE_WINDOW);
#endif

	/* Initialize the network window class. */
	if (InitNetApp() == IR_FAIL)
	{
#ifdef WINSOCKET
		WSACleanup();
#endif
		return(IR_FAIL);
	}
	else
	{
		return(IR_SUCCESS);
	}
}

/****************************************************************************
 * FUNCTION: 	UaCs_Terminate
 *
 * PURPOSE:
 *	The User Application Event Processing Communciation CSU
 *	Terminate function completes processing of the communication
 *	CSU.
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	There is no output from this function.
 *
 * NOTES:
 *	Currently this function doesn't do anything it is provided
 *	to maintain a ballance with the initailaization call. It
 * 	is possilbe that this function may be required in the furture.
 *
 ***************************************************************************/

void UaCs_Terminate(void)
{
#ifdef WINSOCKET
	WSACleanup();
#endif
}

/****************************************************************************
 * FUNCTION: 	UaCs_GetNewAssocId
 *
 * PURPOSE:
 *	User Application Event processing Communication CSU Get New
 *	Association Identifiier obtains the association Id by creating 
 *	an invisible child of the main window which will be used to i
 *	receive Windows Sockets notification messages.  The Window
 *	handle is used as the Association Id and cannot be destroyed
 *	until all references to that association have been closed
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	This function returns an Associatioin Id.
 *
 * NOTES:
 *	This function should only be called after a call to UaEp_Init.
 *
 ***************************************************************************/
IR_ASSOC_ID UaCs_GetNewAssocId(void)
{

	IR_ASSOC_ID assocId = (IR_ASSOC_ID)CreateWindow(
	    NETWINDOW, 
	    "", 
	    WS_OVERLAPPEDWINDOW,
	    CW_USEDEFAULT, 
	    CW_USEDEFAULT,
	    CW_USEDEFAULT, 
	    CW_USEDEFAULT,
	    appNetParentWnd,
	    (HMENU)NULL,
	    hInst, 
	    (void *)NULL);

#if 0
	xvt_note ("Generated association Id = %d", assocId);
#endif
	
	return (assocId);
}

/****************************************************************************
 * FUNCTION: UaCs_DeleteAssocId
 *
 * PURPOSE:
 *	The User application Event Processing Communication CSU Deletete
 *	Association Identiifier destorys thte window indicated by the
 * 	Association Id.
 *
 * INPUT:
 *	This function requires a valid Associatioin Id.
 *
 * OUTPUT:
 *	There is no output from this function.
 *
 * NOTES:
 *	This function should only be called after a call to UaEp_Init.
 *
 ***************************************************************************/
void UaCs_DeleteAssocId(IR_ASSOC_ID assocId)
{
	/* Destroy the window with that association Id */
   	(void)DestroyWindow((HWND)assocId);
}

/****************************************************************************
 * LOCAL FUNCTION: NetWndProc 
 *
 * PURPOSE:
 * 	The NetWndProc callback function for network window.  
 *	The network window is a child window created especially to 
 *	receive windows sockets network messages.  The function decodes 
 *	and routes to appropriate message handler.
 *
 * INPUT:
 *	This function requires a window handle, and three paramaters
 *	two of unsigned int and one long to handle input.
 * OUTPUT:
 *	There is no output from this function.
 *
 * NOTES:
 * 	This function is called by WINSOCK code only.
 *
 ****************************************************************************/

LONG PASCAL NetWndProc(HWND hWnd, UINT wMsg, UINT wParam, LONG lParam)
{
#ifdef XVT_MSDOS
	EVENT *userEvent;
#endif
	NET_MSG_STRUCT *netMsgData;

	/*
	 * Make sure the message is valid.
	 */
	if (wMsg > WM_USER)
	{
#ifdef XVT_MSDOS
		/*
		 * We have a valid message. Store it in an
		 * XVT style EVENT structure.
		 */
		userEvent = (EVENT *)RescIR_Malloc(IR_FILE,
		    sizeof(EVENT));
		netMsgData = (NET_MSG_STRUCT *)RescIR_Malloc(IR_FILE,
		    sizeof(NET_MSG_STRUCT));
		userEvent->type = E_USER;
		userEvent->v.user.id = 1;
		userEvent->v.user.ptr = netMsgData;
		netMsgData->hWnd = hWnd;
		netMsgData->wMsg = wMsg;
		netMsgData->wParam = wParam;
		netMsgData->lParam = lParam;

#if 0
CS_LOG_PRINTV(IR_FILE, "Dispatching event for assocId %d\n", hWnd);
#endif
		/*
		 * Send the information to the Main Task
		 * Window.
		 */
		dispatch_event(TASK_WIN, userEvent);
		(void)RescIR_Free(IR_FILE, (char *)userEvent);
		(void)RescIR_Free(IR_FILE, (char *)netMsgData);
        	return(NULL);
#endif
#ifdef WINPATHWAY
		/*
		 * Package the event into a net message structure
		 * and send it to the net event processing functions.
		 */
		netMsgData = (NET_MSG_STRUCT *)RescIR_Malloc(IR_FILE,
		    sizeof(NET_MSG_STRUCT));
		netMsgData->hWnd = hWnd;
		netMsgData->wMsg = wMsg;
		netMsgData->wParam = wParam;
		netMsgData->lParam = lParam;
		UaEp_ProcessCommEvent(netMsgData);
		(void)RescIR_Free(IR_FILE, (char *)netMsgData);
#endif
	}
	else
	{

		/*
	 	 * We don't handle this message.
	 	 */
#if 0
		CS_LOG_PRINTV(IR_FILE, "Unknown Msg %d\n", wMsg);
#endif
   		return(DefWindowProc(hWnd, wMsg, wParam, lParam));
	}
}

/****************************************************************************
 * LOCAL FUNCTION: InitNetApp 
 *
 * PURPOSE:
 *	The InitNetApp function registers window class for the network 
 *	module's invisible child window. This is called only during the
 *	initialization and is not required for each window created.
 *
 * INPUT:
 *	There is no input to function.
 *
 * OUTPUT:
 *	There is no output from this function.
 *
 * NOTES:
 *	This function should only be called from UaEp_Init.
 *
 ****************************************************************************/

static IR_STATUS InitNetApp(void)
{
	WNDCLASS wndclass;

	wndclass.style = NULL;
	wndclass.lpfnWndProc = NetWndProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInst;
	wndclass.hIcon = NULL;
	wndclass.hCursor = NULL;
	wndclass.hbrBackground = NULL;
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = NETWINDOW;

	if (RegisterClass(&wndclass))
	{
		return(IR_SUCCESS);
	}
	else
	{
		return(IR_FAIL);
	}
}
