
/****************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/************************************************************************
 * UNIT:  User Application Event Processing Initialize CSU
 *
 * DESCRIPTION:
 *	This CSU contains the initialize routines required to
 *	initalize the user application and the IRO API interface.
 *
 * CONTENTS:
 *	UaEp_Initialize
 *	UaEp_InitIRApi
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <xvt.h>
#include <uaep.h>
#include <uaepif.h>
#include <ua/ua_param.h>
#include <resc.h>

static IRO_INIT_REQ	UAEP_IROInitReq;
static BOOLEAN		UAEP_ParamError;
static BOOLEAN		UAEP_CommError;

static long UAEP_MinuteTimer;
static long UAEP_EventTimer;

/****************************************************************************
 * FUNCTION:  UaEp_Initialize
 *
 * PURPOSE:
 *	This function will verify the input command line parameters
 *	and initialize the user application accordingly.
 *
 * INPUT:
 *	Command line argc and argv.
 *
 * OUTPUT:
 *	int (non-zero on error).
 *
 * NOTES:
 *	none.
 *
 ***************************************************************************/

int UaEp_Initialize(int argc, char **argv)
{
	extern IR_STATUS UaCs_Init(void);
	UA_PARAM_TYPE	paramType;
	UA_PARAM_VALUE	paramValue;
	SHORT		index;
	SHORT		subIndex;

	/* 
	 * Establish system configuration polled/event driven 
 	 * and system base language.
	 */
	UAEP_IROInitReq.commConfig = IR_COMM_CONF_EVENT_INT;
	UAEP_IROInitReq.langConfig = IR_ENGLISH;
	UAEP_IROInitReq.debugLog = FALSE;
	UAEP_IROInitReq.logAPDUs = FALSE;
	RESC_ASN_NO_MALLOC(&UAEP_IROInitReq);

	UAEP_ParamError  = FALSE;

	for (index = 1; index < argc; index++)
	{
		subIndex = 0;
		if ((PARAM_START(argv[index][subIndex])) &&
		     (strlen(argv[index]) >= PARAM_LEN))
		{
			subIndex++;
			paramType = toupper(argv[index][subIndex]);
			subIndex++;
			paramValue = toupper(argv[index][subIndex]);

			switch(paramType)
			{
			case LANG_PARAM:
				if (LANG_PARAM_INVALID(paramValue))
				{
					UAEP_ParamError  = TRUE;
				}
				else
				{
					UAEP_IROInitReq.langConfig =
					    GET_LANG(paramValue);
				}
				break;
			case COMM_PARAM:
				if (COMM_PARAM_INVALID(paramValue))
				{
					UAEP_ParamError  = TRUE;
				}
				else
				{
					UAEP_IROInitReq.commConfig =
					    GET_COMM(paramValue);
				}
				break;
			case DEBUG_PARAM:
				UAEP_IROInitReq.debugLog = 
				    BOOLEAN_PARAM(paramValue);
				break;
			case APDU_LOG_PARAM:
				UAEP_IROInitReq.logAPDUs =
				    BOOLEAN_PARAM(paramValue);
				break;
			default:
				UAEP_ParamError  = TRUE;
			}
		}
		else
		{
			UAEP_ParamError  = TRUE;
		}
	}

	if (UAEP_ParamError  == FALSE)
	{
		UAEP_CommError = FALSE;
		if (UaCs_Init() == IR_FAIL)
		{
			UAEP_CommError = TRUE;
		}
	}

	/* 
	 * Always return success so that if an error is detected it
	 * will be displayed when the IRO interface is initialized.
	 * At this point in the initialization process xvt cannot
	 * display the error message.
	 */
	return(0);
}
			
int UaEp_Terminate()
{
	extern void UaCs_Terminate();
	UaCs_Terminate();
	IRO_Terminate();
	xvt_terminate();
	return(0);
}

int UaEp_InitIRApi(void)
{
	if (UAEP_ParamError == TRUE)
	{
		xvt_error(PARAM_ERROR,
		    PARAM_ERROR1(UAEP_IROInitReq.langConfig),
		    PARAM_ERROR2(UAEP_IROInitReq.commConfig));
		xvt_terminate();
	}

	if (UAEP_CommError == TRUE)
	{
		xvt_error("Unable to initialize communication window");
		xvt_terminate();
	}

	UaHec_Initialize();

	if (IRO_Initialize(&UAEP_IROInitReq) != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE, IRO_INITIALIZE_FUNC, -1);
		xvt_terminate();
	}


	return(0);
		
}

void UaEp_InitTaskWin(UAEP_DIALOG *loginData)
{
	UAEP_SYSTEM_INFO_STRUCT	*taskWin;	/* Ptr to task window
					 	 * maintained info.
						 */

	taskWin = (UAEP_SYSTEM_INFO_STRUCT *)RescIR_Zmalloc(IR_FILE,
		    sizeof(UAEP_SYSTEM_INFO_STRUCT));

	taskWin->userId = loginData->data.login.userId;
	taskWin->lang = loginData->data.login.prefLang;
	taskWin->priv = loginData->data.login.privilege;
	taskWin->mode = UAEP_IROInitReq.commConfig;
	taskWin->wsRoot = (UAEP_WORK_SPACE_LIST *)NULL;
	taskWin->activeWs = IR_UNKNOWN_ID_TYPE;
	set_app_data(TASK_WIN, (long)taskWin);

	/* Setup the timers for polling comms and operation abort */
	UAEP_MinuteTimer = set_timer(TASK_WIN, UA_ONE_MINUTE);
	if (UAEP_IROInitReq.commConfig == IR_COMM_CONF_EVENT_POLLED)
	{
		UAEP_EventTimer = set_timer(TASK_WIN, UA_POLL_TIME);
	}
}

int UaEp_Timer(WINDOW win, EVENT *ep)
{
	if (ep->v.timer.id == UAEP_EventTimer)
	{
		/* Process polled communication event. */
	}
	else if (ep->v.timer.id == UAEP_MinuteTimer)
	{
		/* Process minute Timer event */
	}
	return(0);
}

static void userInfoFree(IRO_USER_INFO_REQ *, IRO_USER_INFO_REPLY *);

IR_STATUS UaEp_LoginCmd(WINDOW win, UAEP_DIALOG *login)
{
	IRO_USER_INFO_REQ	req;
	IRO_USER_INFO_REPLY	reply;

	if ((strlen(login->data.login.name) == 0) ||
	    (strlen(login->data.login.password) == 0))
	{
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_EP, 
		    MSG_ID_EP_LOGIN_REQD, "Login Attempt Failed", 0);
		return(IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&req);
	RESC_ASN_NO_MALLOC(&reply);

	
	RESC_ASN_NO_MALLOC(&req.loginName);
	RESC_ASN_INIT_VS(&req.loginName);
	RescASN_AddVS(IR_FILE, &req.loginName, login->data.login.name,
	    strlen(login->data.login.name));
	RESC_ASN_NO_MALLOC(&req.loginPassword);
	RESC_ASN_INIT_VS(&req.loginPassword);
	RescASN_AddVS(IR_FILE, &req.loginPassword, login->data.login.password,
	    strlen(login->data.login.password));

	if (IRO_UserInfo(IRO_UI_LOGIN, &req, &reply) != IR_SUCCESS)
	{
		UaHec_Message("Login Failure", "Invalid user name and password combination. Try again.", TRUE);
		userInfoFree(&req, &reply);
		return(IR_FAIL);
	}

	/* Save the login information. */

	login->data.login.userId = reply.loginInfo->userId;
	login->data.login.prefLang = reply.loginInfo->prefLang;
	login->data.login.privilege =reply.loginInfo->privilege;

	userInfoFree(&req, &reply);
	return(IR_SUCCESS);

}

void userInfoFree(IRO_USER_INFO_REQ *req, IRO_USER_INFO_REPLY *reply)
{
	if (req)
	{
		RescASN_DestroyVS(IR_FILE, &req->loginName);
		RescASN_DestroyVS(IR_FILE, &req->loginPassword);
		if (RESC_ASN_IS_MALLOCED(req))
		{
			RescIR_Free(IR_FILE, (CHAR *)req);
		}
	}
	if(reply)
	{
		if (RESC_ASN_IS_MALLOCED(reply))
		{
			RescIR_Free(IR_FILE, (CHAR *)reply);
		}
	}
}
