/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Help and Error Control Target/Association correlation
 *
 * DESCRIPTION:
 *	This unit contains function used to correlate assocation identifiers
 *	with a displayable string which corresponds to the association.
 *
 * CONTENTS:
 *	UaHec_AddAssocName
 *	UaHec_GetAssocName
 *	UaHec_UpdateAssocName
 *	UaHec_DeleteAssocName
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <resclist.h>
#include <string.h>

static IR_LIST		UAHEC_NameTbl;

typedef struct 
{
	IR_ASSOC_ID		assocId;
	VISIBLE_STRING		name;
	BOOLEAN			isMalloced;
} NAME_ENTRY;

static void nameTblDestFn(CHAR *srcFile, void *data);
static SHORT findAssocInList(IR_ASSOC_ID assocId);

/*
 * Initialization function.
 */
void UaHec_InitAssocNameTbl(void)
{
	RESC_ASN_NO_MALLOC(&UAHEC_NameTbl);
	RESC_LIST_INIT(&UAHEC_NameTbl);
}

/****************************************************************************
 * FUNCTION: UaHec_AddAssocName
 *
 * PURPOSE:
 *	This function will add a name/assocId pair to the nameTbl.
 *
 * INPUT:
 *	NONE.
 *
 * OUTPUT:
 *	This function will return an IR_FAIL status if the assocId or
 *	the name input parameters are invalid, or if a malloc error is
 *	detected.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaHec_AddAssocName(IR_ASSOC_ID assocId, VISIBLE_STRING *name)
{
	NAME_ENTRY	*tmpEntry;

	if ((assocId == IR_UNKNOWN_ID_TYPE) ||
	    (name->len == 0))
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid input parameters\n");
		return(IR_FAIL);
	}

	
	tmpEntry = (NAME_ENTRY *)RescIR_Zmalloc(IR_FILE, sizeof(NAME_ENTRY));

	if (tmpEntry == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Memory Error\n");
		return(IR_FAIL);
	}
	else
	{
		RESC_ASN_MALLOCED(tmpEntry);
		RESC_ASN_NO_MALLOC(&tmpEntry->name);
		tmpEntry->assocId = assocId;
		RescASN_CopyVS(IR_FILE, &tmpEntry->name, name);
		RescList_Append(IR_FILE, &UAHEC_NameTbl, tmpEntry,
		    nameTblDestFn, NULL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: UaHec_GetAssocName
 *
 * PURPOSE:
 *	This function will Get a name for known assocId;
 *
 * INPUT:
 *	The assocId is needed to find the corresponding name string.
 *
 * OUTPUT:
 *	This function will return a pointer to a string.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

VISIBLE_STRING *UaHec_GetAssocName(IR_ASSOC_ID assocId)
{
	NAME_ENTRY		*tmp;
	VISIBLE_STRING		*vs;
	SHORT			pos;

	
	if (assocId == IR_UNKNOWN_ID_TYPE)
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid assocId \n");
		return((VISIBLE_STRING *)NULL);
	}

	pos = findAssocInList(assocId);

	if (pos < 0)
	{
		return((VISIBLE_STRING *)NULL);
	}
	else
	{
		RESC_LIST_SET_CUR(&UAHEC_NameTbl, pos);
		tmp = RESC_LIST_GET(&UAHEC_NameTbl);
		vs = RescASN_CreateVS(IR_FILE);
		RescASN_CopyVS(IR_FILE, vs, &tmp->name);
		return(vs);
	}
}

/****************************************************************************
 * FUNCTION: UaHec_UpdateAssocName
 *
 * PURPOSE:
 *	This function will Update a name for known assocId;
 *
 * INPUT:
 *	The assocId is needed to find the corresponding name string to
 *	replace and the new name.
 *
 * OUTPUT:
 *	This function will return an IR_FAIL status if the assocId or
 *	the name input parameters are invalid, or if a malloc error is
 *	detected.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaHec_UpdateAssocName(IR_ASSOC_ID assocId, VISIBLE_STRING *name)
{
	NAME_ENTRY		*tmp;
	VISIBLE_STRING		*vs;
	SHORT			pos;

	
	if (assocId == IR_UNKNOWN_ID_TYPE)
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid assocId \n");
		return(IR_FAIL);
	}

	pos = findAssocInList(assocId);

	if (pos < 0)
	{
		return(IR_FAIL);
	}
	else
	{
		RESC_LIST_SET_CUR(&UAHEC_NameTbl, pos);
		tmp = RESC_LIST_GET(&UAHEC_NameTbl);
		RescASN_DestroyVS(IR_FILE, &tmp->name);
		RescASN_CopyVS(IR_FILE, &tmp->name, name);
	}
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: UaHec_DeleteAssocName
 *
 * PURPOSE:
 *	This function will Delete a name/assocId pair from the nameTbl.
 *
 * INPUT:
 *	AssocId - identifies the name/assocId pair to delete.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_DeleteAssocName(IR_ASSOC_ID assocId)
{
	SHORT			pos;

	if (assocId == IR_UNKNOWN_ID_TYPE)
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid assocId \n");
		return;
	}

	pos = findAssocInList(assocId);

	if (pos >= 0)
	{
		RescList_Delete(IR_FILE, &UAHEC_NameTbl, pos);
	}

	return;
}

static void nameTblDestFn(CHAR *srcFile, void *data)
{
	NAME_ENTRY	*tmp;

	if (data)
	{
		tmp = (NAME_ENTRY *)data;
		RescASN_DestroyVS(srcFile, &tmp->name);
		if (RESC_ASN_IS_MALLOCED(tmp))
		{
			RescIR_Free(srcFile, (CHAR *)tmp);
		}
	}
}

static SHORT	findAssocInList(IR_ASSOC_ID assocId)
{
	SHORT		pos;
	NAME_ENTRY	*tmp;

	if (RESC_LIST_SIZE(&UAHEC_NameTbl) == 0)
	{
		/* No entries in list */
		return (-1);
	}

	RESC_LIST_SET_FIRST(&UAHEC_NameTbl);

	for (pos = 0; pos < RESC_LIST_SIZE(&UAHEC_NameTbl); pos++)
	{
		tmp = RESC_LIST_GET(&UAHEC_NameTbl);
		if (tmp->assocId == assocId)
		{
			break;
		}
		RESC_LIST_SET_NEXT(&UAHEC_NameTbl);
	}

	if (pos == RESC_LIST_SIZE(&UAHEC_NameTbl))
	{
		/* Not in list */
		return(-1);
	}
	else
	{
		return(pos);
	}
}

