/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Record Display and Control Format
 *
 * DESCRIPTION:
 * 	Record Display and Control Format CSU formats records that are
 *	maintained in a predefined format into a user definded fromat.
 *
 * CONTENTS:
 *
 * 	 UaRdc_Format
 *
 * NOTES:
 *	This CSU can be invoked from any User Application CSU.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <ua/ua_rdc.h>
#include <ua/usmarc.h>

/*
 * Local Function declarations.
 */
static IR_STATUS fmtBriefUSMARC(UARDC_RECORD_FORMAT_STRUCT *,
    UARDC_USMARC_RECORD,
    UARDC_DISPLAY_STRUCT *);

static IR_STATUS fmtMultiUSMARC(UARDC_RECORD_FORMAT_STRUCT *,
    UARDC_USMARC_RECORD,
    UARDC_DISPLAY_STRUCT *);

static IR_STATUS fmtAllUSMARC(UARDC_RECORD_FORMAT_STRUCT *,
    UARDC_USMARC_RECORD,
    UARDC_DISPLAY_STRUCT *);

static BOOLEAN usmarcSearch(CHAR *,
    UARDC_USMARC_RECORD,
    USMARC_FIELD_DATA	*);

static BOOLEAN	usmarcField(SHORT reqType,
    CHAR **dirPtr,
    UARDC_USMARC_RECORD,
    USMARC_FIELD_DATA	*);

static CHAR *usmarcFldFmt(UARDC_FORMAT_STYLE, CHAR *, USMARC_FIELD_DATA *);

static CHAR *usmarcSetMarked(USMARC_FIELD_DATA *);

static CHAR *usmarcSetUnmarked(CHAR *, USMARC_FIELD_DATA *);

static SHORT StrNToShort(CHAR *, USHORT);


/****************************************************************************
 * FUNCTION: UaRdc_Format
 *
 * PURPOSE:
 *	The User Application Record Display Control Format Function converts
 *	records in  a predefined format into a user describedc displayable\
 *	format.
 *
 * INPUT:
 *	This function requires a UARDC_RECORD_FORMAT_STRUCT describing what
 *	format the record is in and how the displayable record format should]
 * 	look. It also requires a UARDC_DISPLAY_STRUCT which contains the 
 *	record to be converted.
 *
 * OUTPUT:
 *	This function returns the displayable record in a UARDC_DISPLAY_STRUCT
 *	and an IR_STATUS indicating success or failure.
 *
 * NOTES:
 *	The list of displayable records must be freed by whoever uses the
 *	information each line is stored in an array of pointers to CHAR so
 *	once you are finished withe array it too must be destroyed.
 *
 ***************************************************************************/

IR_STATUS UaRdc_Format(
    UARDC_RECORD_FORMAT_STRUCT *fmt,	/* Description of how to format
					 * the record. */
    UARDC_RECORD_STRUCT  *rcd,		/* Record to format. */
    UARDC_DISPLAY_STRUCT *display)	/* Pointer to output information. */
{
	IR_STATUS displayCode = IR_SUCCESS;

	/*
	 * Determine what type of record is to be formatted.
	 */
	switch (fmt->rType)
	{
	case USMARC_REC_TYPE:
		/*
	 	 * Format the USMARC record based on type of output.
		 */
		if (fmt->type == UARDC_TYPE_ALL)
		{
			/* Format all fields */
			displayCode = fmtAllUSMARC(fmt, rcd->usmarc, display);
		}
		else if(fmt->type == UARDC_TYPE_UMULTI)
		{
			/* Format all specified fields */
			displayCode = fmtMultiUSMARC(fmt, rcd->usmarc, display);
		}
		else 
		{
			/* 
			 * Format specified fields into a 
			 * single displayable line.
			 */
			displayCode = fmtBriefUSMARC(fmt, rcd->usmarc, display);
		}
		break;
	/*
 	 * MORE TYPES WILL BE HANDLED HERE IN THE FUTURE
	 */
	default:
		break;
	}
	return (displayCode);
}

/******************************************************************************
 * Format of USMARC Records.
 ******************************************************************************/

/****************************************************************************
 * LOCAL FUNCTION: fmtBriefUSMARC
 *
 * PURPOSE:
 *	The fmtBriefUSMARC local function formats USMARC records into
 *	displayable brief records. The fields which are formatted are
 *	specified in the UARDC_RECORD_FORMAT_STRUCT and the USMARC record
 *	is maintained by the UARDC_USMARC_RECORD.  The brief record displays
 *	all specified fields on a single line.
 *
 * NOTES:
 *	The list of displayable records must be freed by whoever uses the
 *	information each line is stored in an array of pointers to CHAR so
 *	once you are finished withe array it too must be destroyed.
 *
 ****************************************************************************/

#define USMARC_BS_SIZE	2
#define USMARC_BRIEF_SEPARATOR	", "
static IR_STATUS fmtBriefUSMARC(
    UARDC_RECORD_FORMAT_STRUCT *fmt,	/* Description of how to format
					 * the record. */
    UARDC_USMARC_RECORD  rcd,		/* Record to format. */
    UARDC_DISPLAY_STRUCT *display)	/* Pointer to output information. */
/*
 * This function returns the UARDC_DISPLAY_STRUCT containing the 
 * formatted USMARC record and an IR_STATUS.
 */
{
	IR_STATUS 	     retCode;
	USHORT		     lines;	/* Lines allocated for display */
	UARDC_DISPLAY_STRUCT tmpDisp;	/* Temporary display setup. */
	SHORT		     totLen;	/* Total length of brief */
	CHAR		     *dispStr;	/* Display String */

	/*
	 * Format the record into a list of displayable fields.
	 */
	if ((retCode = fmtMultiUSMARC(fmt, rcd, &tmpDisp)) !=
	    IR_SUCCESS)
	{
		/*
		 * We failed.
		 */
		return(retCode);
	}

	/*
	 * Initialize the output structure and calculate the length
 	 * of the brief description line.
	 */
	display->lines = 0;
	display->displayLines = (CHAR **)NULL;
	totLen = 0;
	for (lines = 0; lines < tmpDisp.lines; lines++)
	{
		totLen += strlen(tmpDisp.displayLines[lines]);
		totLen += USMARC_BS_SIZE;

	}
	if (totLen == 0)
	{
		/* 
		 * There is nothing to display.
		 */
		return(IR_SUCCESS);
	}
	/*
	 * Allocate space for the return Brief display.
	 */
	dispStr = (char *)RescIR_Malloc(IR_FILE, (totLen+1));
	dispStr[0] = (CHAR)NULL;
	/*
	 * Append each line to the display string and free the space
	 * allocated to that part of the display.
	 */
	for (lines = 0; lines < tmpDisp.lines; lines++)
	{
		(void)strcat(dispStr, tmpDisp.displayLines[lines]);
		(void)strcat(dispStr, USMARC_BRIEF_SEPARATOR);
		(void)RescIR_Free(IR_FILE, tmpDisp.displayLines[lines]);
	}
	/* Free the list of display lines.  */
	(void)RescIR_Free(IR_FILE, (CHAR *)tmpDisp.displayLines);
	/*
	 * Add the display line to the output structure.
	 */
	(display->lines)++;
	display->displayLines = (char **)RescIR_Malloc(IR_FILE, sizeof(char*));
	display->displayLines[0] = dispStr;
	return(IR_SUCCESS);
}
	
/****************************************************************************
 * LOCAL FUNCTION: fmtMultiUSMARC
 *
 * PURPOSE:
 *	The fmtMultiUSMARC local function formats USMARC records into
 *	displayable multi-lined records. The fields which are formatted are
 *	specified in the UARDC_RECORD_FORMAT_STRUCT and the USMARC record
 *	is maintained by the UARDC_USMARC_RECORD.  The Multi record displays
 *	all specified fields each on a separate line.
 *
 * NOTES:
 *	The list of displayable records must be freed by whoever uses the
 *	information each line is stored in an array of pointers to CHAR so
 *	once you are finished withe array it too must be destroyed.
 *
 ****************************************************************************/
static IR_STATUS fmtMultiUSMARC(
    UARDC_RECORD_FORMAT_STRUCT *fmt,	/* Description of how to format
					 * the record. */
    UARDC_USMARC_RECORD  rcd,		/* Record to format. */
    UARDC_DISPLAY_STRUCT *display)	/* Pointer to output information. */
/*
 * This function returns the UARDC_DISPLAY_STRUCT containing the 
 * formatted USMARC record and an IR_STATUS.
 */
{
	IR_STATUS	  	retCode;
	USMARC_FIELD_DATA 	fldData; /* data extracted from the field. */
	USHORT		  	totLines;/* Total lines allocated for display */
	UARDC_USMARC_FORMAT_LIST *list;	 /* Fields to display. */

	/* Initialize variables.  */
	retCode = IR_SUCCESS;
	display->lines = 0;
	display->displayLines = (CHAR **)0;
	totLines = 0;
	list = fmt->l.usmarcList;

	/*
	 * Process each requested tag.
	 */
	while (strlen(list->tag) > 0)
	{
		/*
		 * Check if the tagged field is in the record.
		 */
		if (usmarcSearch(list->tag, rcd, &fldData) == TRUE)
		{
			/*
			 * The specified record exists.
	 		 * Increment the nuber of lines in our
			 */
			if (list->lable == (CHAR *)0)
			{
				list->lable = list->tag;
			}
			if (totLines == 0)
			{
				/*
				 * We have not allocated any output lines.
				 * allocate a block of lines.
			 	 */
				totLines = UARDC_LINEBLK;
				if ((display->displayLines = 
				    (char **)RescIR_Malloc(IR_FILE,
	    			    (sizeof(char*) * totLines))) == (CHAR **)0)
				{
					return (IR_FAIL);
				}
			}
			else if (display->lines >= totLines)
			{
				/*
				 * We have run out of allocated lines
				 * add some more.
				 */
				totLines += UARDC_LINEBLK;
				display->displayLines = (CHAR **)RescIR_Realloc(
			    	IR_FILE, 
				(CHAR *)display->displayLines,
			    	(sizeof(char*) * totLines));
			}
			/*
			 * format the field into the appropriate form.
			 */
			display->displayLines[display->lines] = 
			    usmarcFldFmt(fmt->style, list->lable, &fldData);
			(display->lines)++;
			/* Free the field data */
			fldData.field = (CHAR *)RescIR_Free(IR_FILE, 
			    (CHAR *)fldData.field);
		}
		list++; /* Increment our list of tags. */
	}

	return (retCode);
}


/****************************************************************************
 * LOCAL FUNCTION: fmtAllUSMARC
 *
 * PURPOSE:
 *	The fmtAllUSMARC local function formats USMARC records into
 *	displayable multi-lined records. The all fields specified in the 
 *	UARDC_USMARC_RECORD are formatted.
 *	The Multi record displays all specified fields each on a separate line.
 *
 * NOTES:
 *	The list of displayable records must be freed by whoever uses the
 *	information each line is stored in an array of pointers to CHAR so
 *	once you are finished withe array it too must be destroyed.
 *
 ****************************************************************************/
static IR_STATUS fmtAllUSMARC(
    UARDC_RECORD_FORMAT_STRUCT *fmt,	/* Description of how to format
					 * the record. */
    UARDC_USMARC_RECORD  rcd,		/* Record to format. */
    UARDC_DISPLAY_STRUCT *display)	/* Pointer to output information. */
/*
 * This function returns the UARDC_DISPLAY_STRUCT containing the 
 * formatted USMARC record and an IR_STATUS.
 */
{
	IR_STATUS	  retCode;
	USMARC_FIELD_DATA fldData;	/* data extracted from the field. */
	CHAR		  *rcdPtr;	/* Pointer into record data
					 * maintained for search. */
	USHORT		  totLines;	/* Total lines allocated for display */

	/* Initialize variables */
	rcdPtr = (CHAR *)NULL;
	retCode = IR_SUCCESS;
	display->lines = 0;
	display->displayLines = (CHAR **)0;

	/*
	 * Get the first usmarc field. 
	 */
	if (usmarcField(FIRST_FIELD, &rcdPtr, rcd, &fldData) == FALSE)
	{
		return (IR_SUCCESS);
	}

	/*
	 * Allocate space for the output.
	 */
	totLines = UARDC_LINEBLK;
	if ((display->displayLines = (char **)RescIR_Malloc(IR_FILE,
	    (sizeof(char*) * totLines))) == (CHAR **)0)
	{
		return (IR_FAIL);
	}

	/*
	 * For each field in the record.
	 */
	do
	{
		/*
	 	 * Increment the number of lines in the display.
		 */
		(display->lines)++;
		if (display->lines >= totLines)
		{
			/*
			 * More space is required for the output.
			 * Allocate it.
			 */
			totLines += UARDC_LINEBLK;
			display->displayLines = (CHAR **)RescIR_Realloc(
			    IR_FILE, 
			    (CHAR *)display->displayLines,
			    (sizeof(char*) * totLines));
		}
		/*
		 * Format the field and add it to our output.
		 */
		display->displayLines[display->lines] = 
		    usmarcFldFmt(fmt->style, fldData.marc, &fldData);
		/* Free the field information. */
		fldData.field = (CHAR *)RescIR_Free(IR_FILE, fldData.field);
	}
	while(usmarcField(NEXT_FIELD, &rcdPtr, rcd, &fldData) == TRUE);
		
	return (retCode);
}

/****************************************************************************
 * LOCAL FUNCTION: usmarcSearch
 *
 * PURPOSE:
 *	The usmarcSearch local function searches a USMARC record for
 *	the field with the specified tag and returns  the field and
 *	tag in a USMARC_FIELD_DATA structure.  
 *
 * NOTES:
 *	The field data must be freed by the calling function.
 ****************************************************************************/
static BOOLEAN usmarcSearch(
    CHAR		*tag,	 /* Field indicator. */
    UARDC_USMARC_RECORD	rec,	 /* Record to search through */
    USMARC_FIELD_DATA	*retData)/* Field Indicator and field output data.  */
/*
 * This function returns TRUE if the field is found FALSE otherwise.
 */
{
	SHORT	baseAddr;	/* Base address to record data */
	SHORT	offset;		/* Offset from base to data. */
	SHORT 	fieldSz;	/* Size of field to return. */
	USMARC_DIR_ENTRY *recPtr; /* Pointer into directory structure.  */

	/*
	 * Point our record pointer at the beginning of the directory
	 * information in the USMARC record.
	 */
	recPtr = (USMARC_DIR_ENTRY *)&(rec[USMARC_DIR_START]);

	/*
	 * Check each directory entry in the USMARC record to see if the
	 * specified tag is included.
	 */
	while ((recPtr->tag[0] != USMARC_FIELD_TERM) &&
	    (strncmp(recPtr->tag, tag, USMARC_TAG_SIZE) != 0))
	{
		recPtr++;
	}

	/*
	 * if the first character of the tag information is the end
 	 * of field character than the tag was not found.
	 */
	if (recPtr->tag[0] == USMARC_FIELD_TERM)
	{
		return(FALSE);
	}

	/*
 	 * The tag was found copy the tag to the output structure.
	 */
	strncpy(retData->marc, recPtr->tag, USMARC_TAG_SIZE);
	retData->marc[USMARC_TAG_SIZE] = (CHAR)NULL;
	
	/*
	 * Obtain the offsets to the field data, the offset to the field
	 * required and the lenght of the field required from the Leader
 	 * and the directory.
	 */
	if (((baseAddr = 
	    StrNToShort(&(rec[USMARC_BASE_OFFSET]), USMARC_BASE_SIZE)) == -1)
	    || ((offset = 
	    StrNToShort(recPtr->offset, USMARC_OFFSET_SIZE)) == -1) 
	    || ((fieldSz = 
	    StrNToShort(recPtr->length, USMARC_LEN_SIZE)) == -1))
	{
		/*
		 * We could not obtain the info.
		 */
		return (FALSE);
	}
	
	/*
 	 * Allocate space in the reply structure and return it.
	 */
	retData->field = (CHAR *)RescIR_Malloc(IR_FILE, fieldSz + 1);
	strncpy(retData->field, &(rec[offset+baseAddr]), fieldSz);
	retData->field[fieldSz] = (CHAR)NULL;
	
	return (TRUE);
}
/****************************************************************************
 * LOCAL FUNCTION: usmarcField
 *
 * PURPOSE:
 *	The usmarcField local function allows the caller to get USMARC
 *	record fields in serial order by a maintained directory pointer
 *	and a request type which indicates first/next/previous/current
 *	record. The field information is returned in a USMARC_FIELD_DATA 
 *	structure.  
 *
 * NOTES:
 *	The field data must be freed by the calling function.
 ****************************************************************************/
static BOOLEAN	usmarcField(
    SHORT 		reqType, /* FIRST_FIELD NEXT_FIELD PREV_FIELD        */
    CHAR 		**dirPtr,/* Pointer provided by caller to reference  *
			 	  * current location.                        */
    UARDC_USMARC_RECORD rec,	 /* Record to get data out of.		     */
    USMARC_FIELD_DATA	*retData)/* Field Indicator and field output data.   */
/*
 * This function returns TRUE if the field is found FALSE otherwise.
 */
{

	SHORT	baseAddr;	/* Base address to record data */
	SHORT	offset;		/* Offset from base to data. */
	SHORT 	fieldSz;	/* Size of field to return. */
	USMARC_DIR_ENTRY *recPtr; /* Pointer into directory structure.  */
	

	/*
	 * Set the directory pointer to the saved position.
	 */
	recPtr = (USMARC_DIR_ENTRY *)(*dirPtr);
	
	/* Check to see which record is to be obtained. */
	switch (reqType)
	{
	case NEXT_FIELD:
		/*
	 	 * Caller wants the next field.
		 */
		if (recPtr->tag[0] == USMARC_FIELD_TERM)
		{
			/*
			 * The last field was the last field in the
			 * record they cannot get more.
			 */
			return(FALSE);
		}
		/* Point at the next field */
		recPtr++;
		if (recPtr->tag[0] == USMARC_FIELD_TERM)
		{
			/*
			 * This field is the last field they cannot get more.
			 */
			return(FALSE);
		}
		break;

	case PREV_FIELD:
		/*
		 * Get the previous field.
		 */
		if (recPtr == (USMARC_DIR_ENTRY *)&(rec[USMARC_DIR_START]))
		{
			/* 
			 * The last field was the first field so they
			 * can't get the previous field.
			 */
			return (FALSE);
		}
		/*
		 * Point at the previous field.
		 */
		recPtr--;
		break;

	case FIRST_FIELD:
		/* Point at the first field. */
		recPtr = (USMARC_DIR_ENTRY *)&(rec[USMARC_DIR_START]);
		break;

	case CURRENT_FIELD:
		/* Let's get this one again. */
		break;

	default:
		/* Invalid specification */
		return(FALSE);
		break;
	}
	/*
	 * Set their pointer to the new entry.
	 */
	(*dirPtr) = (CHAR *)recPtr;

	/*
	 * Copy the tag into the reply.
	 */
	strncpy(retData->marc, recPtr->tag, USMARC_TAG_SIZE);
	retData->marc[USMARC_TAG_SIZE] = (CHAR)NULL;
	
	if (((baseAddr = 
	    StrNToShort(&(rec[USMARC_BASE_OFFSET]), USMARC_BASE_SIZE)) == -1)
	    || ((offset = 
	    StrNToShort(recPtr->offset, USMARC_OFFSET_SIZE)) == -1) 
	    || ((fieldSz = 
	    StrNToShort(recPtr->length, USMARC_LEN_SIZE)) == -1))
	{
		return (FALSE);
	}
	
	retData->field = (CHAR *)RescIR_Malloc(IR_FILE, fieldSz + 1);
	strncpy(retData->field, &(rec[offset+baseAddr]), fieldSz);
	retData->field[fieldSz] = (CHAR)NULL;
	
	return (TRUE);
}

static CHAR *usmarcFldFmt(
    UARDC_FORMAT_STYLE style,	/* Style of formatting. */
    CHAR	       *lable,  /* Lable for output. */
    USMARC_FIELD_DATA	       *field)	/* Field Indicator and field
				 * output data.  */
{
	CHAR *returnField;

	switch (style)
	{
	case UARDC_STYLE_TAG:
		returnField = usmarcSetMarked(field);
		break;

	case UARDC_STYLE_NONE:
		lable = "";
	case UARDC_STYLE_LABLE:
		returnField = usmarcSetUnmarked(lable, field);
		break;

	default:
		returnField = (CHAR *)0;
		break;
	}
	return(returnField);
}

static CHAR	bigBuff[USMARC_BUF_SIZE];

static CHAR *usmarcSetMarked(
    USMARC_FIELD_DATA *field)  /* Field Indicator and raw field output data.  */
{
	SHORT 	fieldLen;
	SHORT 	indx;
	CHAR	*retString;

	indx = 0;
	fieldLen = strlen(field->marc);
	strncpy(bigBuff, field->marc, fieldLen);
	bigBuff[fieldLen] = USMARC_SPACE;
	fieldLen++;
	while((fieldLen < USMARC_BUF_SIZE) &&
	    (field->field[indx] != USMARC_FIELD_TERM))
	{
		if (field->field[indx] == USMARC_SUB_FIELD)
		{
			indx++;
			(void)sprintf(&(bigBuff[fieldLen]),
			    USMARC_SF_REPLACEMENT,
			    field->field[indx]);
			fieldLen += USMARC_SF_SIZE;
		}
		else if (field->field[indx] != (CHAR)NULL)
		{
			bigBuff[fieldLen] = field->field[indx];
			fieldLen++;
		}
		indx++;
	}
	retString = (CHAR *)RescIR_Malloc(IR_FILE, fieldLen+1);
	memcpy(retString, bigBuff, fieldLen);
	retString[fieldLen] = (CHAR)NULL;
	return(retString);
}

static CHAR *usmarcSetUnmarked(
    CHAR              *lable,  /* Lable for output. */
    USMARC_FIELD_DATA *field)  /* Field Indicator and raw field output data.  */
{	
	SHORT 	fieldLen;
	SHORT 	indx;
	CHAR	*retString;

	indx = 0;
	fieldLen = strlen(lable);
	if (fieldLen > 0)
	{
		strncpy(bigBuff, lable, fieldLen);
		bigBuff[fieldLen] = USMARC_SPACE;
		fieldLen++;
	}

	if ((strncmp(field->marc, USMARC_FXTLEN_007,
	    strlen(USMARC_FXTLEN_007)) != 0) &&
	    (strncmp(field->marc, USMARC_FXTLEN_007,
	    strlen(USMARC_FXTLEN_008)) != 0))
	{
		indx += USMARC_INDICATOR_SIZE;
	}
	while((fieldLen < USMARC_BUF_SIZE) && 
	    (field->field[indx] != USMARC_FIELD_TERM))
	{
		if (field->field[indx] == USMARC_SUB_FIELD)
		{
			indx++;
			bigBuff[fieldLen] = USMARC_SPACE;
		}
		else if (field->field[indx] == (CHAR)NULL)
		{
			fieldLen--;
		}
		else
		{
			bigBuff[fieldLen] = field->field[indx];
		}
		fieldLen++;
		indx++;
	}
	if (!(retString = (CHAR *)RescIR_Malloc(IR_FILE, fieldLen+1)))
	{
		xvt_fatal("Memory error allocating output");
	}
	memcpy(retString, bigBuff, fieldLen);
	retString[fieldLen] = (CHAR)NULL;
	return(retString);
}

static SHORT StrNToShort(CHAR *data, USHORT length)
{
	CHAR shortString[20];

	if (length > 19)
	{
		return (-1);
	}
	strncpy(shortString, data, length);
	shortString[length] = (CHAR)NULL;

	return(atoi(shortString));
}
