
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Present Control Commnunication Event.
 *
 * DESCRIPTION:
 *	The Search Control Communication Event CSU reacts to 
 *	communication events concerning associations that have been 
 *	detected by a previous call to IRO_CommEvent.
 *
 * CONTENTS:
 *	UaPc_CommEvent
 *	UaRdc_CommEvent
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ***************************************************************************/

#include <ir.h>
#include <resc.h>
#include <ua.h>
#include <uaep.h>
#include <debug.h>
#include <ir_oid.h>

/****************************************************************************
 * FUNCTION: UaPc_CommEvent
 *
 * PURPOSE:
 *	The User Application Search Control Communication Event
 *	function reacts to various sorts of communication events in
 *	order to inform the user and or update the menus.
 *
 * INPUT:
 * 	This function requires an UASC_COMM_EVENT_DATA_TYPE, namely one
 *	of the following:
 *
 * 	    UASC_SEARCH_COMPLETE	- A Search has completed.
 *
 *	An IRO_COMM_EVENT_REPLY is also required and contains the
 *	information returned from a call to IRO_CommEvent.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaPc_CommEvent(
    UASC_COMM_EVENT_DATA_TYPE dataType,  /* Type of communiction event. */
    IRO_COMM_EVENT_REPLY *replyData)	 /* Related comm data. */
{
	IR_STATUS	returnCode;	/* Return value for function.  */
	CHAR			*wsName;
	IR_ASSOC_ID		assocId;
	BOOLEAN 		presentStatus;
	IRO_COMM_PRSNT_RESP	*pResp;
	CHAR                    tmpBuf[IR_LEN_100];
	CHAR			*noteString;
	CHAR			*s, *t;
	VISIBLE_STRING		*targetName;
	CHAR			targetNameStr[IR_LEN_100];


	returnCode = IR_SUCCESS;

	/* Malloc the message display string */
	noteString = (CHAR *)RescIR_Malloc(IR_FILE, IR_LEN_1000);

	assocId = replyData->assocId;
	pResp = &replyData->commData.presentResponse;
	presentStatus = pResp->presentStatus;

	/* Create a new workspace to correspond with the origin process */
CS_LOG_PRINTV(IR_FILE, "Received present response for wsId %d\n",
    pResp->wsId);
	if (pResp->wsId != IR_UNKNOWN_ID_TYPE)
	{
		/* Create a window and pop up any records that are there */
		UaEp_UpdateWin(pResp->wsId);
		/* Get the name from the window */
		wsName = UaEp_GetWsName(pResp->wsId);
	}
	else
	{
		/* 
		 * This name is malloced and must be freed before leaving
		 * because there is no window using this name. 
		 */
		wsName = UaEp_GenerateName(pResp->wsId);
	}

	targetName = UaHec_GetAssocName(assocId);
	if (targetName)
	{
		VSToStrNullTerm(targetNameStr, targetName, IR_LEN_100);
	}
	else
	{
		sprintf(targetNameStr, "Unknown target for association %d\n",
		    assocId);
	}

	/*
	 * Determine the type of Communication event.
	 */
	switch (dataType)
	{
	case IR_COMM_DATA_PRESENT_SRVC:

		if (pResp->presentStatus != 5)
		{
			sprintf(noteString,"Present %s (%s) was successful.",
			    wsName, targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			
			sprintf(s, "Total records in local workspace is %ld.",
			    pResp->numRecInWs);
			while (*s != EOS)
				s++;
			sprintf(s, "Total records at remote target is %ld.",
			    pResp->numRecOnTarget);
			while (*s != EOS)
				s++;
			sprintf(s, "Total records retrieved is %ld.",
			    pResp->numRecRetrieved);
			while (*s != EOS)
				s++;
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_PS, pResp->presentStatus,
			    &t);
			sprintf(s, "Target provided present status = %s.", t);

			UaHec_Message("Present Response", noteString, FALSE);

		}
		else
		{
			sprintf(noteString,"Present %s (%s) failed.",
			    wsName, targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			/* Check the result status and diagnostic */
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_BIB1, 
			    pResp->diagnostic.condition,
			    &t);
			sprintf(s,"Target supplied diagnostic condition number is %ld. %s.",
			    pResp->diagnostic.condition, t);
			
			VSToStrNullTerm(tmpBuf, &pResp->diagnostic.addInfo, 
			    IR_LEN_100);
			strcat(s, tmpBuf);
			strcat(s, ".");

			UaHec_Message("Present Response", noteString, FALSE);

		}

		break;
	default:
		break;
	}
	if ((pResp->wsId == IR_UNKNOWN_ID_TYPE) &&
	    (wsName == (CHAR *)NULL))
	{
		/* Free wsName that is not attached to a window */
		RescIR_Free(IR_FILE, (CHAR *)wsName);
	}

	return (returnCode);
}

/****************************************************************************
 * FUNCTION: UaRdc_CommEvent
 *
 * PURPOSE:
 *	This function is used to process the returned delete response
 *	structure that was provided as a result of the IR delete response
 *	message being received.
 *
 * INPUT:
 *	eventType
 *	Delete comm structure.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaRdc_CommEvent(
    UASC_COMM_EVENT_DATA_TYPE dataType,  /* Type of communiction event. */
    IRO_COMM_EVENT_REPLY *replyData)	 /* Related comm data. */
{
	IR_STATUS	returnCode;	/* Return value for function.  */
	IR_ASSOC_ID		assocId;
	BOOLEAN 		deleteStatus;
	IRO_COMM_DEL_RESP	*dResp;
	CHAR                    tmpBuf[IR_LEN_100];
	CHAR			*noteString;
	CHAR			*s, *t;
	VISIBLE_STRING		*targetName;
	CHAR			targetNameStr[IR_LEN_100];


	returnCode = IR_SUCCESS;

	/* Malloc the message display string */
	noteString = (CHAR *)RescIR_Malloc(IR_FILE, IR_LEN_1000);

	assocId = replyData->assocId;
	dResp = &replyData->commData.deleteResponse;
	deleteStatus = dResp->deleteStatus;

	targetName = UaHec_GetAssocName(assocId);
	if (targetName)
	{
		VSToStrNullTerm(targetNameStr, targetName, IR_LEN_100);
	}
	else
	{
		sprintf(targetNameStr, "Unknown target for association %d\n",
		    assocId);
	}

	/*
	 * Determine the type of Communication event.
	 */
	switch (dataType)
	{
	case IR_COMM_DATA_DELETE_SRVC:

		if (dResp->deleteStatus == 0)
		{
			sprintf(noteString,"Delete (%s) was successful.",
			    targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			UaHec_Message("Delete Response", noteString, FALSE);

		}
		else
		{
			sprintf(noteString,"Delete (%s) failed.",
			    targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_PS, dResp->deleteStatus,
			    &t);
			sprintf(s, "Target provided delete status is %s.", t);

			VSToStrNullTerm(tmpBuf, &dResp->deleteMessage,
			    IR_LEN_100);
			strcat(s, tmpBuf);
			strcat(s, ".");

			UaHec_Message("Delete Response", noteString, FALSE);

		}

		break;
	default:
		break;
	}

	return (returnCode);
}
