/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Search Build Query
 *
 * DESCRIPTION:
 *
 *	The Search Build Query CSU obtains the query the user wants to 
 * 	apply to the current search. Will also attempt to send the search 
 *	at the users request.
 *
 * CONTENTS:
 *	UaSc_BuildQuery
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <uaep.h>
#include <resc.h>
#include <resclist.h>

/****************************************************************************
 * FUNCTION: UaSc_BuildQuery
 *
 * PURPOSE:
 *	The User Application Search Control Build Query CSU obtains
 * 	the search query from the user and uses Search Maintenance to
 *	store the information.
 *	It will attempt to send the search at the request of the user.
 *
 * INPUT:
 *	There is no input to this CSU
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Not yet required
 ***************************************************************************/

IR_STATUS UaSc_BuildQuery(void)
{
	static UAEP_DIALOG	 query;	/* Query information for search. */
    	UASC_MAINT_REQ   mReq;  /* Search Maintenance Request info. */
    	UASC_MAINT_REPLY mReply;/* Search Maintenance Reply info. */
	
	/*
	 * Initialize the query information for the dialog.
	 */
	query.cancelled = FALSE;
	query.data.query.search = FALSE;

	/* Get the current query from storage. */
	UaSc_Maintenance(UASC_MAINT_GET_ACT, &mReq, &mReply);
	query.data.query.tree = mReply.act.search.query;

	/*
	 * Get Query Info from User.
	 */
	create_res_dialog(WD_MODAL, SQ_SEARCH_QUERY_DLG, 
	    EM_ALL, dlg_106_eh, (long)&query);

	if (query.cancelled == TRUE)
	{
		/*
		 * User Cancelled clean up.
		 */
		if (query.data.query.tree != (RPN_Structure *)NULL)
		{
			RPN_Structure_Free(query.data.query.tree);
			query.data.query.tree = (RPN_Structure *)NULL;
		}
		return(IR_SUCCESS);
	}

	/*
	 * Store the active query info.
	 */
	mReq.search.query = query.data.query.tree;
	UaSc_Maintenance(UASC_MAINT_STORE_ACT_QRY, &mReq, &mReply);
	if (query.data.query.search && mReply.act.canSearch)
	{
		/* Lets send the query. */
		return(UaSc_SendSearch());
	}
	else if (query.data.query.search)
	{
		if (query.data.query.tree == (RPN_Structure *)NULL)
		{
			xvt_note("You must enter a query to send first.");
			return (IR_SUCCESS);
		}
		else if (UaEp_MenuGetStatus(M_SEARCH_DATABASE) != TRUE)
		{
			xvt_note("You must activate an association first.");
			return (IR_SUCCESS);
		}
		/* Don't have enough info to send the query ask for it. */
		return(UaSc_GetDatabase());
	}
	return(IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: UaSc_GetDatabase
 *
 * PURPOSE:
 *	The User Application Search Control Get Database CSU obtains
 * 	the target and database to be searched from the user and
 *	stores that information in the Search Maintenance. It
 *	will attempt to send the search at the request of the user.
 *
 * INPUT:
 *	There is no input to this CSU
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Not yet required
 ***************************************************************************/

IR_STATUS UaSc_GetDatabase(void)
{
	static UAEP_DIALOG	 database;/* Database information for search. */
    	UASC_MAINT_REQ   mReq;   /* Search Maintenance Request info. */
    	UASC_MAINT_REPLY mReply; /* Search Maintenance Reply info. */
	
	/*
	 * Initialize the database information for the dialog.
	 */
	database.cancelled = FALSE;
	database.data.database.search = FALSE;

	/* Get the current database from storage. */
	UaSc_Maintenance(UASC_MAINT_GET_ACT, &mReq, &mReply);
	RPN_Structure_Free(mReply.act.search.query);

	database.data.database.assocId =
	    mReply.act.search.database.assocId;
	database.data.database.smallSetUpperBound =
	    mReply.act.search.database.smallSetUpperBound;
	database.data.database.largeSetPresentNum =
	    mReply.act.search.database.largeSetPresentNum;
	database.data.database.mediumSetPresentNum =
	    mReply.act.search.database.mediumSetPresentNum;
	database.data.database.replaceIndicator =
	    mReply.act.search.database.replaceIndicator;

	RESC_ASN_NO_MALLOC(&database.data.database.resultSetName);
	RESC_ASN_INIT_VS(&database.data.database.resultSetName);
	RescASN_CopyVS(IR_FILE, &database.data.database.resultSetName,
	    &mReply.act.search.database.resultSetName);

	RESC_ASN_NO_MALLOC(&database.data.database.dbNameList);
	RESC_LIST_INIT(&database.data.database.dbNameList);
	RescList_Copy(IR_FILE, &database.data.database.dbNameList,
	    &mReply.act.search.database.dbNameList);

	database.data.database.smallSetNameType =
	    mReply.act.search.database.smallSetNameType;
	RESC_ASN_NO_MALLOC( &database.data.database.smallSetList.generic);
	RESC_ASN_INIT_VS( &database.data.database.smallSetList.generic);
	RescASN_CopyVS(IR_FILE,
	    &database.data.database.smallSetList.generic,
	    &mReply.act.search.database.smallSetNames.generic);

	RESC_ASN_NO_MALLOC( &database.data.database.smallSetList.dbSpecific);
	RESC_LIST_INIT( &database.data.database.smallSetList.dbSpecific);
	RescList_Copy(IR_FILE, 
	    &database.data.database.smallSetList.dbSpecific,
	    &mReply.act.search.database.smallSetNames.dbSpecific);
	
    	database.data.database.mediumSetNameType =
	    mReply.act.search.database.mediumSetNameType;
	RESC_ASN_NO_MALLOC( &database.data.database.mediumSetList.generic);
	RESC_ASN_INIT_VS( &database.data.database.mediumSetList.generic);
	RescASN_CopyVS(IR_FILE,
	    &database.data.database.mediumSetList.generic,
	    &mReply.act.search.database.mediumSetNames.generic);

	RESC_ASN_NO_MALLOC( &database.data.database.mediumSetList.dbSpecific);
	RESC_LIST_INIT( &database.data.database.mediumSetList.dbSpecific);
	RescList_Copy(IR_FILE, 
	    &database.data.database.mediumSetList.dbSpecific,
	    &mReply.act.search.database.mediumSetNames.dbSpecific);

	RESC_ASN_NO_MALLOC(&database.data.database.recordSyntax);
	RESC_ASN_INIT_OI(&database.data.database.recordSyntax);
	RescASN_CopyOI(IR_FILE, &database.data.database.recordSyntax,
	    &mReply.act.search.database.recordSyntax);

	/* Clear the db */
	UaSc_ClearDB(&mReply.act.search.database);

	/*
	 * Get Database Info from User.
	 */
	create_res_dialog(WD_MODAL, DLG_SEARCH_DATABASE, 
	    EM_ALL, dlg_107_eh, (long)&database);

	if (database.cancelled == TRUE)
	{
		/*
		 * User Cancelled.
		 */
		return(IR_SUCCESS);
	}

CS_LOG_PRINTS(IR_FILE, "Decoding the selected database info\n");
	mReq.search.database.assocId =
	    database.data.database.assocId;
	mReq.search.database.smallSetUpperBound =
	    database.data.database.smallSetUpperBound;
	mReq.search.database.largeSetPresentNum =
	    database.data.database.largeSetPresentNum;
	mReq.search.database.mediumSetPresentNum =
	    database.data.database.mediumSetPresentNum;
	mReq.search.database.replaceIndicator =
	    database.data.database.replaceIndicator;

	RESC_ASN_NO_MALLOC(&mReq.search.database.resultSetName);
	RESC_ASN_INIT_VS(&mReq.search.database.resultSetName);
	RescASN_CopyVS(IR_FILE, &mReq.search.database.resultSetName,
	    &database.data.database.resultSetName);
	RescUT_DebugVS(IR_FILE, 
		&mReq.search.database.resultSetName);

	RESC_ASN_NO_MALLOC(&mReq.search.database.dbNameList);
	RESC_LIST_INIT(&mReq.search.database.dbNameList);
	RescList_Copy(IR_FILE, &mReq.search.database.dbNameList,
	    &database.data.database.dbNameList);

        mReq.search.database.smallSetNameType =
	    database.data.database.smallSetNameType;
	RESC_ASN_NO_MALLOC( &mReq.search.database.smallSetNames.generic);
	RESC_ASN_INIT_VS( &mReq.search.database.smallSetNames.generic);
	RescASN_CopyVS(IR_FILE,
	    &mReq.search.database.smallSetNames.generic,
	    &database.data.database.smallSetList.generic);

	RESC_ASN_NO_MALLOC( &mReq.search.database.smallSetNames.dbSpecific);
	RESC_LIST_INIT( &mReq.search.database.smallSetNames.dbSpecific);
	RescList_Copy(IR_FILE,
	    &mReq.search.database.smallSetNames.dbSpecific,
	    &database.data.database.smallSetList.dbSpecific);

	mReq.search.database.mediumSetNameType =
	    database.data.database.mediumSetNameType;
	RESC_ASN_NO_MALLOC( &mReq.search.database.mediumSetNames.generic);
	RESC_ASN_INIT_VS( &mReq.search.database.mediumSetNames.generic);
	RescASN_CopyVS(IR_FILE,
	    &mReq.search.database.mediumSetNames.generic,
	    &database.data.database.mediumSetList.generic);

	RESC_ASN_NO_MALLOC( &mReq.search.database.mediumSetNames.dbSpecific);
	RESC_LIST_INIT( &mReq.search.database.mediumSetNames.dbSpecific);
	RescList_Copy(IR_FILE,
	    &mReq.search.database.mediumSetNames.dbSpecific,
	    &database.data.database.mediumSetList.dbSpecific);

	RESC_ASN_NO_MALLOC(&mReq.search.database.recordSyntax);
	RESC_ASN_INIT_OI(&mReq.search.database.recordSyntax);
	RescASN_CopyOI(IR_FILE, &mReq.search.database.recordSyntax,
	    &database.data.database.recordSyntax);
	RescUT_DebugVS(IR_FILE, &mReq.search.database.recordSyntax);
	/*
	 * Store the active database info.
	 */

	RescUT_DebugVS(IR_FILE, &mReq.search.database.resultSetName);
	UaSc_Maintenance(UASC_MAINT_STORE_ACT_DB, &mReq, &mReply);
CS_LOG_PRINTS(IR_FILE, "Stored the database info in maintenance\n");
	UaSc_ClearDB(&mReq.search.database);
CS_LOG_PRINTS(IR_FILE, "Cleared the maintenance input strucutre\n");
	if (database.data.database.search && mReply.act.canSearch)
	{
		/* Lets send the search. */
CS_LOG_PRINTS(IR_FILE, "Sending Search\n");
		return(UaSc_SendSearch());
	}
	else if (database.data.database.search)
	{
		/* Don't have enough info to send the search ask for it. */
CS_LOG_PRINTS(IR_FILE, "Building Query\n");
		return(UaSc_BuildQuery());
	}

CS_LOG_PRINTS(IR_FILE, "Done with the database dialog\n");

	return(IR_SUCCESS);
}

IR_STATUS UaSc_ClearSearch(void)
{
    	UASC_MAINT_REQ   mReq;   /* Search Maintenance Request info. */
    	UASC_MAINT_REPLY mReply; /* Search Maintenance Reply info. */
	
	if (UaSc_Maintenance(UASC_MAINT_CLEAR_ACT,
	    &mReq,
	    &mReply) == UA_SUCCESS)
	{
	    xvt_beep();
	    UaEp_MenuSetStatus(M_SEARCH_CLEAR, FALSE);
	    UaEp_MenuSetStatus(M_SEARCH_SEND, FALSE);
	}
	else
	{
	    xvt_note("Unable to clear Active Search Space.");
	}
	return(IR_SUCCESS);
}
