/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Search Control Commnunication Event.
 *
 * DESCRIPTION:
 *      The Search Control Communication Event CSU reacts to 
 *      communication events concerning associations that have been 
 *      detected by a previous call to IRO_CommEvent.
 *
 * CONTENTS:
 *      UaSc_CommEvent
 *
 * NOTES:
 *      This CSU may only be invoked from with in the User Application
 *      Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ***************************************************************************/

#include <ir.h>
#include <resc.h>
#include <ua.h>
#include <uaep.h>
#include <debug.h>
#include <ir_oid.h>
#include <ua/uaepmsg.h>

/****************************************************************************
 * FUNCTION: UaSc_CommEvent
 *
 * PURPOSE:
 *      The User Application Search Control Communication Event
 *      function reacts to various sorts of communication events in
 *      order to inform the user and or update the menus.
 *
 * INPUT:
 *      This function requires an UASC_COMM_EVENT_DATA_TYPE, namely one
 *      of the following:
 *
 *          UASC_SEARCH_COMPLETE        - A Search has completed.
 *
 *      An IRO_COMM_EVENT_REPLY is also required and contains the
 *      information returned from a call to IRO_CommEvent.
 *
 * OUTPUT:
 *      This function returns an IR_STATUS which returns a non-zero
 *      value on error. A call to UaHec_ErrorControl is required
 *      to inform the user of the problem.
 *
 * NOTES:
 *      Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaSc_CommEvent(
    UASC_COMM_EVENT_DATA_TYPE dataType,  /* Type of communiction event. */
    IRO_COMM_EVENT_REPLY *replyData)     /* Related comm data. */
{
	IR_STATUS       returnCode;     /* Return value for function.  */
	CHAR                    *wsName;
	IR_ASSOC_ID             assocId;
	BOOLEAN                 searchStatus;
	IRO_COMM_SRCH_RESP      *sResp;
	CHAR                    tmpBuf[IR_LEN_100];
	CHAR			*noteString;
	CHAR			*s, *t;
	VISIBLE_STRING		*targetName;
	CHAR			targetNameStr[IR_LEN_100];


	returnCode = IR_SUCCESS;

	/* Malloc the message display string */
	noteString = (CHAR *)RescIR_Malloc(IR_FILE, IR_LEN_1000);

	assocId = replyData->assocId;
	sResp = &replyData->commData.searchResponse;
	searchStatus = sResp->searchStatus;

	/* Create a new workspace to correspond with the origin process */
	if (sResp->wsId != IR_UNKNOWN_ID_TYPE) 
	{
		/* Create a window and pop up any records that are there */
		UaEp_UpdateWin(sResp->wsId);
		/* Get the name from the window */
		wsName = UaEp_GetWsName(sResp->wsId);
	}
	else
	{
		/* 
		 * This name is malloced and must be freed before leaving
		 * because there is no window using this name. 
		 */
		wsName = UaEp_GenerateName(sResp->wsId);
	}

	targetName = UaHec_GetAssocName(assocId);
	if (targetName)
	{
		VSToStrNullTerm(targetNameStr, targetName, IR_LEN_100);
	}
	else
	{
		sprintf(targetNameStr, "Unknown target for association %d\n",
		    assocId);
	}
		

	/*
	 * Determine the type of Communication event.
	 */
	switch (dataType)
	{
	case IR_COMM_DATA_SEARCH_SRVC:

		if (searchStatus == TRUE)
		{
			/* Ensure menu option is available */
			if (sResp->numRecOnTarget > 0)
			{
				UaEp_MenuSetStatus(M_RESULT_DISPLAY, TRUE);
			}

			/* Check the present status and display a message */

			sprintf(noteString,"Search %s (%s) was successful.",
			    wsName, targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			
			sprintf(s, "Total records in local workspace is %ld.",
			    sResp->numRecInWs);
			while (*s != EOS)
				s++;
			sprintf(s, "Total records at remote target is %ld.",
			    sResp->numRecOnTarget);
			while (*s != EOS)
				s++;
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_PS, sResp->presentStatus,
			    &t);
			sprintf(s, "Target provided present status = %s.", t);
			UaHec_Message("Search Response", noteString,
		    	    FALSE);
		}
		else
		{
			sprintf(noteString,"Search %s (%s) failed.",
			    wsName, targetNameStr);
			s = noteString;
			while (*s != EOS)
				s++;
			/* Check the result status and diagnostic */
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_BIB1, 
			    sResp->diagnostic.condition,
			    &t);
			sprintf(s,"Target supplied diagnostic condition number is %ld. %s.",
			    sResp->diagnostic.condition, t);
			
			VSToStrNullTerm(tmpBuf, &sResp->diagnostic.addInfo, 
			    IR_LEN_100);
			strcat(s, tmpBuf);
			strcat(s, ".");
			while (*s != EOS)
				s++;
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_RSS, 
			    sResp->resultSetStatus,
			    &t);
			sprintf(s, "%s. Local work space is empty", t);

			UaHec_Message("Search Response", noteString,
		    	    FALSE);

		}

		break;
	case IR_COMM_DATA_SRCH_FAIL:
		UaHec_Message("Search Response", "Search Attempt Failed",
		    FALSE);
		break;
	
	default:
		break;
	}

	if (targetName)
	{
		RescASN_DestroyVS(IR_FILE, targetName);
	}

	if ((sResp->wsId == IR_UNKNOWN_ID_TYPE) &&
	    (wsName == (CHAR *)NULL))
	{
		/* Free wsName that is not attached to a window */
		RescIR_Free(IR_FILE, (CHAR *)wsName);
	}

	RescIR_Free(IR_FILE, (CHAR *)noteString);
	return (returnCode);
}
