/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Search Maintenance
 *
 * DESCRIPTION:
 *
 *	The Search Maintenance CSU maintains the internal Query file
 *	which can be loaded from a file, saved to a file, or received 
 *	in list form.  It also maintains currnet search informaition 
 *	which can be added, displayed or cleared.
 *
 * CONTENTS:
 *	UaSc_Maintenance
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <ua\rpnquery.h>
#include <uaep.h>

typedef struct query_hist
{
	RPN_Structure *query;
	CHAR *queryString;
	SHORT	query_id;
	struct query_hist *next;
} UASC_QUERY_HIST_STRUCT;


void UaSc_ClearDB(UASC_DATABASE_INFO *db);
static void initScDbInfoStruct(UASC_DATABASE_INFO *);

static UASC_QUERY_HIST_STRUCT *HistRoot = (UASC_QUERY_HIST_STRUCT *)NULL;

/****************************************************************************
 * FUNCTION:  UaSc_Maintenance
 *
 * PURPOSE:
 *	The User Application Search Control Maintenance Function
 *	maintains a list of user queries and an active search.
 *	The list of user queries can be loaded into memory from
 *	a file, saved to a file, or returned to a calling function.
 *	The active search maintains the definition of the active
 *	query, the target and database, and the database options. It
 *	allows you to set the query and database portions, modify
 *	database options and clear all the entries.
 *
 * INPUT:
 * 	This function requires a UASC_MAINT_TYPE and a UASC_MAINT_REQ.
 *	The UASC_MAINT_TYPE indicates the opoeration to perform the
 *	UASC_MAINT_REQ provides specific information to perform the
 *	requested operation. The list that follows describes the possible
 *	operations and the data required.
 *
 *	    UASC_MAINT_LOAD_FILE	- Load query file.
 *					  Requires a filename.
 *	    UASC_MAINT_SAVE_FILE	- Save query file.
 *					  Requires a filename.
 *	    UASC_MAINT_STORE_ACT_QRY	- Store active query.
 *					  Requires a query.
 *	    UASC_MAINT_STORE_ACT_DB	- Store active database.
 *					  Requires a target and database
 *					  or result set.
 *	    UASC_MAINT_UPDATE_ACT_OPT	- Update active options.
 *					  Requires no information.
 *	    UASC_MAINT_GET_FILE		- Get query file.
 *					  Requires no information.
 *	    UASC_MAINT_GET_ACT		- Get active search.
 *					  Requires no information.
 *	    UASC_MAINT_CLEAR_FILE	- Clear query file.
 *					  Requires no information.
 *	    UASC_MAINT_CLEAR_ACT	- Clear active search.
 *					  Requires no information.
 *
 * OUTPUT:
 *	This function returns a UASC_MAINT_REPLY the contents of which
 *	depends on the type of request which is made. The following 
 *	table lists the information on success.
 *
 *	    UASC_MAINT_LOAD_FILE	- Load query file.
 *					  Returns UA_STATUS only.
 *	    UASC_MAINT_SAVE_FILE	- Save query file.
 *					  Returns UA_STATUS only.
 *	    UASC_MAINT_STORE_ACT_QRY	- Store active query.
 *					  Returns UA_STATUS only.
 *	    UASC_MAINT_STORE_ACT_DB	- Store active database.
 *					  Returns UA_STATUS only.
 *	    UASC_MAINT_UPDATE_ACT_OPT	- Update active options.  
 *					  Returns UA_STATUS only.  
 *	    UASC_MAINT_GET_FILE		- Get query file.  
 *					  Returns UA_STATUS and 
 *					  a Query file list.
 *	    UASC_MAINT_GET_ACT		- Get active search.
 *					  Returns UA_STATUS and
 *					  the active search informaton.
 *	    UASC_MAINT_CLEAR_FILE	- Clear query file.
 *					  Returns UA_STATUS only.
 *	    UASC_MAINT_CLEAR_ACT	- Clear active search.
 *					  Returns UA_STATUS only.
 *	The UA_STATUS indicates the success or failure of the 
 *	operation. When a UA_STATUS is returned the calling function
 *	is NOT required to handle the error by notifing the user
 *	
 * NOTES:
 *	Some options are not availabe in prototype.
 *
 ***************************************************************************/

UA_STATUS UaSc_Maintenance(
    UASC_MAINT_TYPE 	reqType,	/* Type of operation to perform. */
    UASC_MAINT_REQ	*req,		/* Request information. */
    UASC_MAINT_REPLY 	*reply) 	/* Reply iinformation. */
{

	static UASC_SEARCH_TYPE	activeSearch;
	static BOOLEAN		gotQuery = FALSE;
	static BOOLEAN		gotDatabase = FALSE;
	static BOOLEAN		init = FALSE;

	UA_STATUS uaStatus;

	uaStatus = UA_SUCCESS;

	if (!init)
	{
		initScDbInfoStruct(&activeSearch.database);
		activeSearch.query = (RPN_Structure *)NULL;
		init = TRUE;
	}
	switch(reqType)
	{
	case UASC_MAINT_LOAD_FILE:
		/* Not Available in Prototype */
		break;

	case UASC_MAINT_SAVE_FILE:
		/* Not Available in Prototype */
		break;

	case UASC_MAINT_GET_FILE:
		/* Not Available in Prototype */
		break;

	case UASC_MAINT_CLEAR_FILE:
		/* Not Available in Prototype */
		break;

	case UASC_MAINT_STORE_ACT_QRY:
CS_LOG_PRINTS(IR_FILE, "Store active query\n");
		if (activeSearch.query != (RPN_Structure *)NULL)
		{
			(void)RPN_Structure_Free(activeSearch.query);
			activeSearch.query = (RPN_Structure *)NULL;
		}

		activeSearch.query = req->search.query;
		req->search.query = (RPN_Structure *)NULL;
		gotQuery = (activeSearch.query != (RPN_Structure *)NULL);
		reply->act.canSearch = (gotDatabase && gotQuery);
		break;

	case UASC_MAINT_STORE_ACT_DB:
CS_LOG_PRINTS(IR_FILE, "Store active database\n");
		UaSc_ClearDB(&activeSearch.database);
		UaSc_CopyDB(&(req->search.database), &activeSearch.database);
		RescUT_DebugVS(IR_FILE, &activeSearch.database.resultSetName);
		if (RESC_LIST_SIZE(&activeSearch.database.dbNameList) > 0)
		{
			gotDatabase = TRUE;
		}
		else
		{
			gotDatabase = FALSE;
		}
		reply->act.canSearch = (gotDatabase && gotQuery);
		break;

	case UASC_MAINT_GET_ACT:
CS_LOG_PRINTS(IR_FILE, "Getting active query and database\n");
		reply->act.search.query = (RPN_Structure *)NULL;
		initScDbInfoStruct(&reply->act.search.database);

		if (gotQuery)
		{
			reply->act.search.query = RPN_Structure_Copy(
			    activeSearch.query);
		}
		if (gotDatabase)
		{
			UaSc_CopyDB(&activeSearch.database,
			    &reply->act.search.database);
		}
		reply->act.canSearch = (gotDatabase && gotQuery);
		break;

	case UASC_MAINT_CLEAR_ACT:
CS_LOG_PRINTS(IR_FILE, "Clearing active query and database\n");
		if (gotQuery)
		{
			(void)RPN_Structure_Free(activeSearch.query);
			activeSearch.query = (RPN_Structure *)NULL;
		}
		if (gotDatabase)
		{
			UaSc_ClearDB(&activeSearch.database);
		}
		gotDatabase = FALSE;
		gotQuery = FALSE;
		reply->act.canSearch = (gotDatabase && gotQuery);
CS_LOG_PRINTS(IR_FILE, "active query and database is cleared\n");
		break;

	default:
		break;
	}
	UaEp_MenuSetStatus(M_SEARCH_CLEAR, (gotDatabase || gotQuery));
	UaEp_MenuSetStatus(M_SEARCH_SEND, (gotDatabase && gotQuery));
	return (uaStatus);
}


static void initScDbInfoStruct(UASC_DATABASE_INFO *db)
{
	memset(db, (CHAR)NULL, sizeof(UASC_DATABASE_INFO));

	RESC_ASN_NO_MALLOC(&db->resultSetName);
	RESC_ASN_INIT_VS(&db->resultSetName);
	RESC_ASN_NO_MALLOC(&db->dbNameList);
	RESC_LIST_INIT(&db->dbNameList);
	db->smallSetNameType = IR_ELEM_SET_NAME_NONE;
	RESC_ASN_INIT_VS(&db->smallSetNames.generic);
	RESC_ASN_NO_MALLOC(&db->smallSetNames.generic);
	RESC_LIST_INIT(&db->smallSetNames.dbSpecific);
	RESC_ASN_NO_MALLOC(&db->smallSetNames.dbSpecific);
	db->mediumSetNameType = IR_ELEM_SET_NAME_NONE;
	RESC_ASN_INIT_VS(&db->mediumSetNames.generic);
	RESC_ASN_NO_MALLOC(&db->mediumSetNames.generic);
	RESC_LIST_INIT(&db->mediumSetNames.dbSpecific);
	RESC_ASN_NO_MALLOC(&db->mediumSetNames.dbSpecific);

	RESC_ASN_NO_MALLOC(&db->recordSyntax);
	RESC_ASN_INIT_OI(&db->recordSyntax);

}

void UaSc_ClearDB(UASC_DATABASE_INFO *db)
{

	RescASN_DestroyVS(IR_FILE, &db->resultSetName);
	RescList_Destroy(IR_FILE, &db->dbNameList);

	db->smallSetNameType = IR_ELEM_SET_NAME_NONE;
	RescASN_DestroyVS(IR_FILE, &db->smallSetNames.generic);
	RESC_ASN_NO_MALLOC(&db->smallSetNames.generic);
	RESC_ASN_INIT_VS(&db->smallSetNames.generic);

	RescList_Destroy(IR_FILE, &db->smallSetNames.dbSpecific);
	RESC_LIST_INIT(&db->smallSetNames.dbSpecific);
	RESC_ASN_NO_MALLOC(&db->smallSetNames.dbSpecific);

	db->mediumSetNameType = IR_ELEM_SET_NAME_NONE;
	RescASN_DestroyVS(IR_FILE, &db->mediumSetNames.generic);
	RESC_ASN_NO_MALLOC(&db->mediumSetNames.generic);
	RESC_ASN_INIT_VS(&db->mediumSetNames.generic);

	RescList_Destroy(IR_FILE, &db->mediumSetNames.dbSpecific);
	RESC_LIST_INIT(&db->mediumSetNames.dbSpecific);
	RESC_ASN_NO_MALLOC(&db->mediumSetNames.dbSpecific);

	RescASN_DestroyOI(IR_FILE, &db->recordSyntax);
	RESC_ASN_INIT_OI(&db->recordSyntax);
	initScDbInfoStruct(db);

}


void UaSc_CopyDB(UASC_DATABASE_INFO *from, UASC_DATABASE_INFO *to)
{
	to->assocId = from->assocId;
	to->smallSetUpperBound = from->smallSetUpperBound;
	to->largeSetPresentNum =  from->largeSetPresentNum;
	to->mediumSetPresentNum = from->mediumSetPresentNum;
	to->replaceIndicator = from->replaceIndicator;

	to->smallSetNameType = from->smallSetNameType;
	RescASN_CopyVS(IR_FILE, &to->smallSetNames.generic,
	    &from->smallSetNames.generic);
	RescList_Copy(IR_FILE, &to->smallSetNames.dbSpecific,
	    &from->smallSetNames.dbSpecific);

	to->mediumSetNameType = from->mediumSetNameType;
	RescASN_CopyVS(IR_FILE, &to->mediumSetNames.generic,
	    &from->mediumSetNames.generic);
	RescList_Copy(IR_FILE, &to->mediumSetNames.dbSpecific,
	    &from->mediumSetNames.dbSpecific);

	RESC_ASN_NO_MALLOC(&to->resultSetName);
	RESC_ASN_INIT_VS(&to->resultSetName);
	RescASN_CopyVS(IR_FILE, &to->resultSetName, &from->resultSetName);
	RescUT_DebugVS(IR_FILE, &to->resultSetName);

	RESC_ASN_NO_MALLOC(&to->dbNameList);
	RESC_LIST_INIT(&to->dbNameList);
	RescList_Copy(IR_FILE, &to->dbNameList, &from->dbNameList);

	RESC_ASN_NO_MALLOC(&to->recordSyntax);
	RESC_ASN_INIT_OI(&to->recordSyntax);
	RescASN_CopyOI(IR_FILE, &to->recordSyntax, &from->recordSyntax);
}


void UaSc_AddDbName(IR_LIST *list, CHAR *name)
{
	VISIBLE_STRING	*newDbName;

	newDbName = RescASN_CreateVS(IR_FILE);

	RESC_ASN_IS_MALLOCED(newDbName);
	RESC_ASN_INIT_VS(newDbName);


	RescASN_AddVS(IR_FILE, newDbName, name, strlen(name));
	RescList_Append(IR_FILE, list, newDbName, RescASN_DestroyVS,
	    RescASN_CloneVS);
}

/*******************************************************************
 *
 * Query History functions.
 *
 *******************************************************************/
UASC_QUERY_HIST_STRUCT *createQueryHistStruct(RPN_Structure *query);

void UaSc_AddToQueryHist(RPN_Structure *newQuery)
{
	UASC_QUERY_HIST_STRUCT *new;
	UASC_QUERY_HIST_STRUCT *cur;
	UASC_QUERY_HIST_STRUCT *last;

	new = createQueryHistStruct(newQuery);

	cur = HistRoot;
	last = cur;
	while (cur != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		last = cur;
		cur = cur->next;
	}
	if (HistRoot == (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		HistRoot = new;
		new->query_id = 1;
		UaEp_MenuSetStatus(M_SEARCH_HISTORY, TRUE);
	}
	else
	{
		last->next = new;
		new->query_id = last->query_id + 1;
	}
}

void UaSc_DestroyQueryHist(void)
{
	UASC_QUERY_HIST_STRUCT *cur;
	UASC_QUERY_HIST_STRUCT *last;

	UaEp_MenuSetStatus(M_SEARCH_HISTORY, FALSE);
	cur = HistRoot;
	last = cur;
	while (cur != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		last = cur;
		cur = cur->next;
		RPN_Structure_Free(last->query);
		(void)RescIR_Free(IR_FILE, (char *)last->queryString);
		(void)RescIR_Free(IR_FILE, (char *)last);
	}
	HistRoot = (UASC_QUERY_HIST_STRUCT *)NULL;
}

int UaSc_DestroyHistQuery(SHORT query_id)
{
	UASC_QUERY_HIST_STRUCT *cur;
	UASC_QUERY_HIST_STRUCT *prev;
	int			rCode;

	rCode = 0;
	cur = HistRoot;
	prev = (UASC_QUERY_HIST_STRUCT *)NULL;
	while (cur != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		if (cur->query_id == query_id)
		{
			if (prev != (UASC_QUERY_HIST_STRUCT *)NULL)
			{
				prev->next = cur->next;
			}
			else
			{
				if ((HistRoot = cur->next) == 
				    (UASC_QUERY_HIST_STRUCT *)NULL)
				{
					UaEp_MenuSetStatus(
					    M_SEARCH_HISTORY, 
					    FALSE);
					rCode = 1;
				}
				
			}
			RPN_Structure_Free(cur->query);
			(void)RescIR_Free(IR_FILE, (char *)cur->queryString);
			(void)RescIR_Free(IR_FILE, (char *)cur);
			return rCode;
		}
		prev = cur;
		cur = cur->next;
	}
	return rCode;

}

void UaSc_SelectHistQuery(SHORT query_id)
{
	UASC_QUERY_HIST_STRUCT *cur;
	UASC_MAINT_REQ		request;	/* Request information. */
	UASC_MAINT_REPLY 	reply; 		/* Reply iinformation. */

	cur = HistRoot;
	while (cur != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		if (cur->query_id == query_id)
		{
			request.search.query = RPN_Structure_Copy(cur->query);
			(void)UaSc_Maintenance(
			    UASC_MAINT_STORE_ACT_QRY,
			    &request,
			    &reply);
			return;
		}
		cur = cur->next;
	}
	return;
}

IR_STATUS UaSc_GetHistoryList(SLIST hslist)
{
	UASC_QUERY_HIST_STRUCT 	*cur;
	char			tmpbuf[160];
	IR_STATUS		retCode;

	retCode = IR_FAIL;
	cur 	= HistRoot;
	while (cur != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		sprintf(tmpbuf, "%d) %-100s", cur->query_id,
		    cur->queryString);
		if (slist_add(hslist,
		    (SLIST_ELT)NULL,
		    (char *)tmpbuf,
		    (long)cur->query_id))
		{
			retCode = IR_SUCCESS;
		}
		cur = cur->next;
	}
	return(retCode);
}
	

UASC_QUERY_HIST_STRUCT *createQueryHistStruct(RPN_Structure *query)
{
	UASC_QUERY_HIST_STRUCT *new;

	new = (UASC_QUERY_HIST_STRUCT *)RescIR_Malloc(IR_FILE,
	    sizeof(UASC_QUERY_HIST_STRUCT));
	if (new != (UASC_QUERY_HIST_STRUCT *)NULL)
	{
		new->query = RPN_Structure_Copy(query);
		new->queryString = RPN_Structure_Print(query);
		new->next = (UASC_QUERY_HIST_STRUCT *)NULL;
	}
	return (new);
}

